/**
 * 设备调用接口（公共）
 */
Fw.device.Device = function(){
        
    //
    return {
        /**
         * 更新原生客户端视图设置（标题栏、导航按钮等）
         */
        applyView: Fw.emptyFn,
        /**
         * 注销登录
         */
        logout : function(msg) {
            Fw.Msg.info(msg, function(){
                Fw.device.Device.gotoLogin('', true);
            });
        },
        /**
         * 返回标题栏、导航按钮等配置信息
         * _private
         */
        getViewConfig: function(app, returnObject){
            //配置信息
            //title 标题栏标题
            //leftButton 左边按钮
            //rightButton 右边按钮
            var cfg = app.viewConfig || app || {};
            
            //解析左边按钮配置
            cfg.leftButton = this.parseLeftButtonConfig(cfg.leftButton, app);
            if(cfg.leftButton === null){
                delete cfg.leftButton;
            }
            //解析右边按钮配置
            cfg.rightButton = this.parseRightButtonConfig(cfg.rightButton, app);
            if(cfg.rightButton === null){
                delete cfg.rightButton;
            }
            
            //
            if(returnObject){
                return cfg;
            }
            else{
                if(cfg.leftButton){
                    this.doLeftButtonHandler = cfg.leftButton.handler;
                    cfg.leftButton.handler = 'Fw.device.Device.doLeftButtonHandler';
                }
                if(cfg.rightButton){
                    this.doRightButtonHandler = cfg.rightButton.handler;
                    cfg.rightButton.handler = 'Fw.device.Device.doRightButtonHandler';
                }
                return Fw.encode(cfg);
            }
        },
        /**
         * 解析左边按钮配置
         * _private
         */
        parseLeftButtonConfig: function(btnCfg, app){
            var defaultHandler = app.leftButtonHandler;
            //如果为false，则左边按钮不显示
            if(btnCfg === false){
                return null;
            }
            //未定义
            else if(!Fw.isDefined(btnCfg)){
                btnCfg = {};
            }
            //直接定义按钮标题
            else if(!Fw.isObject(btnCfg)){
                btnCfg = {
                    text: String(btnCfg)
                };
            }
            
            //未设置按钮标题，则默认为'返回'
            if(Fw.isEmpty(btnCfg.title)){
                btnCfg.text = '返回';
            }
            
            //未设置按钮点击事件，则默认为defaultHandler
            if(!Fw.isFunction(btnCfg.handler)){
                if(Fw.isFunction(defaultHandler)){
                    btnCfg.handler = Fw.bind(defaultHandler, app);
                    // btnCfg.handler = defaultHandler;
                }
                else{
//            		if(document.referrer.indexOf(".html") > 0){
            			btnCfg.handler = Fw.bind(history.back, history);
//            		}else{
//            			btnCfg.handler = Fw.device.Device.gotoIndex;
//            		}
                    // btnCfg.handler = function(){
                        // var level = Fw.util.Storage.get('_HISTORY_LEVEL');
                        // if(level > 0){
                            // history.back();
                        // }
                        // else{
                            // //Fw.util.Storage.remove('_HISTORY_LEVEL');
                            // alert('...');
                        // }
                    // };
                }
            }
            else{
                Fw.bind(btnCfg.handler, app);
            }
            
            //
            // btnCfg.handler = function(){
                // //左边按钮，默认递减跳转层次
                // var level = Fw.util.Storage.get('_HISTORY_LEVEL');
                // if(level){
                    // Fw.util.Storage.set('_HISTORY_LEVEL', --level);
                // }
                // //
                // btnCfg.handler.apply(app, arguments);
            // };
            
            //
            return btnCfg;
        },
        /**
         * 解析右边按钮配置
         * _private
         */
        parseRightButtonConfig: function(btnCfg, app){
            var defaultHandler = app.rightButtonHandler;
            //未定义
            if(!Fw.isDefined(btnCfg) || Fw.isBoolean(btnCfg)){
                return null;
            }
            //直接定义按钮标题
            else if(!Fw.isObject(btnCfg)){
                btnCfg = {
                    text: String(btnCfg)
                };
            }
            
            //未设置按钮点击事件，则默认为defaultHandler
            if(!Fw.isFunction(btnCfg.handler)){
                if(Fw.isFunction(defaultHandler)){
                    btnCfg.handler = Fw.bind(defaultHandler, app);
                }
                else{
                    btnCfg.handler = Fw.emptyFn;
                }
            }
            else{
                Fw.bind(btnCfg.handler, app);
            }
            
            //
            return btnCfg;
        }
    };
}();

//
Fw.Logger.debug('加载设备调用接口（公共）');
