/**
 * 安全组件（iPhone实现）
 */
Fw.define(function(require){
    'require:nomunge,exports:nomunge,module:nomunge';
    
    //安全组件（公共）
    require('{fwPath}/widget/Security');
    
    //重写安全组件
    Fw.apply(Fw.Security, {
        /**
         * MD5
         */
        MD5: {
            /**
             * 摘要
             * @param {String} data
             */
            digest: function(data){
                Fw.Logger.error('安全组件，MD5摘要接口暂未实现');
            }
        },
        /**
         * AES
         */
        AES: {
            /**
             * 加密
             * @param {String} data
             * @param {String} key
             * @param {String} iv
             * @param {String} mode
             * @param {String} pad
             */
            encrypt: function(data, key, iv, mode, pad){
                Fw.Logger.error('安全组件，AES加密接口暂未实现');
            },
            /**
             * 解密
             * @param {String} data
             * @param {String} key
             * @param {String} callback
             */
            decrypt: function(data, key, callback){
                try{
                    var fnName = 'callback-decrypt-' + Fw.id();
                    Fw.Security[fnName] = Fw.bind(function(plainText){
                        plainText = plainText || 'decrypt error...';
                        //构造回调参数（默认最后一个参数为密文）
                        if(Fw.isArray(this.args)){
                            this.args.push(plainText);
                        }
                        else{
                            this.args = [plainText];
                        }
                        //回调
                        Fw.call(this.fn, this.scope, this.args);
                        //销毁临时方法
                        delete Fw.Security[fnName];
                    }, callback);
                    //
                    Fw.device.Device.addEvent('12', {
                        text: data,
                        key: key,
                        callback: 'Fw.Security["' + fnName + '"]'
                    });
                    return null;
                }
                catch(e){
                    Fw.Logger.error('安全组件，AES解密错误', e);
                }
            }
        },
        /**
         * RSA
         */
        RSA: {
            /**
             * 加密
             * @param {String} data
             * @param {String} key
             */
            encrypt: function(data, key){
                Fw.Logger.error('安全组件，RSA加密接口暂未实现');
            },
            /**
             * 解密
             * @param {String} data
             * @param {String} key
             */
            decrypt: function(data, key){
                Fw.Logger.error('安全组件，RSA解密接口暂未实现');
            }
        },
        /**
         * 三段式
         */
        TripleDES: {
            /**
             * 加密
             * @param {String} data
             * @param {String} key
             * @param {String} callback
             */
            encrypt: function(data, key, callback){
                try{
                    var fnName = 'callback-encrypt-' + Fw.id();
                    Fw.Security[fnName] = Fw.bind(function(cipherText){
                        cipherText = cipherText || 'encrypt error...';
                        //构造回调参数（默认最后一个参数为密文）
                        if(Fw.isArray(this.args)){
                            this.args.push(cipherText);
                        }
                        else{
                            this.args = [cipherText];
                        }
                        //回调
                        Fw.call(this.fn, this.scope, this.args);
                        //销毁临时方法
                        delete Fw.Security[fnName];
                    }, callback);
                    //
                    Fw.device.Device.addEvent('11', {
                        text: data,
                        callback: 'Fw.Security["' + fnName + '"]'
                    });
                    return null;
                }
                catch(e){
                    Fw.Logger.error('安全组件，三段式加密错误', e);
                }
            },
            /**
             * 解密
             * @param {String} data
             * @param {String} key
             */
            decrypt: function(data, key){
                Fw.Logger.error('安全组件，三段式解密接口暂未实现');
            }
        }
    });

    //
    Fw.Logger.debug('加载安全组件（iPhone实现）');
});
