/**
 * 弹出框（Web实现）
 */
Fw.define(function(require){
	'require:nomunge,exports:nomunge,module:nomunge';
    //弹出框（公共）
    require('{fwPath}/widget/MessageBox');
    
    //重写弹出框
    var MessageBox = Fw.extend(Fw.MessageBox, {
        /**
         * 显示弹出框
         * 桥接FrozenUI Dialog
         */
        show: function(title, content, buttons, fn, scope){
            //接口封装配置
            var options = {
                title: title,
                content: content
            };
            
            //
            if(Fw.isObject(content)){
                //content中的弹出框配置
                Fw.apply(options, content);
                //content中的自定义按钮
                if(Fw.isArray(content.buttons) && content.buttons.length > 0){
                    buttons = content.buttons;
                }
            }

            //解析按钮
            var btnCfg = this.parseButtons(buttons);
            options.button = btnCfg.text;
            
            //为每个按钮定义了回调函数
            if(btnCfg.handler.length === btnCfg.text.length){
                options.callback = function(type, index, btn){
                    var code = btnCfg.code[options.closable ? index - 1 : index];
                    if(code){
                        Fw.call(btnCfg.handler[index], btnCfg.scope[index] || this, [code, btn, this]);
                    }
                    else if(Fw.isFunction(options.closeHandler)){
                        Fw.call(options.closeHandler, this, [btn, this]);
                    }
                };
            }
            //使用同一个回调函数
            else{
                if(Fw.isFunction(fn)){
                    options.callback = function(type, index, btn){
                        var code = btnCfg.code[options.closable ? index - 1 : index];
                        if(code){
                            Fw.call(fn, scope || this, [code, btn, this]);
                        }
                        else if(Fw.isFunction(options.closeHandler)){
                            Fw.call(options.closeHandler, this, [btn, this]);
                        }
                    };
                }
                else{
                    options.callback = Fw.emptyFn;
                }
            }
                
            //
            $.dialog(options);
        },
        /**
         * 解析按钮
         * _private
         */
        parseButtons: function(buttons){
            var config = {
                code: [],
                text: [],
                handler: [],
                scope: []
            };
            Fw.each(buttons, function(button, index){
                config.code.push(button['code'] || Fw.id());
                config.text.push(button['text'] || ('按钮' + index));
                if(Fw.isFunction(button['handler'])){
                    config.handler.push(button['handler']);
                    config.scope.push(button['scope'] || null);
                }
            });
            return config;
        }
    });
    
    //
    Fw.Msg = new MessageBox();

    //
    Fw.Logger.debug('加载弹出框组件（Web实现）');
});

/**
 * FrozenUI Dialog
 */
!function($){

    // 默认模板
    var _dialogTpl='<div class="ui-dialog">'+
        '<div class="ui-dialog-cnt">'+
            '<% if (headerVisible && title) { %>'+
                '<header class="ui-dialog-hd ui-border-b">'+
                    '<h3><%=title%></h3>'+
                    '<% if (closable) { %>' +
                        '<i class="ui-dialog-close" data-role="button"></i>'+
                    '<% } %>' +
                '</header>' +
            '<% } %>'+
            '<div class="ui-dialog-bd">'+
                //'<div>'+
                // '<h4><%=title%></h4>'+
                    '<%=content%>'+
                //'</div>'+
            '</div>'+
            '<div class="ui-dialog-ft ui-btn-group">'+
                '<% for (var i = 0; i < button.length; i++) { %>' +
                '<% if (i == select) { %>' +
                '<button type="button" data-role="button"  class="select" id="dialogButton<%=i%>"><%=button[i]%></button>' +
                '<% } else { %>' +
                '<button type="button" data-role="button" id="dialogButton<%=i%>"><%=button[i]%></button>' +
                '<% } %>' +
                '<% } %>' +
            '</div>'+
        '</div>'+        
    '</div>';
    // 默认参数
    var defaults={
        title:'',
        content:'',
        button:['确认'],
        select:0,
        headerVisible: true, //davi+
        closable: false,//davi+
        allowScroll:false,
        callback:function(){}
    };
    // 构造函数
    var Dialog  = function (el,option,isFromTpl) {

        this.option=$.extend(defaults,option);
        this.element=$(el);
        this._isFromTpl=isFromTpl;
        this.button=$(el).find('[data-role="button"]');
        this._bindEvent();
        this.toggle();
    };
    Dialog.prototype={
        _bindEvent:function(){
            var self=this;
            self.button.on("tap",function(){
                var el = $(self.button);//davi+
                var index=el.index($(this));
                self.option.callback("button",index, el);//davi*
                var e=$.Event("dialog:action");
                e.index=index;
                self.element.trigger(e);
                self.hide.apply(self);
            });
        },
        toggle:function(){
            if(this.element.hasClass("show")){
                this.hide();
            }else{
                this.show();
            }
        },
        show:function(){
            var self=this;
            // self.option.callback("show");
            self.element.trigger($.Event("dialog:show"));
            self.element.addClass("show");
            this.option.allowScroll && self.element.on("touchmove" , _stopScroll);
        },
        hide :function () {
            var self=this;
            // self.option.callback("hide");
            self.element.trigger($.Event("dialog:hide"));
            self.element.off("touchmove" , _stopScroll);
            self.element.removeClass("show");
            
            self._isFromTpl&&self.element.remove();
        }
    };
    // 禁止冒泡
    function _stopScroll(){
        return false;
    }
    function Plugin(option) {
        return $.adaptObject(this, defaults, option,_dialogTpl,Dialog,"dialog");
    }
    $.fn.dialog=$.dialog= Plugin;
}(window.Zepto);