/**
 * 安全组件（Web实现）
 */
Fw.define(function(require){
	'require:nomunge,exports:nomunge,module:nomunge';
    //安全组件（公共）
    require('{fwPath}/widget/Security');
    
    //CORE
    require('{fwPath}/device/web/security/core');
    //MD5
    require('{fwPath}/device/web/security/md5');
    //AES
    require('{fwPath}/device/web/security/aes');
    //RSA
    require('{fwPath}/device/web/security/rsa');
    //SM（国密）
    require('{fwPath}/device/web/security/sm');
    
    //设置相关密钥（测试用）
    Fw.apply(Fw.Security, {
        //AES密钥
        DEFAULT_AES_KEY: 'YTPSBCMBANKAPPYC',
        //AES偏移量
        DEFAULT_AES_IV: 'YTPSBCMBANKAPPYC',
        //RSA公钥
        DEFAULT_RSA_PUBLIC_KEY: '',
        //RSA私钥
        DEFAULT_RSA_PRIVATE_KEY: '',
        //三段式公钥
        DEFAULT_TRIPLE_PUBLIC_KEY: 'MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCoSFKGuHeSP/hKMzg5Mwfm6PGTj3BJozTiYHkqzgOhXAkiT0HI8DNFKJ3R6vFrpA+j4/uQ1VYfEPKqOpjpZt45cPyk7Z8xDDH15Y3xehQ4WbcNs1X6/JjtHJz+aCsyVV8fch0WQTL9xa6lgGMqwfW/bvuiC0ycAdKv82fKl6fyEwIDAQAB',
        //三段式私钥
        DEFAULT_TRIPLE_PRIVATE_KEY: '',
        //三段式偏移量
        DEFAULT_TRIPLE_IV: 'YTPSBCMBANKAPPYC'
    });
    
    //重写安全组件
    Fw.apply(Fw.Security, {
        /**
         * MD5
         */
        MD5: {
            /**
             * 摘要
             * @param {String} data
             */
            digest: function(data){
                try{
                    return CryptoJS.MD5(data);
                }
                catch(e){
                    Fw.Logger.error('安全组件，MD5摘要错误', e);
                }
            }
        },
        /**
         * AES
         */
        AES: {
            /**
             * 加密
             * @param {String} data
             * @param {String} key
             * @param {String} iv
             * @param {String} mode
             * @param {String} pad
             */
            encrypt: function(data, key, iv, mode, pad){
                try{
                    //
                    key = CryptoJS.enc.Utf8.parse(key || Fw.Security.getRemoteProperty('AES_KEY'));
                    iv = CryptoJS.enc.Utf8.parse(iv || Fw.Security.getRemoteProperty('AES_IV')) || key;
                    mode = mode || CryptoJS.mode.CBC;
                    pad = pad || CryptoJS.pad.Pkcs7;
                    
                    //
                    data = CryptoJS.AES.encrypt(data, key, {
                        iv: iv,
                        mode: mode,
                        padding: pad
                    });
                    
                    //
                    return data.ciphertext.toString(CryptoJS.enc.Base64);
                }
                catch(e){
                    Fw.Logger.error('安全组件，AES加密错误', e);
                }
            },
            /**
             * 解密
             * @param {String} data
             * @param {String} key
             * @param {String} iv
             * @param {String} mode
             * @param {String} pad
             */
            decrypt: function(data, key, iv, mode, pad){
                if(Fw.isEmpty(data)){
                    return data;
                }
                //
                try{
                    //
                    key = CryptoJS.enc.Utf8.parse(key || Fw.Security.getRemoteProperty('AES_KEY'));
                    iv = Fw.isObject(iv) ? null : iv;
                    iv = CryptoJS.enc.Utf8.parse(iv || Fw.Security.getRemoteProperty('AES_IV')) || key;
                    mode = mode || CryptoJS.mode.CBC;
                    pad = pad || CryptoJS.pad.Pkcs7;
                    
                    //
                    data = CryptoJS.AES.decrypt({
                        ciphertext: CryptoJS.enc.Base64.parse(data)
                    }, key, {
                        iv: iv,
                        mode: mode,
                        padding: pad
                    });
                    //
                    return data.toString(CryptoJS.enc.Utf8);
                }
                catch(e){
                    Fw.Logger.error('安全组件，AES解密错误', e);
                }
            }
        },
        /**
         * RSA
         */
        RSA: {
            /**
             * 加密
             * @param {String} data
             * @param {String} key
             */
            encrypt: function(data, key){
                try{
                    var pubkey = pem_import(key || Fw.Security.getRemoteProperty('RSA_PUBLIC_KEY'));
                    data = pubkey.encrypt(data);
                    return Base64.hex2b64(data);
                }
                catch(e){
                    Fw.Logger.error('安全组件，RSA加密错误', e);
                }
            },
            /**
             * 解密
             * @param {String} data
             * @param {String} key
             */
            decrypt: function(data, key){
                Fw.Logger.error('安全组件，RSA解密接口暂未实现');
            }
        },
        /**
         * 三段式
         */
        TripleDES: {
            /**
             * 加密
             * @param {String} data
             * @param {String} key
             * @param {String} returnObject
             */
            encrypt: function(data, key, returnObject){
                try{
                    if(key === true){
                        returnObject = true;
                        key = null;
                    }
                    var aesKey = Fw.uuid(16);
                    var md5Digest = Fw.Security.MD5.digest(aesKey + data);
                    var rsaText = Fw.Security.RSA.encrypt(aesKey, key || Fw.Security.getRemoteProperty('TRIPLE_PUBLIC_KEY'));
                    var aesText = Fw.Security.AES.encrypt(data, aesKey, Fw.Security.getRemoteProperty('TRIPLE_IV'));
                    var result = [md5Digest, aesText, rsaText];
                    var cipherText = result.join(String.fromCharCode(29));
                    if(returnObject){
                        return {
                            text: cipherText,
                            key: aesKey
                        };
                    }
                    else{
                        return cipherText;
                    }
                }
                catch(e){
                    Fw.Logger.error('安全组件，三段式加密错误', e);
                }
            },
            /**
             * 解密
             * @param {String} data
             * @param {String} key
             */
            decrypt: function(data, key){
                Fw.Logger.error('安全组件，三段式解密接口暂未实现');
            }
        },
        /**
         * 从服务端获取相关属性
         */
        getRemoteProperty: function(key){
            if(Fw.isDefined(Fw.Security['DEFAULT_' + key])){
                return Fw.Security['DEFAULT_' + key];
            }
            else{
                Fw.Logger.error('安全组件，从服务端获取相关属性暂未实现');
                // Fw.util.Ajax(key, {
                    // service: '.../...',//需在服务报文配置中设置允许明文（allowPlain=true）
                    // async: false,
                    // success: function(data, result) {
                        // Fw.Security['DEFAULT_' + key] = data;
                    // }
                // });
            }
        }
    });

    //
    Fw.Logger.debug('加载安全组件（Web实现）');
});
