/**
 * Ajax对象
 */
Fw.Ajax = function(config){
	Fw.apply(this, config);
    if(this.page === false){
        this.pageSize = 0;
    }
	//call parent
	Fw.Ajax.superclass.constructor.call(this);
    //初始化事件
    this.initEvents();
	return this;
};
Fw.extend(Fw.Ajax, Fw.util.Observable, {
    /**
     * 每页条数 
     */
    pageSize: 10,
    /**
     * 显示页数 
     */
    pageLimit: 5,
    /**
     * 当前页 
     */
    currentPage: 1,
    currentPages: 1,
    /**
     * ajax默认配置
     */
	timeout: 30000,
    requestMethod: 'POST',
    dataType: 'json',
    async: true,
    /**
     * 初始化事件
     */
    initEvents: function(){
        //请求前
        this.on('beforerequest', this.onBeforeRequest, this);
        //请求成功
        this.on('success', this.onSuccess, this);
        //请求错误
        this.on('error', this.onFailure, this);
    },
    /**
     * 查询数据
     */
    query: function(page){
        this.setPage(page);
        var operation = Fw.Ajax.getRequestOperation.call(this, Fw.Ajax.QUERY_METHOD);
        Fw.Ajax.request.call(this, operation);
    },
    /**
     * 数据获取
     * @param {Array} arguments
     */
    get: function(){
        Fw.Ajax.parseOperationArguments(arguments, this, Fw.Ajax.GET_METHOD);
    },
    /**
     * 新增数据
     * @param {Array} arguments
     */
    insert: function(){
        Fw.Ajax.parseOperationArguments(arguments, this, Fw.Ajax.INSERT_METHOD);
    },
    /**
     * 删除数据
     * @param {Array} arguments
     */
    del: function(){
        Fw.Ajax.parseOperationArguments(arguments, this, Fw.Ajax.DELETE_METHOD);
    },
    /**
     * 更新数据
     * @param {Array} arguments
     */
    update: function(){
        Fw.Ajax.parseOperationArguments(arguments, this, Fw.Ajax.UPDATE_METHOD);
    },
    /**
     * 执行数据提交
     * @param {Array} arguments
     */
    execute: function(){
        Fw.Ajax.parseOperationArguments(arguments, this, Fw.Ajax.EXECUTE_METHOD);
    },
    /**
     * 请求前
     */
    onBeforeRequest: function(ajax, operation){
        this.operation = operation;
        if(this.action){
            return this.fireEvent('before' + this.action, this, operation.params, operation.url);
        }
    },
    /**
     * 请求成功
     */
    onSuccess: function(ajax, data, result, xhr){
        this.action && this.fireEvent(this.action, this, data, result, xhr);
        delete this.operation;
    },
    /**
     * 请求错误
     */
    onFailure: function(ajax, status, textStatus, error, xhr){
        this.action && this.fireEvent(this.action + 'error', this, status, textStatus, error, xhr);
        delete this.operation;
    },
    /**
     * 设置分页 
     */
    setPage: function(page){
        if(Fw.isNumber(page)){
            page = page || 1;
            this.currentPage = page;
            this.options = {};
            this.options[Fw.Ajax.allowCountProperty] = !this.simplePage;
            this.options[Fw.Ajax.startParam] = (page - 1) * this.pageSize;
            this.options[Fw.Ajax.limitParam] = this.pageSize;
        }
    },
    /**
     * 设置请求参数
     * @param {} key
     * @param {} value
     * @param {} append
     */
    setParams: function(key, value, append){
        if(Fw.isObject(key)){
            this.params = key;
        }
        else {
            if(!this.params || append === false){
                this.params = {};
            }
            this.params[key] = value;
        }
        //
        if(this.operation){
            if(this.operation.isStructMessage){
                if(!this.operation.params.payload || append === false){
                    this.operation.params.payload = this.params;
                }
                else{
                    Fw.apply(this.operation.params.payload, this.params);
                }
            }
            else{
                if(append === false){
                    this.operation.params = this.params;
                }
                else{
                    Fw.apply(this.operation.params, this.params);
                }
            }
        }
    },
    /**
     * 返回请求参数
     * @param {} key
     */
    getParams: function(key){
        var params = null;
        if(this.operation){
            if(this.operation.isStructMessage){
                params = this.operation.params.payload;
            }
            else{
                params = this.operation.params;
            }
        }
        else{
            params = this.params;
        }
        //
        if(params){
            if(key){
                return params[key];
            }
            else{
                return params;
            }
        }
        return null;
    },
    /**
     * 清除请求参数
     * @param {} key
     */
    clearParams: function(key){
        if(this.operation){
            if(this.operation.isStructMessage){
                delete this.operation.params.payload;
            }
            else{
                delete this.operation.params;
            }
        }
        else{
            if(this.params){
                if(key){
                    delete this.params[key];
                }
                else{
                    delete this.params;
                }
            }
        }
    },
    /**
     * 设置排序方式
     * @param {} key
     * @param {} value
     * @param {} append
     */
    setSorts: function(key, value, append){
        if(Fw.isArray(key)){
            this.sort = key;
        }
        else {
            if(!this.sort || append === false){
                this.sort = [];
            }
            this.sort.push({
                property: key,
                direction: value || 'asc'
            });
        }
    },
    /**
     * 返回排序方式
     */
    getSorts: function(){
        return this.sort || null;
    },
    /**
     * 清除排序方式
     */
    clearSorts: function(){
        delete this.sort;
    }
});

/**
 * 常量和静态方法
 */
Fw.apply(Fw.Ajax, {
    /**
     * 请求方法 
     */
    GET_METHOD: 'GET',
    QUERY_METHOD: 'QUERY',
    INSERT_METHOD: 'INSERT',
    DELETE_METHOD: 'DELETE',
    UPDATE_METHOD: 'UPDATE',
    EXECUTE_METHOD: 'EXECUTE',
    /**
     * HTTP请求通道
     */
    HTTP_CHANNEL: 'channel/http.do',
    /**
     * HTTP适配器SPI 
     */
    HTTP_ADAPTER_SPI: 'SPI/ADAPTER-HTTP',
    /**
     * SOCKET适配器SPI 
     */
    SOCKET_ADAPTER_SPI: 'SPI/ADAPTER-SOCKET',
    /**
     * 数据ROOT
     */
    rootProperty: 'result',
    /**
     * 记录条数
     */
    totalProperty: 'total',
    /**
     * 响应成功 
     */
    successProperty: 'success',
    /**
     * 成功消息 
     */
    messageProperty: 'message',
    /**
     * 错误消息 
     */
    errorMessageProperty: 'error',
    /**
     * 是否允许计算记录总数
     */
    allowCountProperty: 'allowCount',
    /**
     * 分页起始行
     */
    startParam: 'start',
    /**
     * 分页行数
     */
    limitParam: 'limit',
    /**
     * 请求
     */
    request: function(operation){
        if(this.fireEvent('beforerequest', this, operation) !== false){
            var url = operation.url;
            var data = operation.params;
            if(!Fw.isURL(url)){
                url = Fw.getBasePath() + url;
            }
            if(Fw.isObject(data) || Fw.isArray(data)){
                data = Fw.encode(data);
                if(data === '{}') data = null;
            }
            // url += (url.indexOf('?') > 0 ? '&' : '?') + 't=' + new Date().getTime();
            var timestamp = (new Date()).getTime();
            if($.ajax.lastRequest && this.action !== Fw.Ajax.QUERY_METHOD && this.action !== Fw.Ajax.GET_METHOD){
                if(timestamp - $.ajax.lastRequest['timestamp'] < 3000 && data){
                	var session = Fw.decode(Fw.util.Storage.get('_CURRENT_SESSION'));
                	if(Fw.decode(data).payload.hasOwnProperty("pwdCode")
                			&& Fw.decode($.ajax.lastRequest['data']).payload.hasOwnProperty("pwdCode")
                			&& !Fw.isEmpty(session) 
                			&& !Fw.isEmpty(session["CUST_NO"])
                			&& !Fw.isEmpty(session["SMSHFlag"]) 
                			&& typeof session["SMSHFlag"] == "string"
                			&& session["SMSHFlag"].indexOf("H")>-1){                		
                		// E盾客户  短信验证交易
                		var last={};
                		Fw.apply(last,Fw.decode($.ajax.lastRequest['data']));
                		last.payload.pwdCode = "";
                		var now={};
                		Fw.apply(now,Fw.decode(data));
                		now.payload.pwdCode = "";
                		if(Fw.encode(last) == Fw.encode(now)){
                			return;
                		}
                	}
                	else if($.ajax.lastRequest['data'] === data){
                		//防止重复提交（简单处理）
                        return;
                	}
                }
                else{
                    $.ajax.lastRequest['data'] = data;
                    $.ajax.lastRequest['timestamp'] = timestamp;
                }
            }
            else{
                $.ajax.lastRequest = {
                    data: data,
                    timestamp: timestamp
                };
            }
            //
            Fw.Ajax.doRequest.call(this, {
                url: url,
                data: data ? data.replace('${_t}', timestamp) : data///
            });
        }
    },
    /**
     * 执行请求
     * @param {} options
     * @param {} cipherText 用于兼容IOS回调事件（默认最后一个参数为密文）
     */
    doRequest: function(options, cipherText){
        if(!cipherText){
            //自动显示等待层
            if(this.loadMask === false){
            	//如果不显示等待层，则不加入队列
            	this.ignoreQueue = true;
            }
            else{
            	Fw.LoadMask && Fw.LoadMask.show(this.loadMask);
            }
        }
        
        //
        if(options.data || cipherText){
            //由原生客户端发起通讯时加密请求报文
            if($.ajax.isNative === true){
                options.scope = this;
            }
            //调用客户端加密接口加密请求报文
            else{
                if(cipherText || !Fw.isEmpty(Fw.Ajax.encryptModel)){
                    if(!cipherText){
                        cipherText = Fw.Security[Fw.Ajax.encryptModel].encrypt(options.data, true, {
                            fn: Fw.Ajax.doRequest,//指定回调方法（仅IOS有效）
                            scope: this,
                            args: [options]
                        });
                        //兼容IOS回调事件（IOS为异步回调，没有返回值）
                        if(!cipherText){
                            return;
                        }
                    }
                    options.data = cipherText.text;
                    this['_KEY'] = cipherText.key;
                    if(Fw.Ajax.isJSONP()){
                        options.data = encodeURIComponent(options.data);
                    }
                    else{
                        options.data = encodeURI(options.data);
                    }
                }
                //不加密
                else{
                    options.data = encodeURIComponent(options.data);
                }
            }
        }
        else{
            options.data = null;
        }
            
        //默认配置
        Fw.applyIf(options, {
            type: this.requestMethod,
            dataType: this.dataType,
            async: this.async,
            timeout: this.timeout,
            contentType: 'application/json',
            success: Fw.bind(Fw.Ajax.onRequestSuccess, this),
            error: Fw.bind(Fw.Ajax.onRequestFailure, this)
        });
        
        //是否JSONP
        if(Fw.Ajax.isJSONP()){
            Fw.apply(options, {
                type: 'GET',
                dataType: 'jsonp',
                jsonp: 'JSONPCallback',
                data: options.data ? 'data=' + options.data : null
            });
        }

        //执行请求
        Fw.Ajax.Queue.add.call(this, $.ajax(options));
    },
    /**
     * 请求成功后
     * @param {} result
     * @param {} status
     * @param {} xhr
     * @param {} plainText 兼容IOS回调事件（默认最后一个参数为明文）
     */
    onRequestSuccess: function(result, status, xhr, plainText){
        //解密
        if(plainText){
            //兼容IOS回调事件
            result = plainText;
        }
        else{
            if(!Fw.isObject(result) && !Fw.isEmpty(Fw.Ajax.decryptModel)){
                result = Fw.Security[Fw.Ajax.decryptModel].decrypt(result, this['_KEY'], {
                    fn: Fw.Ajax.onRequestSuccess,
                    scope: this,
                    args: [result, status, xhr]
                });
                delete this['_KEY'];
                
                //兼容IOS回调事件（IOS为异步回调，没有返回值）
                if(!result && $.ajax.isNative === true){
                    return;
                }
                
                //
                result = Fw.decode(result);
            }
        }
        
        //
        try {
            var data;
            if (result) {
                
                //sm 显示code码
                if(Fw.Ajax.viewErrorCode){
                    result[Fw.Ajax.messageProperty] = '[CODE：'+(result[Fw.Ajax.errorMessageProperty] || '') +']' + result[Fw.Ajax.messageProperty];
                }
                //sm 显示code码
                
                if (result[Fw.Ajax.successProperty] !== true) {
                    xhr.status = result[Fw.Ajax.errorMessageProperty] ? result[Fw.Ajax.errorMessageProperty] : -1;
                    Fw.Ajax.onRequestFailure.call(this, xhr, null, result[Fw.Ajax.messageProperty]);
                    return;
                }
                if(this.rootProperty){
                    data = result;
                    Fw.each(this.rootProperty.split('.'), function(property){
                        data = data[property];
                    });
                }
                else{
                    data = result[Fw.Ajax.rootProperty];
                }
                var total = result[Fw.Ajax.totalProperty];
                this.totalCount = Fw.isDefined(total) ? total : (Fw.isArray(data) ? data.length : Fw.isObject(data) ? 1 : 0);
            }
            else{
                data = null;
                result = null;
                this.totalCount = 0;
            }
            //
            this.fireEvent('success', this, data, result, xhr);
        }
        catch (e) {
            throw e;
        }
        
        //移出队列
        Fw.Ajax.Queue.remove(xhr);

        //自动隐藏等待层
        Fw.Ajax.Queue.isEmpty() && this.loadMask !== false && Fw.LoadMask && Fw.LoadMask.hide(200);
    },
    /**
     * 请求失败
     * @param {} xhr
     * @param {} textStatus
     * @param {} error
     */
    onRequestFailure: function(xhr, textStatus, error){
        //移出队列
        Fw.Ajax.Queue.remove(xhr);
        
        //自动隐藏等待层
        Fw.LoadMask && Fw.LoadMask.hide();
        
        //响应状态
        var status = xhr.status;
        
        //
        switch(status) {
            //系统异常
            case 500://'error'
                Fw.Msg.error(Fw.Ajax.error[status]);
                break;
            //在其他设备登录
            case 403:
            //信息已变更，请重新登录
            case 405:
            //客户端IP发生变化
            case 417:
            //登录超时，拒绝访问
            case 401://'Unauthorized'
                Fw.device.Device.logout(Fw.Ajax.error[status]);
                break;
            //服务器挂了或者取消请求
            case 0://'communication failure'
                if($.ajax.isNative === true){
                    Fw.Msg.error('[' + status + ']' + Fw.Ajax.error[status]);
                }
                else{
                    Fw.Ajax.doRequest.call({
                        loadMask: false,
                        ignoreQueue: true
                    }, {
                        url: Fw.Ajax.getRequestURL(),
                        type: 'POST',
                        async: Fw.Ajax.isJSONP(),
                        timeout: 5000,
                        success: Fw.emptyFn,
                        error: Fw.bind(function(xhr, textStatus, error){
                            if(xhr.status === 0){
                                Fw.Msg.error(Fw.Ajax.error[status]);
                            }
                            else{
                                Fw.Ajax.onRequestFailure.call(this, xhr, textStatus, error);
                            }
                        }, this)
                    });
                }
                break;
            //其他，抛异常
            default:
                var allowErrorMsg = this.fireEvent('error', this, status, textStatus, error, xhr);
                if(allowErrorMsg !== false){
                    Fw.Msg.error(error || ('[' + status + ']' + Fw.Ajax.error['default']));
                }
        }
    },
    /**
     * 返回Ajax请求配置
     * @param {Object} action
     */
    getRequestOperation: function(action){
        var operation;
        this.action = action;
        //自定义URL
        if(this.url){
            operation = {
                url: this.url,
                params: Fw.call(this.params)
            };
        }
        //Ares服务接口
        else{
            if(Fw.isDefined(this['method'])){
                this[action.toLowerCase() + 'Method'] = this['method'];
            }
            operation = {
                url: Fw.Ajax.getRequestURL(),
                params: Fw.Ajax.getRequestMessage.call(this, action),
                isStructMessage: true
            };
        }

        //设置等待层提示信息
        this.loadMask !== false && Fw.LoadMask && (this.loadMask = Fw.Ajax.loadMask[action]);
        
        //
        return operation;
    },
    /**
     * 返回Ajax请求报文（message）
     */
    getRequestMessage: function(action){
        //
        var spi, code;
        if(code = this.httpAdapter){
            spi = Fw.Ajax.HTTP_ADAPTER_SPI;
        }
        else if(code = this.socketAdapter){
            spi = Fw.Ajax.SOCKET_ADAPTER_SPI;
        }
        if(code){
            this.service = spi;
            action = Fw.Ajax.EXECUTE_METHOD;
            if(Fw.isArray(this.options) || Fw.isObject(this.options)){
                this.options = [{
                    _options: this.options,
                    _code: code
                }];
            }
            else{
                this.options = [code];
            }
        }
        //
        if(this.service){
            var message = {};
            var service = this.service.split('.');
            var method = action.toLowerCase();
            method = service.length > 1 ? service[1] : (this[method + 'Method'] || method || null);
            //原子服务
            if(method){
                var options = this.options || null;
                if(action === Fw.Ajax.QUERY_METHOD){
                    if(this.sort){
                        if(!options){
                            options = {};
                        }
                        options.sort = this.sort;
                    }
                }
                message.header = {
                    service: service[0],
                    method: method,
                    staticData: this.staticData || false,
                    options: options,
                   _t: '${_t}'//(new Date()).getTime()
                };
                delete this.options;
            }
            else{
                alert('未设置服务请求方法[method]');
                return null;
            }
            //请求参数
            message.payload = Fw.call(this.params) || null;
            //
            return message;
        }
        else{
            alert('未设置请求服务[service]');
            return null;
        }
    },
    /**
     * 返回Ajax请求路径
     */
    getRequestURL: function(){
        if(!Fw.isURL(Fw.Ajax.HTTP_CHANNEL)){
            Fw.Ajax.HTTP_CHANNEL = Fw.getBasePath() + '/' + Fw.Ajax.HTTP_CHANNEL;
        }
        return Fw.Ajax.HTTP_CHANNEL;
    },
    /**
     * 是否JSONP
     */
    isJSONP: function(){
        return $.ajax.isNative !== true && Fw.Ajax.getRequestURL().indexOf(Fw.getBasePath()) !== 0;
    },
    /**
     * 解析请求参数（message.header.options）
     */
    parseOperationArguments: function(args, owner, action){
        var autoRequest = !!action;
        if(owner){
            owner.options = Array.prototype.slice.call(args, 0, args.length);
        }
        else{
            owner = args[args.length - 1];
            owner.options = Array.prototype.slice.call(args, 0, args.length - 1);
        }
        if(autoRequest){
            //自动提交请求
            Fw.Ajax.request.call(owner, Fw.Ajax.getRequestOperation.call(owner, action));
        }
        
        //
        return owner;
    }
});

/**
 * Ajax加载队列
 */
Fw.Ajax.Queue = function(){
    //加载队列
    var queue = {};
    
    //
    return {
        /**
         * 加入队列  
         */
        add: function(xhr){
            if(xhr){
                xhr['_KEY'] = this['_KEY'];
                if(this.ignoreQueue !== true){
                    queue[xhr.id] = xhr;
                }
            }
        },
        /**
         * 移出队列 
         */
        remove: function(xhr){
            if(xhr){
                delete queue[xhr.id];
            }
        },
        /**
         * 队列是否为空  
         */
        isEmpty: function(){
            for(var p in queue){
                return false;
            }
            return true;
        }
    };
}();

/**
 * Ajax工具类 
 */
Fw.util.Ajax = function(){
    //默认配置
    var config = {
        //default
        requestMethod: Fw.Ajax.prototype.requestMethod,
        dataType: Fw.Ajax.prototype.dataType,
        async: Fw.Ajax.prototype.async,
        timeout: Fw.Ajax.prototype.timeout,
        //接收Ajax事件
        fireEvent: function(){
            var type = arguments[0];
            var args = Array.prototype.slice.call(arguments, 2, arguments.length);
            switch(type){
                case 'success':
                    return Fw.call(this.success, null, args);
                    break;
                case 'error':
                	return Fw.call(this.failure, null, args);
                    break;
            }
        }
    };
    //设置请求配置
    var applyRequestConfig = function(){
        Fw.applyIf(this, config);
    };
    //执行请求
    var doRequest = function(args, action){
        args = Fw.Ajax.parseOperationArguments(args);
        applyRequestConfig.call(args);
        var operation = Fw.Ajax.getRequestOperation.call(args, action);
        Fw.Ajax.request.call(args, operation);
    };
    
    //工具方法
    return {
        /**
         * Ajax请求
         * @param {Object} args
         */
        request: function(args){
            Fw.util.Ajax.query(args);
        },
        /**
         * 查询数据
         * @param {Object} args
         */
        query: function(args){
            applyRequestConfig.call(args);
            var operation = Fw.Ajax.getRequestOperation.call(args, Fw.Ajax.QUERY_METHOD);
            Fw.Ajax.request.call(args, operation);
        },
        /**
         * 数据获取
         * @param {Array} arguments
         */
        get: function(){
            doRequest(arguments, Fw.Ajax.GET_METHOD);
        },
        /**
         * 新增数据
         * @param {Array} arguments
         */
        insert: function(){
            doRequest(arguments, Fw.Ajax.INSERT_METHOD);
        },
        /**
         * 删除数据
         * @param {Array} arguments
         */
        del: function(){
            doRequest(arguments, Fw.Ajax.DELETE_METHOD);
        },
        /**
         * 更新数据
         * @param {Array} arguments
         */
        update: function(){
            doRequest(arguments, Fw.Ajax.UPDATE_METHOD);
        },
        /**
         * 执行数据提交
         * @param {Array} arguments
         */
        execute: function(){
            doRequest(arguments, Fw.Ajax.EXECUTE_METHOD);
        }
    };
}();
