/**
 * Fw.util.Format
 */
Fw.util.Format = {
    /**
     * 转换为日期型
     * @param {} v
     * @return {date}
     */
    toDate: function(v){
        if(Fw.isDate(v)){
            return v;
        }
        if(Fw.isNumber(v*1)){
            v = Date.parse(v * 1);
        }
        else{
            v = Date.parse(Fw.util.Format.fmtDate(v, 'yyyy/MM/dd HH:mm:ss'));
        }
        return Fw.isNumber(v) ? new Date(v) : null;
    },
    
    /**
     * 利率格式
     * @param {} v	原始数字
	 * @param {} format	长度
     * @return {string}
     */
    fmtInterest: function(v, format){
    	if(!v) return null;
    	var baseNum = 1; 
    	baseNum = baseNum.toFixed(format);
    	baseNum = baseNum.replace('.','');
    	var interestNum = (v / baseNum);
    	interestNum = interestNum.toFixed(format);
    	return interestNum + '%';
    },
    
    /**
     * 江苏农信日期格式
     * @param {} v	原始数字
	 * @param {} format	长度
     * @return {string}
     */
    fmtJSNXStringDate: function(v){
    	return v = v.substring(0, 4)+'-'+v.substring(5,7)+'-'+v.substring(8,11);
    },
    /**
     * 江苏农信日期格式化
     * @param {} v日期字符串
     * @return {string}
     */
    fmtStrSubDate: function(v){
    	return v = v.substring(0, 4)+'-'+v.substring(4,6)+'-'+v.substring(6,8);
    },
    /**
     * 江苏农信时间格式化
     * @param {} v日期字符串
     * @return {string}
     */
    fmtStrDataPoint :function(v){
    	//不能用replaceAll
    	return v =  v.replace('.', ':').replace('.', ':');
    },
    
	/**
	 * 日期格式化
	 * @param {} v
	 * @param {} format
     * @return {string}
	 */
	fmtDate: function(v, format, /*private*/ parse){
		if(!v) return null;
		if (!Fw.isDate(v)) {
		    if(parse !== false){
		        var value = v;
		        v = String(v).replace(/[^0-9.]/gi, '').split('.')[0];
                switch(v.length){
                    //HHmm 2359
                    case 4:
                        return Fw.util.Format.fmtDate('19000101' + v + '00', format || Fw.util.Format.dateFormat['HM'], false);
                    //HHmmss 235959
                    case 6:
                        return Fw.util.Format.fmtDate('19000101' + v, format || Fw.util.Format.dateFormat['HMS'], false);
                    //yyyyMMdd 20151230
                    case 8:
                        return Fw.util.Format.fmtDate(v + '000000', format, false);
                    //yyyyMMddHHmmss 20151230235959
                    case 14:
                        return Fw.util.Format.fmtDate(v, format, false);
                    //long || other
                    default:
                        return Fw.util.Format.fmtDate(Fw.isNumber(value*1) ? value*1 : Date.parse(value), format, false);
                }
		    }
		    else{
		        if(v.length === 14){
		            //parse
                    v = v.substring(0, 4) + '/' + v.substring(4, 6) + '/' + v.substring(6, 8) + ' ' + v.substring(8, 10) + ':' + v.substring(10, 12) + ':' + v.substring(12, 14);
                    return Fw.util.Format.fmtDate(Date.parse(v), format, false);
		        }
		        else{
		            //long
		            v = new Date(v);
		        }
		    }
		}
		return v.format(format || Fw.util.Format.dateFormat['YMD']);
	},
    /**
     * 日期反格式化
     * @param {} v
     * @return {string}
     */
    unFmtDate: function(v){
        var format = 'yyyy/MM/dd';
        if(!Fw.isDate(v) && v){
            v = String(v).replace(/[^0-9.]/gi, '').split('.')[0];
            switch(v.length){
                //yyyyMMddHHmmss 20151230235959
                case 14:
                    format = 'yyyy/MM/dd HH:mm:ss';
                    break;
                //HHmm 2359
                case 4:
                    format = 'HH:mm';
                    break;
                //HHmmss 235959
                case 6:
                    format = 'HH:mm:ss';
                    break;
            }
        }
        return Fw.util.Format.fmtDate(v, format);
    },
    /**
     * 百分比格式化
     * @param v 原始数字
     * @param format 格式/小数点后保留位数（默认为2）
     * @return {string}
     */
    fmtPercent: function(v, format) {
        return Fw.util.Format.fmtNumber(v, format) + '%';
    },
    /**
     * 百分比反格式化
     * @param v 百分比
     * 去逗号、去百分号
     * @return {number}
     */
    unFmtPercent: function(v) {
        return Fw.util.Format.unFmtNumber(v.replaceAll('%', ''));
    },
	/**
	 * 格式化金额
     * @param v 原始金额
     * @param decimals 小数点后保留位数（默认为2）
     * @param end 货币符号是否放在末尾（默认为false）
     * @param currencySign 货币符号（默认为￥）
     * @return {string}
	 */
    fmtAmt: function(v, decimals, end, currencySign) {
        var negativeSign = '', format = ',0';
        v = Fw.util.Format.unFmtAmt(v);
        if(!Fw.isNumber(v)){
            return v;
        }
        v = v - 0;
        if (v < 0) {
            v = -v;
            negativeSign = '-';
        }
        currencySign = Fw.isDefined(currencySign) ? currencySign || '' : Fw.util.Format.currencySign;
        decimals = Fw.isDefined(decimals) ? decimals : 2;
        format += (decimals > 0 ? '.' : '') + '0'.repeat(decimals);
        v = Fw.util.Format.fmtNumber(v, format);
        //
        if(end === true){
            return '{0}{1}{2}'.format(negativeSign, v, currencySign);
        }
        else{
            return '{0}{1}{2}'.format(negativeSign, currencySign, v);
        }
    },
    /**
     * 金额反格式化
     * 去逗号、去货币符号
     * @return {number}
     */
    unFmtAmt: function(v) {
        return Fw.isNumber(v) ? v : Fw.util.Format.unFmtNumber(String(v).replaceAll(Fw.util.Format.currencySign, ''));
    },
    /**
     * 小写金额转大写金额（默认到分）
     * @param v 原始数字/金额
     * @return {string}
     */
    fmtAmt2Chinese: function(v){
        v = Math.abs(Fw.util.Format.unFmtAmt(v));
        var number = Math.round(v * 100) / 100;
        number = number.toString(10).split('.');
        var a = number[0];
        if (a.length > 12){
            return '金额超出最大范围[999999999999.99]';
        }
        var e = '零壹贰叁肆伍陆柒捌玖';
        var num1 = '';
        var len = a.length - 1;
        for (var i = 0; i <= len; i++){
            num1 += e.charAt(parseInt(a.charAt(i))) + [['圆','万','亿'][Math.floor((len-i)/4)],'拾','佰','仟'][(len - i) % 4];
        }
        if (number.length == 2 && number[1] != '') {
            var a = number[1];
            for (var i = 0; i < a.length; i++){
                num1 += e.charAt(parseInt(a.charAt(i))) + ['角','分'][i];
            }
        }
        num1 = num1.replace(/零佰|零拾|零仟|零角/g, '零');
        num1 = num1.replace(/零{2,}/g, '零');
        num1 = num1.replace(/零(?=圆|万|亿)/g, '');
        num1 = num1.replace(/亿万/, '亿');
        num1 = num1.replace(/^圆零?/, '');
        if (num1 != '' && !/分$/.test(num1)){
            num1 += '整';
        }
        return num1;
    },
    
    fmtToRate:function(v){
    	if (!Fw.isNumber(v)) {
            v = parseFloat(v);
        }
    	v=v*100;
    	v=v + '%';
    	return v;
    },
    
    /**
     * 数字格式化
     * @param v 原始数字
     * @param format 格式/小数点后保留位数（默认为2）
     * 0 - 123456
     * 0.00 - 123456.78
     * 0.0000 - 123456.7890
     * 0,000 - 123,456
     * 0,000.00 - 123,456.78
     * 0,0.00 - 123,456.78
     * @return {string}
     */
    fmtNumber: function(v, format) {
        if (!Fw.isDefined(format)) {
            format = '0.00';
        }
        if (Fw.isNumber(format)) {
            if(format <= 0){
                format = '0';
            }
            else{
                format = '0.' + '0'.repeat(format);
            }
        }
        
        //
        v = v * 1;
        if (!Fw.isNumber(v)) {
            return '';
        }
        var comma = ',', dec = '.', i18n = false, neg = v < 0;
        v = Math.abs(v);
        if (format.substr(format.length - 2) == '/i') {
            format = format.substr(0, format.length - 2);
            i18n = true;
            comma = '.';
            dec = ',';
        }
        var hasComma = format.indexOf(comma) != -1, psplit = (i18n ? format.replace(/[^\d\,]/g, '') : format.replace(/[^\d\.]/g, '')).split(dec);
        if (1 < psplit.length) {
            v = v.toFixed(psplit[1].length);
        } 
        else if (2 < psplit.length) {
            throw ('NumberFormatException: invalid format, formats should have no more than 1 period: ' + format);
        } 
        else {
            v = v.toFixed(0);
        }
        var fnum = v.toString();
        psplit = fnum.split('.');
        if (hasComma) {
            var cnum = psplit[0], parr = [], j = cnum.length, m = Math.floor(j / 3), n = cnum.length % 3 || 3, i;
            for ( i = 0; i < j; i += n) {
                if (i != 0) {
                    n = 3;
                }
                parr[parr.length] = cnum.substr(i, n);
                m -= 1;
            }
            fnum = parr.join(comma);
            if (psplit[1]) {
                fnum += dec + psplit[1];
            }
        } 
        else {
            if (psplit[1]) {
                fnum = psplit[0] + dec + psplit[1];
            }
        }
        return (neg ? '-' : '') + format.replace(/[\d,?\.?]+/, fnum);
    },
    /**
     * 数字反格式化
     * 去逗号
     * @param v 格式化数字
     * @return {number}
     */
    unFmtNumber: function(v){
        if(Fw.isNumber(v) || !v){
            return v;
        }
        else{
            return v.replaceAll(',', '') * 1;
        }
    },
    /**
     * 账号格式化、 添加空格分隔符
     * @param  acc {string} 账户号
     * @param mask {boolean}  是否隐藏账户  true为隐藏
     * @return {string}
     * 账号格式要求： 前4位+**** ****+后四位
     */
    fmtAcctNo: function(acc, mask){
        if (Fw.isEmpty(acc)) {
            return '';
        }
        if (acc.length < 8) {
            return acc;
        }
        acc = String(acc).trim(true);
        var tmpStr = '';
        if (mask) {
//            acc = Fw.util.Format.fmtAcctNo(acc);//帐号格式化
            var start = acc.length - 4;
            if (start < 4) {
                start = 4;
            }
            tmpStr = tmpStr + acc.substring(0, 4) + '**** ****' + acc.substring(start, acc.length);
        }
        else {
            while (acc.length > 4 ) {
                tmpStr = tmpStr + acc.substring(0, 4);
                tmpStr = tmpStr + ' ';
                acc = acc.substring(4, acc.length);
            }
            tmpStr = tmpStr + acc;
        }
        return tmpStr;
    },
    
    getAcctLast4:function(acc){
    	if (Fw.isEmpty(acc)) {
            return '';
        }
    	if (acc.length < 4) {
            return acc;
        }
    	
    	return acc.substring(acc.length-4);
    },
    
    /**
     * 手机号格式化、 添加空格分隔符
     * @param  phnum {string} 手机号
     * @return {string}
     * 手机号格式要求： 前3位+****+后四位
     */
    fmtPhoneNum: function(phnum){
        if (Fw.isEmpty(phnum)) {
            return '';
        }
        phnum = String(phnum).trim(true);
        var tmpStr = '';
      
            var start = phnum.length - 4;
            if (start < 4) {
                start = 4;
            }
        tmpStr = tmpStr + phnum.substring(0, 3) + '****' + phnum.substring(start, phnum.length);
        return tmpStr;
    },
    
    /**
     * 字符串隐藏
     * @param  str 原始字符串
     * @param  pos 开始位置
     * @param  length 要替换的字符串的长度
     * @param  newChar 替换使用的字符串，默认为*
     * @return 替换后的字符串
     * 
     */
    hideCode: function(str, pos, length, newChar){
    	pos = pos || 0;
    	length = length || 0;
    	newChar = newChar || '*';
    	
    	if(pos<0 || length<=0 || pos+length>str.length)
    	{
    		return str;
    	}
    	
    	var repStr="";
    	for(var i=0; i<length; i++){
    		repStr = repStr+newChar;
    	}
    	
    	var result = str.slice(0,pos) + repStr + str.slice(pos+length, str.length);
    	return result;
    },
    
    /**
     * 帐号反格式化
     * 去空格（不支持星号反显）
     * @param acc 格式化数字
     * @return {string}
     */
    unFmtAcctNo: function(acc){
        return String(acc).trim(true);
    },
    fmtDeviceNo: function(deviceNo){
    	var length  = deviceNo.length;
    	var No = deviceNo;
		if(length>20){
			No = deviceNo.substring(1,9)+"..." +deviceNo.substring(length-8);
		}
    	return No;
    },
    /**
     * 付款状态
     */
    fmtTransStateType : function(state){
    	var state_cn = "";
		if(state == "OK"){
			state_cn = "交易成功";			
		}else if(state == "BH"){
			state_cn = "待付款";
		}else if(state == "CL"){
			state_cn = "已撤销";
		}else if(state == "UC"){
			state_cn = "状态未明";
		}else if(state == "FL"){
			state_cn = "交易失败";
		}else{
			state_cn = "其他";
		}
		return state_cn;
    },
    /**
     * 格式化证件类型
     * @param type
     * @returns {string}
     */
    fmtIdentityType: function(type){
        switch(String(type)){
            case 'I': return '身份证';
            case 'T': return '临时身份证';
            case 'S': return '军官证/士兵证';
            case 'P': return '护照';
            case 'L': return '营业执照';
            case 'O': return '其他有效证件';
            case 'R': return '户口簿';
            case 'H': return '港澳居民来往内地通行证';
            case 'W': return '台湾同胞来往内地通行证';
            case 'F': return '外国人居留证';
            case 'C': return '警官证';
            default: return type;
        }
    },
    /**
     * 格式化证件类型
     * @param type
     * @returns {string}
     */
    fmtStmCodeType: function(type){
        switch(String(type)){
            case 'EM': return '电子邮件寄送';
            case 'LE': return '信件电子邮件寄送';
            case 'LT': return '信件寄送';
            case 'NO': return '不寄送账单';           
            default: return type;
        }
    },
    /**
     * 格式化工作类型
     * @param type
     * @returns {string}
     */
    fmtJobType: function(type){
        switch(String(type)){
            case '01': return '机关团体';
            case '02': return '邮电通信';
            case '03': return '计算机';
            case '04': return '网络';
            case '05': return '商业/贸易';
            case '06': return '银行/保险/证券/金融/投资';
            case '07': return '财政/税务';
            case '08': return '咨询';
            case '09': return '自由职业者';
            case '10': return '社会服务';
            case '11': return '旅游/饭店';
            case '12': return '健康/医疗服务';
            case '13': return '房地产';
            case '14': return '交通运输';
            case '15': return '法律司法';
            case '16': return '文化/娱乐/体育';
            case '17': return '科研/教育';
            case '18': return '农林牧渔';
            case '19': return '矿业/制造';
            case '20': return '学生';
            default: return type;
        }
    },
    /**
     * 格式化文件大小
     * @param {} size
     * @return {string}
     */
    fileSize: function(size){
        var byteLimit = 1024, kbLimit = 1048576, mbLimit = 1073741824;
        var out;
        if (size < byteLimit) {
            if (size === 1) {
                out = '1 byte';    
            } else {
                out = size + ' bytes';
            }
        } else if (size < kbLimit) {
            out = (Math.round(((size*10) / byteLimit))/10) + ' KB';
        } else if (size < mbLimit) {
            out = (Math.round(((size*10) / kbLimit))/10) + ' MB';
        } else {
            out = (Math.round(((size*10) / mbLimit))/10) + ' GB';
        }
        return out;
    },
    /**
     * 对于大于指定长度的字符串进行裁剪，增加（…）
     * @param {} v
     * @param {} length
     * @param {} [word]
     * @return {string}
     */
    ellipsis: function(v, length, word) {
        if (v && v.length > length) {
            if (word) {
                var vs = v.substr(0, length - 2),
                index = Math.max(vs.lastIndexOf(' '), vs.lastIndexOf('.'), vs.lastIndexOf('!'), vs.lastIndexOf('?'));
                if (index !== -1 && index >= (length - 15)) {
                    return vs.substr(0, index) + '...';
                }
            }
            return v.substr(0, length - 3) + '...';
        }
        return v;
    },
    /**
     * 库存变化类型
     */
    fmtKCBHLX: function(v){
        switch(String(v)){
            case '1': return '入库';
            case '2': return '提货申请';
            case '3': return '提货申请修改';
            case '4': return '当日撤销提货申请';
            case '5': return '自提出库';
            case '6': return '交提出库';
            case '7': return '质押（质权方）';
            case '8': return '质押（出质方）';
            case '9': return '质押注销（质权方）';
            case 'A': return '质押注销（出质方）';
            case 'B': return '租借（借出）';
            case 'C': return '租借（借入）';
            case 'D': return '法律冻结';
            case 'E': return '法律冻结解冻';
            case 'F': return '现货买入';
            case 'G': return '现货卖出';
            case 'H': return '处置质物';
            case 'K': return '出库';
            case 'M': return '大宗交易买入';
            case 'N': return '大宗交易卖出';
            case 'O': return '划出库存';
            case 'P': return '划入库存';
            case 'Q': return '充抵做市商透支';
            case 'R': return '减做市商入库库存';
            case 'S': return '非即期卖出';
            case 'T': return '非即期买入';
            case 'U': return '次日撤销提货申请';
            case 'V': return '系统撤销提货申请';
            default: return v;
        }
    },
    /**
     * 风险等级
     */
    fmtRiskLevel: function(v){
    	switch(String(v)){
            case '0': return '低';
            case '1': return '较低';
            case '2': return '中等';
            case '3': return '中高';
            case '4': return '高';
            default: return v;
        }
    },
    /**
     * 车牌颜色
     */
    fmtLicenseColorStatues: function(v){
    	switch(String(v)){
    	case '0': return '蓝色';
    	case '1': return '黄色';
    	case '2': return '黑色';
    	case '3': return '白色';
    	case '4': return '渐变绿色';
    	case '5': return '黄绿双拼色';
    	case '6': return '蓝白渐变色';
    	case '9': return '其他';
    	case '12': return '红色';
    	case '11': return '绿色';
    	default: return v;
    	}
    },
    /**
     * 隐藏过长字符串内容
     */
    fmtContent : function(c,m) {
		if (Fw.isEmpty(c)) {
			return '';
		}
		str = String(c).trim(true);
		var tmpStr = '';
		if(str.length>8){
			var start = str.length - 4;
			tmpStr = tmpStr + str.substring(0, 4) + '***'
					+ str.substring(str.length-2, str.length);
		}else{
			tmpStr=c;
		}
			
		return tmpStr;
	},
	
	/**
     * 卡类型
     */
    fmtCardType: function(v){
    	switch(String(v)){
            case 'PCRC': return '圆鼎信用卡';
            case 'PDBC': return '圆鼎卡';
            case 'PSAV': return '一证通';
            case 'PSDP': return '存折';
	        case 'PDAC': return '电子账户';
	        case 'PDBCZX': return '圆鼎直销银行卡';
            default: return v;
        }
    },
    
    //制卡信息查询
    /*
	 * 卡邮寄状态
	 * 包含：0 普通投递1 加急制卡3 缺卡待处理  5 放弃制卡
	 */
    fmtMailStatusType: function(v){
    	switch(String(v)){
            case '0': return '普通投递';
            case '1': return '加急制卡';
            case '3': return '缺卡待处理 ';
            case '5': return '放弃制卡';
            default: return v;
        }
    },
	/*卡种包含：
	1 借记卡
	2 贷记卡
	*/
    fmtCodeType: function(v){
    	switch(String(v)){
            case '0': return '借记卡';
            case '1': return '贷记卡';
            default: return v;
        }
    },
    
    //卡片邮寄状态 包含：0 普通投递1 加急制卡3 缺卡待处理5 放弃制卡
    fmtMailStatusType: function(v){
    	switch(String(v)){
            case '0': return '普通投递';
            case '1': return '加急制卡';
            case '3': return '缺卡待处理';
            case '5': return '放弃制卡';
            default: return v;
        }
    },
    
    
    //卡片任务状态包含： 1 制卡中2 已邮寄
    fmtBusStatusType: function(v){
    	switch(String(v)){
            case '1': return ' 制卡中';
            case '2': return '已邮寄';
            default: return v;
        }
    },
    
    //二次投递的卡片状态1 退卡入库 2 退卡出库 3 退卡邮寄 
    fmtEmsStatus2Type: function(v){
    	switch(String(v)){
            case '1': return '退卡入库';
            case '2': return '退卡出库';
            case '3': return '退卡邮寄 ';
            default: return v;
        }
    },
    
    
    
    fmtCardflag: function(v){
    	switch(String(v)){
        case 'PCRC': 
        case 'PDBC': return '1';
        case 'PSAV':
        case 'PSDP': return '2';
        default: return v;
    }
    },
    /**
     * 币种变化类型
     */
    fmtCurrency: function(v){
        switch(String(v)){
            case 'ALL': return '阿尔巴尼亚列克';
			case 'ATS': return '奥地利先令';
			case 'AUD': return '澳大利亚元';
			case 'BEF': return '比利时法郎';
			case 'BEU': return '金融比利时法郎';
			case 'BGL': return '保加利亚列瓦';
			case 'BUK': return '缅甸币';
			case 'CAD': return '加拿大元';
			case 'CHF': return '瑞士法郎';
			case 'CNY': return '人民币';
			case 'CSK': return '捷克克郎';
			case 'DEM': return '德国马克';
			case 'DKK': return '丹麦克郎';
			case 'DNE': return '第那儿';
			case 'DZD': return '阿尔及利亚第纳尔';
			case 'EEE': return '折欧元合计';
			case 'ESP': return '西班牙比塞塔';
			case 'EUR': return '欧元';
			case 'FIM': return '芬兰马克';
			case 'FRF': return '法国法郎';
			case 'GBP': return '英镑';
			case 'GHC': return '加纳塞地';
			case 'GNS': return '几内亚西里';
			case 'HKD': return '港币';
			case 'HUF': return '匈牙利福林';
			case 'IEP': return '爱尔兰镑';
			case 'INR': return '印度罗比';
			case 'IQD': return '伊拉克地那尔';
			case 'IRR': return '伊朗里亚尔';
			case 'ITL': return '意大利里拉';
			case 'JPY': return '日元';
			case 'KPW': return '朝鲜币';
			case 'KWD': return '科威特第纳尔';
			case 'LKP': return '斯里兰卡卢比';
			case 'LUF': return '卢森堡法郎';
			case 'MAD': return '摩洛哥地拉姆';
			case 'MLF': return '马里法郎';
			case 'MOP': return '澳门币';
			case 'MTP': return '马尔他镑';
			case 'MUL': return 'RD, 多币种';
			case 'MYR': return '马来西亚林元';
			case 'NAD': return '新西兰元';
			case 'NLG': return '荷兰盾';
			case 'NOK': return '挪威克郎';
			case 'NPR': return '尼泊尔卢比';
			case 'NZD': return '新西兰元';
			case 'PHP': return '菲律宾比索';
			case 'PKR': return '巴基斯坦卢比';
			case 'PLZ': return '波兰兹罗提';
			case 'PTE': return '葡萄牙埃斯库多';
			case 'ROL': return '罗马尼亚列依';
			case 'RRR': return '折人民币合计';
			case 'RUB': return '卢布';
			case 'SEK': return '瑞典克郎';
			case 'SGD': return '新加坡元';
			case 'SLL': return '塞拉利昂利昂';
			case 'THB': return '泰币';
			case 'TUG': return '蒙古图格里克';
			case 'TZS': return '坦桑尼亚先令';
			case 'USD': return '美元';
			case 'UUU': return '折本外币';
			case 'VND': return '越南盾';
			case 'ZAR': return '兰特';
			case 'ZMK': return '赞比亚克瓦查';
            default: return v;
        }
    },
    /**
     * 存期参数
     */
    fmtTerm: function(v){
    	switch(String(v)){
    		case 'N1': return '一天';
			case 'N7': return '七天';
    		case 'D1': return '一天';
    		case 'D7': return '七天';
            case 'M3': return '三个月';
            case 'M6': return '六个月';
            case 'Y1': return '一年';
            case 'Y2': return '二年';
            case 'Y3': return '三年';
            case 'Y5': return '五年';
            default: return v;
        }
    },
    /**
     * 自动还款方式
     */
    fmtPayPattern : function(v){
    	var payPattern = v;
    	switch (v) {
		case "T":
			payPattern = "全额还款";
			break;
		case "M":
			payPattern = "最低额度还款";
			break;
		case "F":
			payPattern = "固定金额还款";
			break;
		}
    	return payPattern;
    },
    /**
     * 支付状态
     */
    fmtIsPayType : function(v){
    	var isPay = v;
    	switch (v) {
		case "0":
			isPay = "待支付";
			break;
		case "1":
			isPay = "已支付";
			break;
		case "2":
			isPay = "受理中";
			break;
		case "3":
			isPay = "已拒绝";
			break;
		}
    	return isPay;
    },
    /**
     * 存款类型相关
     */
	fmtDepositType : function(v) {
	  switch (String(v)) {
		case 'AGS':
			return '协定存款';
		case 'ARA':
			return '个人智能利生利7天';
		case 'ARZ':
			return '个人智能利生利1天';
		case 'CCE':
			return '本票';
		case 'CHE':
			return '支票';
		case 'DC':
			return '通知存款';
		case 'DC1':
			return '一天通知存款';
		case 'DC7':
			return '七天通知存款';
		case 'DPB':
			return '存折';
		case 'DPF':
			return '同业存放';
		case 'DRA':
			return '汇票';
		case 'EDD':
			return '教育储蓄';
		case 'GDD':
			return '一般存款账户';
		case 'GJJ':
			return '公积金贷款';
		case 'ISD':
			return '保值储蓄';
		case 'LDD':
			return '大额定期';
		case 'LNS':
			return '贷款';
		case 'MAR':
			return '保证金';
		case 'NDB':
			return '国债账户';
		case 'PRI':
			return '结算子账户';
		case 'SAT':
			return '专用户';
		case 'SAV':
			return '活期';
		case 'SDK':
			return '商业贷款';
		case 'SFD':
			return '商业房贷';
		case 'TDA':
			return '临时存款账户';
		case 'TDD':
			return '定活两便';
		case 'TIM':
			return '定期';
		case 'TLL':
			return '整存整取';
		case 'ELL':
			return 'e行无忧';
		case 'TLO':
			return '整存零取';
		case 'TOL':
			return '零存整取';
		case 'TPI':
			return '存本取息';
		case 'PDAC':
			return '电子账户';
		default: return v;
		}
	},
	/**
	 * 存款方式
	 */
	fmtAutoType:function(v){
		switch (v) {
		case 'T':
			return '定期存款';
			break;
		case 'N':
			return '通知存款';
			break;
		default:
			return v;
			break;
		}
	},
	/**
	 * 签约状态
	 */
	fmtSignStatus:function(v){
		switch (v) {
		case '0':
			return "开通，已签约";
			break;
		case '1':
			return "解约，已终止";
			break;
		case '2':
			return "未签约";
			break;
		default:
			return v;
			break;
		}
	},
	/**
	 * 转存
	 */
	fmtRedeposit:function(d){
		if(d=="A" || d=="1"){
			return "自动转存";
		}else if(d=="0"){
			return "非自动转存";
		}
		return d;
	},
	/**
	 * 账户状态
	 */
	fmtAutoSubStatus:function(z){
		if(z=="0"){
			return "正常";
		}else if(z=="1"){
			return "销户";
		}else if(z=="2"){
			return "没收";
		}else if(z=="3"){
			return "上缴";
		}
		return z;
	},
	/**
	 * 理财产品类型
	 */
	fmtRunType: function(RunType){
		var runTypeName = RunType;
		if(RunType=="1"){
			runTypeName="封闭式";
		}else if(RunType=="2"){
			runTypeName="开放式净值型";
		}else{
			runTypeName="开放式非净值";
		}
		return runTypeName;
	},
	/**
	 * 理财交易状态
	 */
	fmtCustState:function(state){
		switch(state){
			case "1": return "认购";
			case "2": return "预约认购";
			case "3": return "质押";
			case "4": return "撤单";
			case "5": return "部分赎回";
			case "6": return "全部赎回";
			default: return state;
		}
	},
	
	/**
	 * 证件类型转换
	 */
	fmtIdType : function(IdType){
		switch(IdType){
		case "P00": return "身份证";
		case "P01": return "军官证";
		case "P02": return "文职干部证";
		case "P03": return "警官证";
		case "P04": return "士兵军人证";
		case "P05": return "中国护照";
		case "P06": return "港澳台居民通行证";
		case "P07": return "户口簿";
		case "P08": return "边民出入境通行证";
		case "P09": return "居民身份证（移植待规范）";
		case "P10": return "临时身份证";
		case "P11": return "离休干部荣誉证";
		case "P12": return "武警警官证誉证补折";
		case "P13": return "台湾居民往来通行证";
		case "P14": return "军官退休证";
		case "P15": return "文职干部退休证";
		case "P16": return "军事院校学员证";
		case "P17": return "武警士兵证";
		case "P18": return "武警文职干部证";
		case "P19": return "武警军官退休证";
		case "P20": return "武警文职干部退休证";
		case "P21": return "驾驶执照";
		case "P22": return "旧身份证";
		case "P23": return "外国护照";
		case "P24": return "外国人永久居留证";		
		case "P25": return "港澳台居民居住证";		
		case "P99": return "个人其他";
		case "E01": return "组织机构代码证";
		case "E02": return "营业执照";
		case "E03": return "企业名称核准书";
		case "E04": return "政府机构/公共机构批文";
		case "E05": return "香港商业登记证";
		case "E06": return "开户证明";
		case "E07": return "军队开户许可证";
		case "E08": return "社会团体";
		case "E09": return "非法人组织机构代码证";
		case "E10": return "贷款卡号";
		case "E99": return "企业其他";

		default: return IdType;
	}
	},
	
	/**
	 *  自助缴费快速管理【交行】【兴业】
	 */
	fmtSCodetoName:function(code){
			switch(code){
			case "10040000" : return "渤海证券";
			case "10059999" : return "长城融资";
			case "10000000" : return  "1000券商";
			case "10009999" : return  "1000券商";
			case "10040000" : return  "渤海证券";
			case "10049999" : return  "渤海证券";
			case "10050000" : return  "长城证券";
			case "10059999" : return  "长城融资融券";
			case "10060000" : return  "长江证券";
			case "10069999" : return  "长江证券融资融券";
			case "10080000" : return  "东海证券";
			case "10089999" : return  "东海融资融券";
			case "10150000" : return  "东北证券";
			case "10159999" : return  "东北融资融券";
			case "10160000" : return  "东方证券";
			case "10169999" : return  "东方融资融券";
			case "10170000" : return  "东莞证券";
			case "10200000" : return  "光大证券";
			case "10209999" : return  "光大融资融券";
			case "10230000" : return  "广发证券";
			case "10239999" : return  "广发融资融券";
			case "10240000" : return  "国海证券";
			case "10270000" : return  "国泰君安";
			case "10280000" : return  "招商证券";
			case "10289999" : return  "招商融资融券";
			case "10290000" : return  "国信证券";
			case "10299999" : return  "国信融资融券";
			case "10330000" : return  "海通证券";
			case "10339999" : return  "海通融资融券";
			case "10380000" : return  "宏源证券";
			case "10389999" : return  "宏源证券融资融券";
			case "10420000" : return  "华龙证券";
			case "10439999" : return  "华泰融资融券";
			case "10440000" : return  "华西证券";
			case "10460000" : return  "华鑫证券";
			case "10620000" : return  "平安证券";
			case "10680000" : return  "山西证券";
			case "10710000" : return  "上海证券";
			case "10720000" : return  "申银万国";
			case "10729999" : return  "申万融资融券";
			case "10730000" : return  "五矿证券";
			case "10760000" : return  "首创证券";
			case "10769999" : return  "首创融资融券";
			case "10820000" : return  "中信万通";
			case "10899999" : return  "西南证券融资融券";
			case "10900000" : return  "厦门证券";
			case "10910000" : return  "湘财证券";
			case "10949999" : return  "兴业证券融资融券";
			case "10979999" : return  "方正证券融资融券";
			case "10990000" : return  "中金证券";
			case "10999999" : return  "中金证券融资融券";
			case "11010000" : return  "中山证券";
			case "11019999" : return  "中山证券融资融券";
			case "11020000" : return  "中信证券";
			case "11029999" : return  "中信融资融券";
			case "11100000" : return  "大通证券";
			case "11109999" : return  "1110融资融券";
			case "11119999" : return  "9999测试";
			case "11179999" : return  "国元证券融资融券";
			case "11300000" : return  "开源证券";
			case "11620000" : return  "国都证券";
			case "11629999" : return  "国都证券融资融券";
			case "12340000" : return  "1234券商";
			case "12360000" : return  "齐鲁证券";
			case "12370000" : return  "江南证券";
			case "12710000" : return  "中信金通";
			case "12970000" : return  "日信证券";
			case "13080000" : return  "华宝证券";
			case "13119999" : return  "红塔证券融资融券";
			case "13160000" : return  "东吴证券";
			case "13180000" : return  "浙商证券";
			case "13360000" : return  "天源证券";
			case "13400000" : return  "德邦证券";
			case "13450000" : return  "江海证券";
			case "13480000" : return  "太平洋证券";
			case "13510000" : return  "中天证券";
			case "13549999" : return  "高华融资融券";
			case "13579999" : return  "中投证券融资融券";
			case "13590000" : return  "中信建投";
			case "13599999" : return  "中信建投融资融券";
			case "13660000" : return  "安信证券";
			case "13669999" : return  "安信证券融资融券";
			case "13690000" : return  "银河证券";
			case "13699999" : return  "银河融资融券";
			case "13739999" : return  "东兴证券融资融券";
			case "33339999" : return  "33融资融券";
			case "44440000" : return  "44券商";
			case "50019999" : return  "55券商融资融券";
			case "60018888" : return  "上海股交中心";
			case "60028888" : return  "辽宁股权交易中心";                                         			
			case "60050000" : return  "宏远证券";
			case "66669999" : return  "66券商";
			case "77779999" : return  "77券商";
			case "95550000" : return  "9555券商";
			case "95950000" : return  "95券商";
			case "97970000" : return  "97券商";
			case "97979999" : return  "97融资融券";
			case "98980000" : return  "98券商";
			case "98989999" : return  "98融资融券";
			case "99990000" : return  "99券商";
			case "10889999" : return  "西藏同信";
			case "13200000" : return  "财富证券";
			case "13209999" : return  "财富融资融券";
			case "94949999" : return  "94券商";
			case "10470000" : return  "测试使用券商";
			case "10610000" : return  "南京证券";
			default: return code;
		}
	},
	/**
	 *  二维码行号名称P
	 */
	BanktoNameP:function(code){
		console.log(code);
			switch(code){
		 case"402290000011":return "上海农商";
         case"306581000003":return "广发银行";
         case"102100099996":return "工商银行";
         case"103100000026":return "农业银行";
         case"104100000004":return "中国银行";
         case"313305066661":return "苏州银行";
         case"313591001001":return "南粤银行";
         case"402241000015":return "吉林农信";
         case"402451000010":return "山东农信";
         case"593100000020":return "友利银行";
         case"313791030003":return "长安银行";
         case"313585000990":return "华润银行";
         case"313335081005":return "嘉兴银行";
         case"313655091983":return "自贡商行";
         case"402331000007":return "浙江农信";
         case"313146000019":return "廊坊银行";
         case"313617000018":return "桂林银行";
         case"320345790018":return "浙江三门银座村镇银行";
         case"313451000019":return "齐鲁银行";
         case"313586000006":return "华兴银行";
         case"320100010011":return "北京顺义银座村镇银行";
         case"320428090311":return "江西赣州银座村镇银行";
         case"320584002002":return "深圳福田银座村镇银行";
         case"320653000104":return "重庆渝北银座村镇银行";
         case"320687000016":return "重庆黔江银座村镇银行";
         case"320343800019":return "浙江景宁银座村镇银行";
         case"313397075189":return "泉州银行";
         case"313312300018":return "江苏长江商业银行";
         case"314305106644":return "太仓农村商业银行";
         case"313791000015":return "西安银行";
         case"313454000016":return "枣庄银行";
         case"314641000014":return "海口联合农村商业银行";
         case"313424076706":return "九江银行";
         case"313673093259":return "南充市商业银行";
         case"313651099999":return "成都银行";
         case"502290000006":return "东亚银行";
         case"314302200018":return "江苏江阴农村商业银行";
         case"313521006000":return "湖北银行";
         case"105100000017":return "建设银行";
         case"301290000007":return "交通银行";
         case"302100011000":return "中信银行";
         case"303100000006":return "光大银行";
         case"304100040000":return "华夏银行";
         case"305100000013":return "民生银行";
         case"307584007998":return "平安银行";
         case"308584000013":return "招商银行";
         case"309391000011":return "兴业银行";
         case"310290000013":return "浦发银行";
         case"313100000013":return "北京银行";
         case"313110000017":return "天津银行";
         case"313121006888":return "河北银行";
         case"313127000013":return "邯郸市商业银行";
         case"313131000016":return "邢台银行";
         case"313138000019":return "张家口银行";
         case"313141052422":return "承德银行";
         case"313143005157":return "沧州银行";
         case"313161000017":return "晋商银行";
         case"313168000003":return "晋城银行";
         case"313191000011":return "内蒙古银行";
         case"313192000013":return "包商银行";
         case"313205057830":return "鄂尔多斯银行";
         case"313222080002":return "大连银行";
         case"313223007007":return "鞍山市商业银行";
         case"313227000012":return "锦州银行";
         case"313227600018":return "葫芦岛银行";
         case"313228000276":return "营口银行";
         case"313229000008":return "阜新银行";
         case"313241066661":return "吉林银行";
         case"313261000018":return "哈尔滨银行";
         case"313261099913":return "龙江银行";
         case"313175000011":return "晋中银行";
         case"313301008887":return "南京银行";
         case"313301099999":return "江苏银行";
         case"313331000014":return "杭州银行";
         case"313332082914":return "宁波银行";
         case"313333007331":return "温州银行";
         case"313336071575":return "湖州银行";
         case"313337009004":return "绍兴银行";
         case"313338707013":return "浙江稠州商业银行";
         case"313345001665":return "台州银行";
         case"313345010019":return "浙江泰隆商业银行";
         case"313345400010":return "浙江民泰商业银行";
         case"313391080007":return "海峡银行";
         case"313393080005":return "厦门银行";
         case"313421087506":return "江西银行";
         case"313428076517":return "赣州银行";
         case"313433076801":return "上饶银行";
         case"313452060150":return "青岛银行";
         case"313453001017":return "齐商银行";
         case"313455000018":return "东营银行";
         case"313456000108":return "烟台银行";
         case"313458000013":return "潍坊银行";
         case"313461000012":return "济宁银行";
         case"313463000993":return "泰安市商业银行";
         case"313463400019":return "莱商银行";
         case"313465000010":return "威海市商业银行";
         case"313468000015":return "德州银行";
         case"313473070018":return "临商银行";
         case"313473200011":return "日照银行";
         case"313491000232":return "郑州银行";
         case"313551070008":return "华融湘江银行";
         case"313493080539":return "洛阳银行";
         case"313736000019":return "曲靖市商业银行";
         case"402791000010":return "陕西农信";
         case"313193057846":return "乌海银行";
         case"313521000011":return "汉口银行";
         case"313551088886":return "长沙银行";
         case"313581003284":return "广州银行";
         case"313602088017":return "东莞银行";
         case"313611001018":return "广西北部湾银行";
         case"313614000012":return "柳州银行";
         case"313653000013":return "重庆银行";
         case"313656000019":return "攀枝花市商业银行";
         case"313658000014":return "德阳银行";
         case"313659000016":return "绵阳市商业银行";
         case"313701098010":return "贵阳市商业银行";
         case"313731010015":return "富滇银行";
         case"313821001016":return "兰州银行";
         case"313851000018":return "青海银行";
         case"313871000007":return "宁夏银行";
         case"313881000002":return "乌鲁木齐市商业银行";
         case"313882000012":return "昆仑银行";
         case"402651020006":return "四川农信";
         case"314305206650":return "昆山农村商业银行";
         case"314305400015":return "吴江农村商业银行";
         case"314305506621":return "江苏常熟农村商业银行";
         case"314305670002":return "张家港农村商业银行";
         case"314581000011":return "广州农村商业银行";
         case"314588000016":return "佛山顺德农村商业银行";
         case"314653000011":return "重庆农村商业银行";
         case"315456000105":return "恒丰银行";
         case"316331000018":return "浙商银行";
         case"317110010019":return "天津农村商业银行";
         case"318110000014":return "渤海银行";
         case"319361000013":return "徽商银行";
         case"402100000018":return "北京农村商业银行";
         case"402332010004":return "宁波鄞州农村合作银行";
         case"402361018886":return "安徽省农村信用联社";
         case"402391000068":return "福建省农村信用社联合社";
         case"402521000032":return "湖北省农村信用社联合社";
         case"402584009991":return "深圳农村商业银行";
         case"402602000018":return "东莞农村商业银行";
         case"402611099974":return "广西壮族自治区农信";
         case"402641000014":return "海南农信";
         case"402731057238":return "云南农信";
         case"402871099996":return "宁夏黄河农村商业银行";
         case"403100000004":return "邮政储蓄银行";
         case"313491099996":return "中原银行";
         case"595100000007":return "新韩银行";
         case"596110000013":return "企业银行";
         case"313495081900":return "平顶山银行";
         case"597100000014":return "韩亚银行";
         case"322290000011":return "上海农村商业银行";
         case"323584000888":return "深圳前海微众银行";
         case"313148053964":return "衡水银行";
         case"313501080608":return "焦作中旅银行";
         case"402521090019":return "武汉农村商业银行";
         case"325290000012":return "上海银行";
         case"313332090019":return "宁波通商银行";
         case"320455000019":return "东营莱商村镇银行";
         case"402421099990":return "江西农信";
         case"314651000000":return "成都农商银行";
         case"313338009688":return "金华银行";
         case"787290000019":return "富邦华一银行";
         case"402581090008":return "广东农信";
         case"781393010011":return "厦门国际银行";
         case"314302066666":return "无锡农村商业银行";
         case"402701002999":return "贵州农信";
         case"313741095715":return "玉溪市商业银行";
         case"313221030008":return "盛京银行";
         case"321667090019":return "三峡银行";
         case"402301099998":return "江苏农信";
		 default: return "暂无此行";
		}
	},
	/**
	 *  二维码行号名称D
	 */
	BanktoNameD:function(code){
		console.log("code"+code);
		var code=code.substring(0,3);
		switch(code){
		case "102":return "工商银行";
		case "103":return "农业银行";
		case "104":return "中国银行";
		case "105":return "建设银行";
		case "203":return "农业发展银行";
		case "301":return "交通银行";
		case "302":return "中信银行";
		case "303":return "光大银行";
		case "304":return "华夏银行";
		case "305":return "民生银行";
		case "306":return "广发银行";
		case "307":return "平安银行";
		case "308":return "招商银行";
		case "309":return "兴业银行";
		case "310":return "浦发银行";
		case "313":return "城市商业银行";
		case "314":return "农村商业银行";
		case "316":return "浙商银行";
		case "318":return "渤海银行";
		case "319":return "徽商银行";
		case "401":return "城市信用社";
		case "402":return "农村信用社";
		case "403":return "邮政储蓄银行";
		case "531":return "花旗银行";
		case "501":return "汇丰银行";
		case "671":return "渣打银行";
		case "320":return "村镇银行";
		case "623":return "星展银行";
		case "564":return "瑞穗实业银行";
		case "315":return "恒丰银行";
		case "561":return "三菱东京日联银行";
		case "713":return "德国商业银行";
		case "712":return "德意志银行";
		case "694":return "东方汇理银行";
		case "563":return "三井住友银行";
		case "787":return "华一银行";
		case "201":return "国家开发银行";
		case "322":return "上海农商银行";
		case "661":return "荷兰银行";
		case "502":return "东亚银行";
		case "317":return "农村合作银行";
		case "321":return "重庆三峡银行";
		case "202":return "中国进出口银行";
		case "503":return "南洋商业银行";
		case "504":return "恒生银行";
		case "505":return "中国银行（香港）";
		case "506":return "集友银行";
		case "507":return "创兴银行";
		case "509":return "星展银行";
		case "510":return "永亨银行";
		case "512":return "永隆银行";
		case "532":return "美国银行";
		case "533":return "摩根大通银行";
		case "562":return "日本日联银行";
		case "565":return "日本山口银行";
		case "591":return "韩国外换银行";
		case "593":return "友利银行";
		case "594":return "韩国产业银行";
		case "595":return "新韩银行";
		case "596":return "韩国中小企业银行";
		case "597":return "韩亚银行";
		case "621":return "华侨银行";
		case "622":return "大华银行";
		case "631":return "泰国盘谷银行";
		case "641":return "奥地利中央合作银行";
		case "651":return "比利时联合银行";
		case "652":return "比利时富通银行";
		case "662":return "荷兰安智银行";
		case "672":return "英国苏格兰皇家银行";
		case "691":return "法国兴业银行";
		case "692":return "法国巴黎银行";
		case "693":return "法国东方汇理银行";
		case "695":return "法国外贸银行";
		case "711":return "德国德累斯登银行";
		case "714":return "德国西德银行";
		case "715":return "德国巴伐利亚州银行";
		case "716":return "德国北德意志州银行";
		case "732":return "意大利联合圣保罗银行";
		case "741":return "瑞士信贷银行";
		case "742":return "瑞士银行";
		case "751":return "加拿大丰业银行";
		case "752":return "加拿大蒙特利尔银行";
		case "761":return "澳大利亚和新西兰银行";
		case "771":return "摩根士丹利国际银行";
		case "775":return "联合银行";
		case "776":return "荷兰合作银行";
		case "781":return "厦门国际银行";
		case "782":return "法国巴黎银行";
		case "785":return "华商银行";
		case "999":return "其他银行"; 
		default: return "暂无此行";
		}
	},
	/**
	 * 确认状态
	 */
	fmtTransConfirmStatus:function(v){
		var TransConfirmStatus=v; //确认状态
		if(null==v||v==""){
			TransConfirmStatus="未确认";
		}else{
			switch(v){
				case "0":
				TransConfirmStatus="确认失败";
				break;
				case "1":
				TransConfirmStatus="确认成功";
				break;
				case "3":
				TransConfirmStatus="确认中";
				break;
				case "4":
				TransConfirmStatus="撤单";
				break;
				case "5":
				TransConfirmStatus="认购行为确认";
				break;
				case "9":
				TransConfirmStatus="未确认";
				break;
			};
		}
		return TransConfirmStatus;
	},
	/**
     * 金额单位分转元
     */
    formatterMoney: function(money){
    	var moneyNum = (Fw.util.Format.fmtNumber(money,'0.00')/100).toFixed(2);
    	return Fw.util.Format.fmtAmt(moneyNum);
    },
    /**
     * 金额单位元转分   格式  100000
     */
    fmtMoneyToPoint: function(money){
    	var moneyNum = Fw.util.Format.fmtNumber(money,'0.00')+"";
    	moneyNum = moneyNum.replace(".","");
    	return Number(moneyNum);
    },
	/**
     * 10：申请待处理20：待授权25：待文件请款30：待请款40：已请款90：取消91：拒绝
     */
    fmtCreditInstStatus : function(v){
    	var Status = v;
    	switch (v) {
		case '10':
			Status = '申请待处理';
			break;
		case '20':
			Status = '待授权';
			break;
		case '25':
			Status = '待文件请款';
			break;
		case '30':
			Status = '待请款';
			break;
		case '40':
			Status = '已请款';
			break;
		case '90':
			Status = '取消';
			break;
		case '91':
			Status = '拒绝';
			break;
		default:
			break;
		}
    	return Status;
    },
	 /**
     * 某个日期前或后m个月的日期
     */
	fmtSomeMonth:function(date,m){
		var rdate;
		//当前日期
		var arr=date.split('/');
		var year=arr[0];
		var month=arr[1];
		var day=arr[2];
		if(m>=0){
			month=parseInt(month)+m;
			if(month>12){
				month=parseInt(month)-12;
				year=parseInt(year)+1;
			};
		}else{
			month=parseInt(month)+m;
			if(month<0){
				month=parseInt(month)+12;
				year=parseInt(year)-1;
			}else if(month==0){
				month=12;
				year=parseInt(year)-1;
			};
		};

		var days=new Date(year,month,0);
		days=days.getDate();
			if(month<10){
			month='0'+month;
			}
		if(days>=day){
		rdate= year+'/'+month+'/'+day;
		}else{
		rdate= year+'/'+month+'/'+days;
		}
		return rdate;
	},

    /**
     * 截取字符串
     */
    fmtStrCut: function(v,start,end){
    	return v.substring(start,end);
    },
    
    /**
     * 二维码行名行号[D]跨行
     */
    fmtBankToNameD: function(v){
    		var v=v.substring(0,3);
    		var Status="";
        	switch (v) {
    		case '102':
    			Status = '102';
    			break;
    		case '103':
    			Status = '103';
    			break;
    		case '104':
    			Status = '104';
    			break;
    		case '105':
    			Status = '105';
    			break;
    		case '301':
    			Status = '301';
    			break;
    		case '302':
    			Status = '302';
    			break;
    		case '303':
    			Status = '303';
    			break;
    		case '304':
    			Status = '304';
    			break;
    		case '305':
    			Status = '305';
    			break;
    		case '306':
    			Status = '306';
    			break;
    		case '307':
    			Status = '307';
    			break;
    		case '308':
    			Status = '308';
    			break;
    		case '309':
    			Status = '309';
    			break;
    		case '310':
    			Status = '310';
    			break;
    		case '315':
    			Status = '315';
    			break;
    		case '316':
    			Status = '316';
    			break;
    		case '318':
    			Status = '318';
    			break;
    		case '319':
    			Status = '319';
    			break;
    		case '403':
    			Status = '403';
    			break;
    		default:
    			Status = 'bank';
    			break;
    		}
    	return Status;
    },
    
    /**
     * 二维码行名行号[P]跨行快速
     */
    fmtBankToNameP: function(v){
    	var Status = "";
		switch (v) {
		case '102100099996':
			Status = '102100099996';
			break;
		case '103100000026':
			Status = '103100000026';
			break;
		case '104100000004':
			Status = '104100000004';
			break;
		case '105100000017':
			Status = '105100000017';
			break;
		case '301290000007':
			Status = '301290000007';
			break;
		case '302100011000':
			Status = '302100011000';
			break;
		case '303100000006':
			Status = '303100000006';
			break;
		case '304100040000':
			Status = '304100040000';
			break;
		case '305100000013':
			Status = '305100000013';
			break;
		case '306581000003':
			Status = '306581000003';
			break;
		case '307584007998':
			Status = '307584007998';
			break;
		case '308584000013':
			Status = '308584000013';
			break;
		case '309391000011':
			Status = '309391000011';
			break;
		case '310290000013':
			Status = '310290000013';
			break;
		case '313100000013':
			Status = '313100000013';
			break;
		case '313222080002':
			Status = '313222080002';
			break;
		case '313301008887':
			Status = '313301008887';
			break;
		case '313301099999':
			Status = '313301099999';
			break;
		case '313305066661':
			Status = '313305066661';
			break;
		case '313332082914':
			Status = '313332082914';
			break;
		case '315456000105':
			Status = '315456000105';
			break;
		case '316331000018':
			Status = '316331000018';
			break;
		case '318110000014':
			Status = '318110000014';
			break;
		case '319361000013':
			Status = '319361000013';
			break;
		case '325290000012':
			Status = '325290000012';
			break;
		case '403100000004':
			Status = '403100000004';
			break;
		default:
			Status = 'bank';
			break;
		}
    	return Status;
    },
    /**
     * HH:mm:ss格式化
     */
    fmtTime: function(v){
    	var time = v;
    	if(v.indexOf('.')!=-1){
    		time = v.replaceAll('\\.',':');
    	}
    	return time;
    },
    
    /**
     * 处理金额undefined数据
     * return ''
     */
    fmtUndefined: function(data){
    	if(data){
    		return Fw.util.Format.fmtAmt(data);
    	}else{
    		return "";
    	}
    },
    
    /**
     * 截取字符串
     */
    fmtSubStr: function(v,start,end){
    	if(v.length>end){
    		v = v.substring(start,end)+"...";
    	}
    	return v;
    },
    
    /**
     * 格式化基金类型
     */
    fmtFundStyle: function(FundStyle){
    	var FundType = "";
    	switch(FundStyle){
	    	case "01":
	    		FundType="股票型基金";
	    		break;
	    	case "02":
	    		FundType="债券型基金";
	    		break;
	    	case "03":
	    		FundType="混合型基金";
	    		break;
	    	case "04":
	    		FundType="货币型基金";
	    		break;
	    	default:
	    		FundType="股票型";
	    		break;
    	}
    	return FundType;
    },
    /*
     * 网上支付交易查询交易状态
     */
    showTransState:function (state){
		switch(state){
			case "00": return "交易成功";
			case "01": return "交易失败";
			case "02": return "撤销成功";
			case "03": return "部分退货";
			case "04": return "全部退货";
			case "99": return "交易超时";
			default: return state;
		}
	},
	/*
	 * 贷款状态
	 */
	showAcctState : function(state){
    	switch(state){
			case "1": return "正常";
			case "2": return "逾期";
			case "3": return "非应计";
			case "5": return "结清";
			case "6": return "部分逾期";
			default: return state;
		}
    },
    
    /**
     * 茅山年卡状态
     */
    showCardState : function(state){
    	switch(state){
			case "0": return "正常";
			case "1": return "停用";
			case "2": return "已退订";
			default: return state;
		}
    },
	
    showDynamicSerialType : function(type){
    	switch(type){
			case "RSA_TIMETOKEN": return "RSA电子令牌";
			case "DC_TIMETOKEN": return "DC电子令牌";
			case "TODOS_EASYTOKEN": return "TODOS双向认证令牌";
			case "TODOS_ONETIMECARDTOKEN": return "TODOS双向认证刮刮卡";
			case "TODOS_SMARTCARDTOKEN": return "TODOS智能卡";
			case "RSA_HY2TOKEN": return "RSA恒银二代口令";
			case "RSA_HB2TOKEN": return "RSA恒宝二代口令";
			case "RSA_HBHHTOKEN": return "RSA恒宝混合口令";
			case "RSA_FTHHTOKEN": return "RSA飞天混合口令";
			case "RSA_FT2TOKEN": return "RSA飞天二代口令";
			default: return type;
		}
    },
	/**
	 * 格式化带毫秒时间
	 */
	dateFormat: function(date){
		return date.substring(0,date.indexOf("."));
	},
	
    /**
     * 货币单位变化类型
     */
    fmtCurrencyUnit: function(v){
        switch(String(v)){
			case 'ALL': return 'Af. ';
			case 'ATS': return 'Sch. ';
			case 'AUD': return '$A. ';
			case 'BEF': return 'Bi. ';
			case 'BGL': return 'Lev. ';
			case 'BUK': return 'K. ';
			case 'CAD': return 'Can.$ ';
			case 'CHF': return 'SFR. ';
			case 'CNY': return '￥';
			case 'CSK': return 'Kcs.. ';
			case 'DEM': return 'DM. ';
			case 'DZD': return 'AD. ';
			case 'ESP': return 'Pes. ';
			case 'EUR': return '€ ';
			case 'FIM': return '￡ ';
			case 'FRF': return 'F.F. ';
			case 'GBP': return '£ ';
			case 'GHC': return '￠';
			case 'GNS': return 'GS. ';
			case 'HKD': return 'HK$ ';
			case 'HUF': return 'FT. ';
			case 'IEP': return '￡.Ir. ';
			case 'INR': return 'Rs. ';
			case 'IQD': return 'ID ';
			case 'IRR': return 'RI. ';
			case 'ITL': return 'Lit. ';
			case 'JPY': return 'J￥';
			case 'KPW': return '₩ ';
			case 'KWD': return 'K.D. ';
			case 'LUF': return 'Lux.F. ';
			case 'MAD': return 'DH. ';
			case 'MOP': return 'P. ';
			case 'MTP': return '￡.M. ';
			case 'MYR': return 'M.$ ';
			case 'NAD': return 'N$ ';
			case 'NLG': return 'Gs. ';
			case 'NZD': return '$NZ. ';
			case 'PHP': return 'Phil.P. ';
			case 'PLZ': return 'ZL. ';
			case 'PTE': return 'ESC. ';
			case 'ROL': return 'L. ';
			case 'SGD': return 'S.$ ';
			case 'SLL': return 'Le. ';
			case 'THB': return '฿ ';
			case 'USD': return '$ ';
			case 'VND': return 'D. ';
			case 'ZAR': return 'R. ';
			case 'ZMK': return 'KW. ';
			default: return v;
        }
    },
    	
	/**
	 * 丹阳社保缴费类型
	 */
	fmtDyPayType : function(v){
		var insType = v;
		switch (v) {
		case "per.MCSocialSecurityDYLatePay":
			insType = "个人欠费补缴缴费";
			break;
		case "per.MCSocialSecurityDYNoticePay":
			insType = "通知单缴费";
			break;
		case "per.MCSocialSecurityDYPay":
			insType = "参保人员缴费";
			break;
		case "per.MCSocialSecurityDYUnitLatePay":
			insType = "单位欠费补缴缴费";
			break;
		
		}
		return insType;
	},
	
	/**
	 * 丹阳社保保险类型
	 */
	fmtDyInsType : function(v){
		var insType = v;
		switch (v) {
		case "10":
			insType = "企业养老";
			break;
		case "11":
			insType = "机关事业养老";
			break;
		case "15":
			insType = "居民养老";
			break;
		case "20":
			insType = "基本医疗";
			break;
		case "24":
			insType = "城乡居民医疗保险+补充";
			break;
		case "25":
			insType = "城乡居民医疗保险";
			break;
		case "30":
			insType = "工伤";
			break;
		case "40":
			insType = "生育";
			break;
		case "41":
			insType = "机关生育";
			break;
		case "50":
			insType = "失业";
			break;
		}
		return insType;
	},
	
    /**
     * 货币单位变化类型
     */
    fmtCurrencyUnit: function(v){
        switch(String(v)){
			case 'ALL': return 'Af. ';
			case 'ATS': return 'Sch. ';
			case 'AUD': return '$A. ';
			case 'BEF': return 'Bi. ';
			case 'BGL': return 'Lev. ';
			case 'BUK': return 'K. ';
			case 'CAD': return 'Can.$ ';
			case 'CHF': return 'SFR. ';
			case 'CNY': return '￥';
			case 'CSK': return 'Kcs.. ';
			case 'DEM': return 'DM. ';
			case 'DZD': return 'AD. ';
			case 'ESP': return 'Pes. ';
			case 'EUR': return '€ ';
			case 'FIM': return '￡ ';
			case 'FRF': return 'F.F. ';
			case 'GBP': return '£ ';
			case 'GHC': return '￠';
			case 'GNS': return 'GS. ';
			case 'HKD': return 'HK$ ';
			case 'HUF': return 'FT. ';
			case 'IEP': return '￡.Ir. ';
			case 'INR': return 'Rs. ';
			case 'IQD': return 'ID ';
			case 'IRR': return 'RI. ';
			case 'ITL': return 'Lit. ';
			case 'JPY': return 'J￥';
			case 'KPW': return '₩ ';
			case 'KWD': return 'K.D. ';
			case 'LUF': return 'Lux.F. ';
			case 'MAD': return 'DH. ';
			case 'MOP': return 'P. ';
			case 'MTP': return '￡.M. ';
			case 'MYR': return 'M.$ ';
			case 'NAD': return 'N$ ';
			case 'NLG': return 'Gs. ';
			case 'NZD': return '$NZ. ';
			case 'PHP': return 'Phil.P. ';
			case 'PLZ': return 'ZL. ';
			case 'PTE': return 'ESC. ';
			case 'ROL': return 'L. ';
			case 'SGD': return 'S.$ ';
			case 'SLL': return 'Le. ';
			case 'THB': return '฿ ';
			case 'USD': return '$ ';
			case 'VND': return 'D. ';
			case 'ZAR': return 'R. ';
			case 'ZMK': return 'KW. ';
			default: return v;
        }
    },
    
    /**
	 * 二维码扫码交易类型
	 */
	fmtQrcodeTransType : function(v){
		switch(String(v)){
			case '993004': return 'ATM取现';
			case '999007': return '被扫消费';
			case '998003': return '主扫消费';
			case '999013': return '转账';
			case '999008': return '退货';
			case '200004': return '条码缴费';
			default: return v;
		}
	},
	
	/**
	 * 格式化金额
     * @param v 原始金额  单位：分
     * @param decimals 小数点后保留位数（默认为2）
     * @param end 货币符号是否放在末尾（默认为false）
     * @param currencySign 货币符号（默认为￥）
     * @return {string}
	 */
    fmtAmtFen: function(v, decimals, end, currencySign) {
        var negativeSign = '', format = ',0';
        v = Fw.util.Format.unFmtAmt(v);
        if(!Fw.isNumber(v)){
            return v;
        }
        v = v - 0;
        if (v < 0) {
            v = -v;
            negativeSign = '-';
        }
        v = v/100;
        currencySign = Fw.isDefined(currencySign) ? currencySign || '' : Fw.util.Format.currencySign;
        decimals = Fw.isDefined(decimals) ? decimals : 2;
        format += (decimals > 0 ? '.' : '') + '0'.repeat(decimals);
        v = Fw.util.Format.fmtNumber(v, format);
        //
        if(end === true){
            return '{0}{1}{2}'.format(negativeSign, v, currencySign);
        }
        else{
            return '{0}{1}{2}'.format(negativeSign, currencySign, v);
        }
    },
    
    fmtChanlStr : function(data){
    	switch(data){
            case '01':
                data = "柜面";
                break;
            case '02':
                data = "网上银行";
                break;
            case '03':
                data = "手机银行";
                break;
            default:
    	}
    	return data;
    },
    
    fmtAcctStatus : function(data){
    	switch(data){
            case '0':
                data = "正常";
                break;
            case '1':
                data = "销户";
                break;
            case '2':
                data = "没收";
                break;
            case '3':
                data = "上缴";
                break;
            case '4':
                data = "挂入";
                break;
            case '5':
                data = "挂出";
                break;
            case '6':
                data = "冲正";
                break;
            case '7':
                data = "账户移除";
                break;
            default:
    	}
    	return data;
    },
    
    /**
	 * 学费缴费学年类型
	 */
	fmtSchoolYear : function(v){
		switch(String(v)){
			case 'T01': return '第一学期';
			case 'T02': return '第二学期';
			default: return v;
		}
	},
	
    /**
	 * 学费缴费费用类型
	 */
	fmtBusinessType : function(v){
		switch(String(v)){
			case 'B01': return '学费';
			case 'B02': return '书本费';
			case 'B03': return '伙食费';
			case 'B04': return '住宿费';
			case 'B05': return '其他';
			default: return v;
		}
	},
	
    /**
     * 处理undefined数据
     * return ''
     */
    fmtUndefinedData: function(data){
    	if(data=="" || data==null || data==undefined || data=="(null)" || data=="undefined"){
 		   return "";
 	   }else{
 		   return data;
 	   }
    },
    
    /**
	 * 绑定卡账户类别
	 */
	fmtBindCardCategory : function(v){
		switch(String(v)){
			case '1': return 'Ⅰ类账户';
			case '2': return 'Ⅱ类账户';
			case '3': return 'Ⅲ类账户';
			default: return v;
		}
	},
	
	/**
	 * 银行账户类型
	 */
	fmtBankAcType : function(v) {
		switch(String(v)){
			case '11': return '行内借记卡';
			case '12': return '行内活期';
			case '13': return '行内信用卡';
			case '21': return '他行借记卡';
			case '22': return '他行活期';
			case '23': return '他行信用卡';
			default: return v;
		}
	},
	
	/**
	 * 账户类别
	 */
	fmtAccLevel : function(v){
		switch(String(v)){
			case '1': return 'Ⅰ类';
			case '2': return 'Ⅱ类';
			case '3': return 'Ⅲ类';
			case '4': return 'Ⅱ类';
			case '5': return 'Ⅲ类';
			default: return v;
		}
	},
	
	/**
	 * 账户类别
	 */
	fmtAccLevelType : function(v){
		switch(String(v)){
			case '1': return 'Ⅰ类';
			case '2': return 'Ⅱ类';
			case '3': return 'Ⅲ类';
			case '4': return '电子账户Ⅱ类';
			case '5': return '电子账户Ⅲ类';
			default: return v;
		}
	},
	
	/**
     * 盐城东台保险类型
     * @param type
     * @returns {string}
     */
    fmtDongTaiSocialType: function(type){
        switch(String(type)){
            case '01': return '企业职工基本养老保险';
            case '02': return '城乡居民养老保险';
            case '03': return '机关事业养老保险';
            case '04': return '医疗保险';
            case '05': return '工伤保险';
            case '06': return '生育保险';
            case '07': return '失业保险';
            case '08': return '居民医疗保险';
            default: return type;
        }
    },
    /**
     * 盐城东台证件类型
     * @param type
     * @returns {string}
     */
    fmtDongTaiIdType: function(type){
    	switch(String(type)){
    	case 'A': return '身份证';
    	case 'B': return '军官证';
    	case 'G': return '港澳通行证';
    	case 'H': return '台湾通行证';
    	case 'I': return '外国人永久居留证';
    	case 'J': return '护照';
    	case 'P': return '组织机构代码';
    	default: return type;
    	}
    },
    
    /**
     * 盐城阜宁业务类型
     * @param type
     * @returns {string}
     */
    fmtFuNingBusType: function(type){
        switch(String(type)){
            case '000': return '默认业务类型';
            case '001': return '正常缴费';
            case '002': return '补缴';
            case '003': return '定期待遇发放';
            case '004': return '一次性支付';
            default: return type;
        }
    },
    /**
     * 兴化缴费档次类型
     * @param type
     * @returns {string}
     */
    fmtXingHuaPerAmtType: function(type){
    	switch(String(type)){
    	case '1': return '整百';
    	case '2': return '整数';
    	case '3': return '允许两位小数';
    	default: return type;
    	}
    },
    /**
     * 兴化银行状态
     * @param type
     * @returns {string}
     */
    fmtXingHuaBankType: function(type){
    	switch(String(type)){
    	case '0': return '失败';
    	case '1': return '成功';
    	case '2': return '超时';
    	case '3': return '已冲正成功';
    	default: return type;
    	}
    },
    
    /**
	 * 格式化钞汇鉴别
	 */
    fmtAmtFlag:function(type){
    	switch(String(type)){
    	case 'R': return '现汇';
    	case 'C': return '现钞';
    	default: return type;
    	}
    },
    
    /**
	 * 反格式化钞汇鉴别
	 */
    unfmtAmtFlag:function(type){
    	switch(String(type)){
    	case '现汇': return 'R';
    	case '现钞': return 'C';
    	default: return type;
    	}
    },
    
    /**
     * 转换证件类型
     */
    transforIdType:function(idType){
		switch (idType) {
		case "P00":	//居民身份证
			return "01";
			break;
		case "P23":
			return "04";
			break;
		case "P24":
			return "05";
			break;
		case "P06":	//
			return "06";
			break;
		case "P13":
			return "07";
			break;
		default:
			break;
		}
    },
    
    /**
     * 结售汇存期格式化
     */
	fmtTermCode : function(v) {
	  switch (String(v)) {
		case 'D0':
			return '活期';
		case 'N1':
			return '一天';
		case 'N7':
			return '七天';
		case 'M1':
			return '一个月';
		case 'M3':
			return '三个月';
		case 'M6':
			return '六个月';
		case 'Y1':
			return '一年';
		case 'Y2':
			return '二年';
		case 'Y3':
			return '三年';
		case 'Y5':
			return '五年';
		default: return v;
		}
	},
    
    
	/**
	 * json数据合并
	 */
	jsonAddJson : function(jsonObject,jsonObjectAdd){
		if(!Fw.isObject(jsonObject) || !Fw.isObject(jsonObjectAdd)){
			return false;	
		}
		var jsonObjectStr = Fw.encode(jsonObject);
		var jsonObjectAddStr = Fw.encode(jsonObjectAdd);
		return Fw.decode(jsonObjectStr.substring(0,jsonObjectStr.length-1) +','+ jsonObjectAddStr.substr(1));
	},
	
	/**
	 * 分离拼接数据
	 * str：拼接数据
	 * v：分隔符
	 * i：分隔后的数组索引
	 */
	fmtSpliceData: function(str, v, i){
		v=String(v);
		return str.split(v)[i];
	},
	
	/**
	 * 大额存单结息方式
	 */
	fmtTdClsdIntcTyp : function(v) {
		switch(String(v)){
			case '0': return '到期付息';
			case '1': return '按月结息';
			case '2': return '按季结息';
			default: return v;
		}
	},
	/**
     * 百分比格式化(100倍)
     * @param v 原始数字
     * @param format 格式/小数点后保留位数（默认为2）
     * @return {string}
     */
    fmtPercent2: function(v, format) {
        return (Fw.util.Format.fmtNumber(v,'0.00')/100).toFixed(2) + '%';
    },
    /**
     * 网上支付明细查询映射
     */
    formatAA: function(v) {
		switch(v) {
			case "D100":
				v = "普通实时借记";
				break;
			case "D101":
				v = "个人账户通兑";
				break;
			case "D103":
				v = "消费";
				break;
			case "D104":
				v = "预授权完成";
				break;
			case "D105":
				v = "手工预授权完成";
				break;
			case "D200":
				v = "协议付款";
				break;
			case "D201":
				v = "贷款还款";
				break;
			case "D202":
				v = "助农取款";
				break;
			case "D203":
				v = "网关支付";
				break;
			case "D204":
				v = "委托付款";
				break;
			case "D900":
				v = "手工预授权完成";
				break;
			case "C100":
				v = "普通实时贷记";
				break;        	 	 
			case "C101":
				v = "实时代付";
				break;
			case "C102":
				v = "个人账户通存";
				break;
			case "C103":
				v = "对公账户通存";
				break;
			case "C104":
				v = "IC卡圈存";
				break;
			case "C105":
				v = "IC卡充值";
				break;
			case "C106":
				v = "跨境汇款";
				break;
			case "C200":
				v = "投资理财";
				break;
			case "C201":
				v = "缴费";
				break;
			case "C202":
				v = "慈善捐款";
				break;
			case "C203":
				v = "贷款还款";
				break;
			case "C300":
				v = "退货";
				break;
			case "C301":
				v = "线上消费退货";
				break;
			case "C302":
				v = "手工消费退货";
				break;
			case "C400":
				v = "薪金报酬";
				break;
			case "C900":
				v = "其他";
				break;
			case "00":
				v = "支付宝";
				break;
			case "01":
				v = "财付通";
				break;
			case "02":
				v = "京东支付";
				break;
			case "03":
				v = "百付宝";
				break;
			case "04":
				v = "天翼支付";
				break;
			case "05":
				v = "易宝支付";
				break;
			case "06":
				v = "拉卡拉支付";
				break;
			case "07":
				v = "中国移动";
				break;
			case "08":
				v = "苏宁";
				break;
			case "13":
				v = "快钱";
				break;
			case "15":
				v = "美团-钱袋宝";
				break;
			case "16":
				v = "滴滴一九付";
				break;
			case "99":
				v = "全部";
				break;
			case "S":
				v = "成功";
				break;
			case "F":
				v = "失败";
				break;
			case "U":
				v = "超时";
				break;
			default:
		}
		return v;
    },
    
    /**
     * 姓名格式化、 掩码户名
     * @param  name {string} 姓名
     * @return {string}
     * 掩码户名掩码规则：客户户名为两个字的，显示第二个字，首字显示为“*”；客户户名为三个字及三个字以上的，显示首、尾两个字，其他字显示为“*”
     */
    fmtName: function(name){
        if (Fw.isEmpty(name)) {
            return '';
        }
        if(name.length == 1){
        	return name;
        }else if(name.length == 2){
        	return '*' + name.substring(1, 2);
        }else{
        	var mask = '';
        	for(var i=0; i < name.length-2; i++){
        		mask += "*";
        	}
        	return name.substring(0, 1) + mask + name.substring(name.length-1, name.length);
        }
    },
    
    /**
     * 他行收款账户类型
     */
	fmtAcType: function(v){
		switch(v){
		  case '0': return "对公客户";
		  case '1': return "借记卡";
		  case '2': return "个人存折";
		  case '3': return "贷记卡";
		}
	},
    
    /**
     * 他行收款业务类型
     */
    fmtRevBusiType: function(v){
    	switch(v){
    		case  "D200": return "实时代收";
    		case  "D201": return "贷款还款";
    		case  "D202": return "其他";
    		case  "D203": return "收款";
    		case  "D204": return "消费";
    		case  "D205": return "慈善捐款";
    		case  "D206": return "投资理财";
		}
    },
    
    /**
	 * 他行收款业务种类
	 * */
	fmtRevBusiKind: function(v){
	    switch(v){
	    	//实时代收
            case "00100":  return "电费 ";
        	case "00200":  return "水暖费 ";
        	case "00300":  return "煤气费 ";
            case "00400":  return "电话费 ";
        	case "00500":  return "通讯费 ";
        	case "00600":  return "保险费 ";
            case "00700":  return "房屋管理费 ";
            case "00800":  return "代理服务费 ";
        	case "00900":  return "学教费 ";
        	case "01000":  return "有线电视费 ";
            case "01100":  return "企业管理费用 ";
        	case "01200":  return "薪金报酬 ";
        	//贷款还款
        	case "02025":  return "贷款还款房贷类 ";
            case "02026":  return "贷款还款车贷类 ";
            case "02027":  return "贷款还款信用卡类 ";
            //其他
        	case "09001":  return "其他 ";
        	//收款
        	case "02029":  return "收款 ";
        	//消费
        	case "02030":  return "服装类消费 ";
        	case "02031":  return "饰品类消费 ";
        	case "02032":  return "家居类消费 ";
        	case "02033":  return "生活类消费 ";
        	case "02034":  return "食品类消费 ";
        	case "02035":  return "虚拟类消费 ";
        	case "02036":  return "机票类消费 ";
        	case "02037":  return "旅游类消费 ";
        	case "02038":  return "美容类消费 ";
        	case "02039":  return "数码类消费 ";
        	case "02040":  return "电器类消费 ";
        	case "02041":  return "文体类消费 ";
        	case "02042":  return "酒店类消费 ";
        	case "02043":  return "其他类消费 ";
        	//慈善捐款
        	case "02024":  return "慈善捐款 ";
        	//投资理财
        	case "02002":  return "理财股票类 ";
        	case "02003":  return "理财基金类 ";
        	case "02004":  return "理财保险类 ";
        	case "02005":  return "理财彩票类 ";
        	case "02006":  return "理财黄金类 ";
        	case "02007":  return "理财债券类 ";
        	case "02008":  return "理财其他类 ";
		}
	},
	
	/**
     * 微信支付宝交易状态
     */
    fmtVAStatus: function(v){
    	switch(v){
    		case  "0": return "成功";
    		case  "1": return "失败";
    		case  "2": return "等待对账";
    		case  "3": return "全部";
    		case  "S": return "成功";
    		case  "F": return "失败";
    		case  "U": return "等待对账";
		}
    },
    /**
     * 微信支付宝商户类型
     */
    fmtVAMchType: function(v){
    	switch(v){
    		case  "A": return "支付宝";
    		case  "V": return "微信";
		}
    },
    /**
     * 微信支付宝对账状态
     */
    fmtVAClrStatus: function(v){
    	switch(v){
    		case  "0": return "未对账";
    		case  "9": return "已对账";
		}
    },
    
};

/**
 * 注册到juicer
 */
Fw.each(Fw.util.Format, function(prop, fun){
    if(Fw.isFunction(fun)){
        juicer.register(prop, fun);
    }
});

