/**
 * @name Fw.util.JSON
 * @class JSON工具类
 * @example : <br>
 * 			json --> string 
 *          Fw.encode({...});<br>
 *          string --> json
 *          Fw.decode('{...}');
 */
Fw.util.JSON = (new(function() {
    var me = this,
    encodingFunction,
    decodingFunction,
    useNative = null,
    useHasOwn = !! {}.hasOwnProperty,
    isNative = function() {
        if (useNative === null) {
            useNative = Fw.USE_NATIVE_JSON && window.JSON && JSON.toString() == '[object JSON]';
        }
        return useNative;
    },
    pad = function(n) {
        return n < 10 ? "0" + n : n;
    },
    doDecode = function(json) {
        return eval("(" + json + ')');
    },
    doEncode = function(o, newline) {
        if (o === null || o === undefined) {
            return "null";
        } else if (Fw.isDate(o)) {
            return Fw.util.JSON.encodeDate(o);
        } else if (Fw.isString(o)) {
            return Fw.util.JSON.encodeString(o);
        } else if (typeof o == "number") {
            return isFinite(o) ? String(o) : "null";
        } else if (Fw.isBoolean(o)) {
            return String(o);
        }
        else if (o.toJSON) {
            return o.toJSON();
        } else if (Fw.isArray(o)) {
            return encodeArray(o, newline);
        } else if (Fw.isObject(o)) {
            return encodeObject(o, newline);
        } else if (typeof o === "function") {
            return "null";
        }
        return 'undefined';
    },
    m = {
        "\b": '\\b',
        "\t": '\\t',
        "\n": '\\n',
        "\f": '\\f',
        "\r": '\\r',
        '"': '\\"',
        "\\": '\\\\',
        '\x0b': '\\u000b' //ie doesn't handle \v
    },
    charToReplace = /[\\\"\x00-\x1f\x7f-\uffff]/g,
    encodeString = function(s) {
        return '"' + s.replace(charToReplace, function(a) {
            var c = m[a];
            return typeof c === 'string' ? c : '\\u' + ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
        }) + '"';
    },
    encodeArray = function(o, newline) {
        var a = ["[", ""],
            len = o.length,
            i;
        for (i = 0; i < len; i += 1) {
            a.push(Fw.util.JSON.encodeValue(o[i]), ',');
        }
        a[a.length - 1] = ']';
        return a.join("");
    },
    encodeObject = function(o, newline) {
        var a = ["{", ""],
            i, val;
        for (i in o) {
            val = o[i];
            if (!useHasOwn || o.hasOwnProperty(i)) {
                if (typeof val === 'function' || val === undefined) {
                    continue;
                }
                a.push(Fw.util.JSON.encodeValue(i), ":", Fw.util.JSON.encodeValue(val), ',');
                
            }
        }
        a[a.length - 1] = '}';
        return a.join("");
    };
    me.encodeString = encodeString;
    me.encodeValue = doEncode;
    me.encodeDate = function(o, returnString) {
        var val = o.getFullYear() + "-"
        + pad(o.getMonth() + 1) + "-"
        + pad(o.getDate()) + "T"
        + pad(o.getHours()) + ":"
        + pad(o.getMinutes()) + ":"
        + pad(o.getSeconds());
        //
        if(returnString !== false){
            val = '"' + val + '"';
        }
        return val;
    };
    me.encode = function(o) {
        if (!encodingFunction) {
            encodingFunction = isNative() ? JSON.stringify : me.encodeValue;
        }
        return encodingFunction(o);
    };
    me.decode = function(json, safe) {
        if (!decodingFunction) {
            decodingFunction = isNative() ? JSON.parse : doDecode;
        }
        try {
            return decodingFunction(json);
        } catch (e) {
            if (safe === true) {
                return null;
            }
        }
    };
})());
//
Fw.encode = Fw.util.JSON.encode;
Fw.decode = Fw.util.JSON.decode;