/**
 * 蓝牙key
 */
Fw.define(function(require){
	'require:nomunge,exports:nomunge,module:nomunge';
	
	var BluetoothKey = Fw.extend(Fw.Component, {
		/**
	     * 蓝牙key签名加密
	     * @param clearData 签名明文参数{}
	     * 		  session 会话
	     *        callback 蓝牙key签名成功回调结束后返回继续交易方法
	     */
		encryptByBluetoothKey: function(clearData, session, callback){
			this.signatureCleartext = this.fmtBluetoothKeySignData(clearData); //签名明文
			var encryptKey = this.getRandomNums(24);
	    	var params = {
	    			signatureCleartext: this.signatureCleartext, //蓝牙key签名明文xml
	    			encryptKey: encryptKey, //蓝牙key加密随机密钥
	    			deviceNo: session.getOtpNo(), //蓝牙key口令号
	    			otpType: session.getOtpType(), //蓝牙key口令类型
	    			callBack: "Fw.BluetoothKey.setSignDataOfBluetoothKey" //蓝牙key签名成功后回调方法
	    	}
	    	this.bluetoothKeyCallback = callback; //返回继续交易方法
        	Fw.device.Device.toSignDataOfBluetoothKey(Fw.encode(params)); //调用客户端蓝牙key签名方法
		},
		
		/**
		 * 蓝牙key签名后客户端回调方法
		 * @return {certInfo, //证书
		 * 		    SignRet //签名密文}
		 */
		setSignDataOfBluetoothKey: function(data){
			if(data){
				if("string" == typeof(data)){
					data = Fw.decode(data);
				}
				var params = {};
				params.signCert = data.certData || ""; //证书
				params.signatureCleartext = data.signatureCleartext || ""; //签名明文
				params.signatureCiphertext = data.signRet || ""; //签名密文
				this.bluetoothKeyCallback(params); //返回继续交易
			}else{
				Fw.Msg.error("蓝牙key加密出错：返回参数为空");
			}
		},
			
	    /**
	     * 蓝牙key数据签名格式化，组装为xml格式
	     * @param Json {
	     * 	tradeName：交易名称
	     * 	tradeId：交易id
	     *  labelData：标签信息（数组）[
	     *  	labelName：标签字段名
	     *  	labelDesc：标签名称（若无此参数，蓝牙key上不显示此标签；若有此参数，蓝牙key上回显标签信息）
	     *  	labelValue：标签value值
	     *  ]
	     * }
	     * @return xml（
	     * 	<?xml version="1.0" encoding="utf-8"?>
	     * 		<SignData>
	     * 			<TradeType name="tradeName">tradeId</TradeType>
	     * 			<Timestamp>new Date().getTime()</Timestamp>
	     * 			<Fields>
	     * 				<labelName name="labelDesc">labelValue</labelName>
	     * 			</Fields>
	     * 		</SignData>
	     * ）
	     */
		fmtBluetoothKeySignData: function(params){
			var signXml = '<?xml version="1.0" encoding="utf-8"?>'
					+ '<SignData>'
						+ '<TradeType name="'+params.tradeName+'">'+params.tradeId+'</TradeType>'
						+ '<Timestamp>'+ new Date().getTime() +'</Timestamp>'
						+ '<Fields>';
							var data = params.labelData;
							var s=''; //Fields
							//请求参数标签
							for(i=0; i<data.length; i++){
								if(!Fw.isEmpty(data[i].labelName)){ //标签名
									s = s + '<'+data[i].labelName;
									if(!Fw.isEmpty(data[i].labelDesc)){ //标签描述
										s = s + ' name="'+ data[i].labelDesc + '"'
									}
									s = s + '>'+data[i].labelValue+'</'+data[i].labelName+'>' //标签值	
								}
							}
						signXml = signXml + s + '</Fields>'
					+ '</SignData>';
			return signXml;
	    },
		
		/**
		 * 获得固定长度随机数字
		 */
		getRandomNums: function(length){
			var randomNums = "";
			for(var i=0; i<length; i++){
				randomNums += Math.floor(Math.random()*10);
			}
			return randomNums;
		},
		
	});
	
	Fw.Logger.debug('加载蓝牙key');
	Fw.BluetoothKey = new BluetoothKey();
	Fw.Logger.debug('初始化蓝牙key');
	
});