/**
 * 表单
 */
Fw.Form = Fw.extend(Fw.Component, {
    /**
     * 基本样式
     */
    cls: 'x-form',
    /**
     * 是否激活重置事件
     * 单击输入框时，自动重置为初始值（仅处理文本输入框、多行文本输入框）
     */
    resettable: false,
    /**
     * 校验提示方式
     * tips：浮动层
     * msg：弹出框
     */
    msgTarget: 'tips',
    /**
     * 初始化事件
     */
    initEvents: function(){
        //激活重置事件（不含单选、多选、下拉框）
        if(this.resettable){
            //全局
            this.el.find('input:not([type=checkbox]):not([type=radio]),textarea').on('focus', Fw.bind(this.doReset, this));
        }
        
        //表单值改变时触发（包含文本输入框、多行文本输入框、单选、多选、下拉框）
        this.el.find('input,select,textarea').on('change', Fw.bind(this.onChangeValue, this));
        
        //sm history.back() 金额更新原始值、格式化显示值
        var that = this;
        this.el.find('input[data-xtype^="keyboard,AMT"],textarea[data-xtype^="keyboard,AMT"]').each(function(e){
        	var name = $(this).attr('name');
        	var displayValue = $(this).val();
        	//
        	if(displayValue) {
        		//
            	that.fields[name] = null;
            	//
        		that.onChangeValue(name);
        	}
        });
        //sm history.back() 金额更新原始值、格式化显示值
    },
    /**
     * 初始化HTML模版
     */
    initTpl: function(){
        this.el.attr('onsubmit', 'return false;').addClass(this.cls);//.attr('action', '#');
        //sm 防止默认聚焦 调起系统键盘、聚焦时placeholder默认没有
        this.el.find('input:not([type=checkbox]):not([type=radio]),textarea').blur();
        //sm 防止默认聚焦 调起系统键盘、聚焦时placeholder默认没有
        this.fields = {};
        this.defaultValues = this.el.serializeObject();//不含displayField和未选中的checkbox、radio
        
        //未选中的checkbox、radio
        this.el.find('input[type=checkbox],input[type=radio]').each(Fw.bind(function(index, field){
            if(!field.checked && !Fw.isDefined(this.defaultValues[field.getAttribute('name')])){
                this.defaultValues[field.getAttribute('name')] = null;
            }
        }, this));
        
        //displayField
        this.el.find('.x-display-field[name]').each(Fw.bind(function(index, field){
            if(!Fw.isDefined(this.defaultValues[field.getAttribute('name')])){
                this.defaultValues[field.getAttribute('name')] = field.innerHTML || null;
            }
        }, this));
        
        //
        this.oldValues = {};
        this.values = {};
    },
    /**
     * 表单值改变时触发（包含文本输入框、多行文本输入框、单选、多选、下拉框）
     */
    onChangeValue: function(e){
        var name = e.target ? e.target.name : e;
        var field = this.getField(name);
        if(field){
            var displayValue = this.getValues(name, false);
            var rawValue = this.getValues(name);
    
            //sm 调用客户端键盘输入最后一次的值不会进行格式
            var config = field.data();
            var format = config['format'];
            //sm 调用客户端键盘输入最后一次的值不会进行格式
            
            //
//            if(Fw.isEqual(displayValue, rawValue)){
            if(Fw.isEqual(displayValue, rawValue) && Fw.isEmpty(format) ){//sm
                //更新原始值
                this.values[name] = rawValue;
            }
            else{
                //反格式化
                rawValue = this.formatValue(name, displayValue, true);
    
                //更新原始值、格式化显示值
                this.setValues(name, rawValue, true);
            }
            
            //
            var oldValue = this.oldValues[name] || null;
            this.oldValues[name] = rawValue;
            this.fireEvent('change', name, rawValue, oldValue, field, this);
            
            //sm htc下拉文本不变(HTC E9pw,Android 5.0,HTC Sense 7.0)
            var tarName = $('[name="'+name+'"]')[0] ? $('[name="'+name+'"]')[0].tagName : '';
            if(Fw.isAndroid && tarName === 'SELECT'){//
            	$('[name="'+name+'"]').blur();
            }
            //sm htc下拉文本不变
        }
    },
    /**
     * 表单输入框重置
     */
    doReset: function(e){
        var name = e.target.name;
        var rawValue = this.getValues(name);
        if(!Fw.isEqual(rawValue, this.defaultValues[name])){
            Fw.defer(this.reset, 100, this, [name]);
        }
    },
    /**
     * 设置表单值
     */
    setValues: function(){
        var argsLen = arguments.length;
        if(argsLen > 0){
            var field = null, name, values;
            //多个
            if(argsLen === 1){
                this.values = values = arguments[0];//values
                if(Fw.isObject(values)){
                    Fw.each(values, this.setValues, this);
                }
            }
            //单个
            else{
                name = arguments[0];//name
                values = arguments[1] || '';//value
                field = this.getField(name);
                //
                if(field){
                    var suppressEvent = arguments[2] === true;
                    var type = field[0].type.toLocaleUpperCase();
                    var nodeName = field[0].nodeName.toLocaleUpperCase();
                    var _checked, _value;
                    
                    //
                    if(nodeName === 'INPUT' && ['CHECKBOX', 'RADIO'].indexOf(type) > -1){
                        if(!Fw.isArray(values)){
                            values = [values];
                        }
                    
                        //
                        this.values[name] = values;
                        
                        //
                        field.each(function(index){
                            _checked = false;
                            _value = this.getAttribute('value');
                            Fw.each(values, function(v){
                                _checked = String(v) === _value;
                                if(_checked){
                                    return false;
                                }
                            });
                            $(this).prop('checked', _checked);
                        });
                        if(!suppressEvent){
                            field.eq(0).trigger('change');
                        }
                    }
                    else{
                        //
                        if(Fw.isString(values)){
                            values = values.trim();
                        }
                        
                        //
                        this.values[name] = values;
                        this.setDisplayValue(name, values);
                        
                        //!suppressEvent
                        if(!suppressEvent){
                            if(field[0].isDisplayField){
                                this.onChangeValue(name);
                            }
                            else{
                                field.trigger('change');
                            }
                        }
                    }
                }
            }
        }
    },
    /**
     * 返回表单值
     */
    getValues: function(name, rawValue){
        //单个
        if(name){
            var field = this.getField(name), value = null;
            if(field){
                var type = field[0].type.toLocaleUpperCase();
                var nodeName = field[0].nodeName.toLocaleUpperCase();
                //
                if(nodeName === 'INPUT' && ['CHECKBOX', 'RADIO'].indexOf(type) > -1){
                    value = [];
                    field.each(function(index){
                        if(this.checked){
                            value.push($(this).val());
                        }
                    });
                    if(value.length < 2){
                        value = value.length === 0 ? null : value[0];
                    }
                }
                else{
                    if(nodeName === 'SELECT' || rawValue === false){
                        value = field[field[0].isDisplayField ? 'html' : 'val']();
                    }
                    else{
                        value = this.values[name];
                    }
                }
            }
            return value || null;
        }
        //整个表单
        else{
            return Fw.apply({}, this.values, this.defaultValues);
        }
    },
    /**
     * 设置显示值
     * private
     */
    setDisplayValue: function(name, value){
        var field = this.getField(name);
        if(field){
            if(field[0].nodeName.toLocaleUpperCase() !== 'SELECT'){
                //格式化
                value = this.formatValue(name, value);
            }
            //
            field[field[0].isDisplayField ? 'html' : 'val'](value);
        }
    },
    /**
     * sm 重新设置输入框allowblank验证信息
     */
    resetAllowblank: function(name,boolVal){
    	//
    	var field = this.fields[name];
    	//
    	if(!field){
    		field = this.getField(name);
    	}
    	//
    	if(field){
    		field.data()['allowblank'] = boolVal;
    	}
    },
    //sm
    /**
     * 返回Field
     */
    getField: function(name){
        var field = this.fields[name];
        if(!field){
            field = this.el.find('input[name=' + name + '],select[name=' + name + '],textarea[name=' + name + '],.x-display-field[name=' + name + ']');
            field = field[0] ? field : null;
            if(field && field.hasClass('x-display-field')){
                field[0].type = '';
                field[0].isDisplayField = true;///
            }
            this.fields[name] = field;
        }
        return field;
    },
    /**
     * 返回FieldLable
     */
    getFieldLable: function(name){
        var field = this.getField(name), lable = null;
        if(field){
            lable = field.data('label') || name || null;
        }
        return lable;
    },
    /**
     * 验证表单输入值是否合法
     */
    isValid: function(name, value){
    	if($("[name="+name+"]").is(':hidden')){
    		return true;
    	};
        if(name){
            var field = this.getField(name), lable = this.getFieldLable(name);
            //sm
            if(Fw.isEmpty(field)){
            	return true;
            }
            //sm
            var config = field.data();           
            if(config['val']==false){
            	return true;
            }
            //sm 如果
//            if(arguments.length === 1){
            //sm 如果是密码键盘，获取密码长度验证
            if(arguments.length === 2){
//                value = this.getValues(name);
                value = this.getValues(name,Fw.isEmpty(config['xtype']));//密码键盘长度获取
            }
            //sm
            
            var isBlank = Fw.isEmpty(String(value).trim()), lableValue;
            this.errors = {
                msgTarget: config['msgtarget'] || null
            };

            //为空
            if(isBlank){
                //是否允许为空
                var allowBlank = config['allowblank'];
                //如果明确指定允许为空，则不做后续校验
                if(allowBlank === true){
                    return true;
                }
                //如果为false，则校验是否为空
                else if(allowBlank === false){
                    var isSelect = field[0].nodeName.toLocaleUpperCase() === 'SELECT' || ['CHECKBOX', 'RADIO'].indexOf(field[0].type.toLocaleUpperCase()) > -1;
                    this.errors.message = (config['blanktext'] || Fw.Form[isSelect ? 'unSelectedText' : 'blankText']).format(lable);
                    return false;
                }
            }
            //非法字符校验
//            else if(Fw.util.VTypes.special(value)){
//                this.errors.message = (config['specialtext'] || Fw.util.VTypes['specialText']).format(lable);
//                return false;
//            }
            
            //自定义正则校验
            var regex = config['regex'];
            if(!Fw.isEmpty(regex) && !(new RegExp(regex)).test(value)){
                this.errors.message = (config['regextext'] || Fw.Form['invalidText']).format(lable);
                return false;
            }
            
            //值类型校验（对应Fw.util.VTypes.*）
            var vtype = config['vtype'];
            if(!Fw.isEmpty(vtype) && !Fw.util.VTypes[vtype](value)){
                this.errors.message = (config[vtype + 'text'] || Fw.util.VTypes[vtype + 'Text'] || Fw.util.VTypes['invalidText']).format(lable);
                return false;
            }
            
            //字符型校验
            var minLength = config['minlength'] * 1, maxLength = config['maxlength'] * 1, fixedLength = config['fixedlength'] * 1;
//            var valueLength = isBlank ? 0 : String(value).length;
            //sm 是否进行对取到的值去除空格后判断
            var trimeFlag = config['trim'];
            var valueLength = isBlank ? 0 : (trimeFlag ? String(value).trim(true).length : String(value).length);
            //sm 是否进行对取到的值去除空格后判断
            
            //固定长度
            if(Fw.isNumber(fixedLength)){
                if(valueLength !== fixedLength){
                    this.errors.message = (config['fixedlengthtext'] || Fw.Form['fixedLengthText']).format(lable, fixedLength);
                    return false;
                }
            }
            else{
                //最小长度
                if(Fw.isNumber(minLength) && valueLength < minLength){
                    this.errors.message = (config['minlengthtext'] || Fw.Form['minLengthText']).format(lable, minLength);
                    return false;
                }
                //最大长度
                if(Fw.isNumber(maxLength) && valueLength > maxLength){
                    this.errors.message = (config['maxlengthtext'] || Fw.Form['maxLengthText']).format(lable, maxLength);
                    return false;
                }
            }
            
            //最小值
            var minValue = config['minvalue'];
            if(!Fw.isEmpty(minValue) && !isBlank){
                //数值型
                if(Fw.isNumber(minValue * 1)){
                    value *= 1;
                    lableValue = minValue *= 1;
                    if(!Fw.isNumber(value)){
                        this.errors.message = (config['invalidtext'] || Fw.Form['invalidText']).format(lable, lableValue);
                        return false;
                    }
                }
                //日期型
                if(Fw.util.VTypes.date(minValue)){
                    value = Fw.util.Format.toDate(value);
                    minValue = Fw.util.Format.toDate(minValue);
                    lableValue = Fw.util.Format.fmtDate(minValue);
                }
                //校验
                if(value < minValue){
                    this.errors.message = (config['minvaluetext'] || Fw.Form['minValueText']).format(lable, lableValue);
                    return false;
                }
            }
            
            //最大值
            var maxValue = config['maxvalue'];
            if(!Fw.isEmpty(maxValue) && !isBlank){
                //数值型
                if(Fw.isNumber(maxValue * 1)){
                    value *= 1;
                    lableValue = maxValue *= 1;
                    if(!Fw.isNumber(value)){
                        this.errors.message = (config['invalidtext'] || Fw.Form['invalidText']).format(lable, lableValue);
                        return false;
                    }
                }
                //日期型
                if(Fw.util.VTypes.date(maxValue)){
                    value = Fw.util.Format.toDate(value);
                    maxValue = Fw.util.Format.toDate(maxValue);
                    lableValue = Fw.util.Format.fmtDate(maxValue);
                }
                //校验
                if(value > maxValue){
                    this.errors.message = (config['maxvaluetext'] || Fw.Form['maxValueText']).format(lable, lableValue);
                    return false;
                }
            }
            
            //
            return true;
        }
        else{
            var isValid = false;
            Fw.each(this.values, function(prop, value){
                isValid = this.isValid(prop, value);
                return isValid;
            }, this);
            return isValid;
        }
    },
    /**
     * 格式化（反格式化）
     */
    formatValue: function(name, value, inverse){
        var field = this.getField(name), args;
        if(field){
            if(args = field.data('format')){
                args = String(args).trim().split(',');
                if(args.length > 0){
                    var fmtName = args[0], formater;
                    args = [value].concat(args.splice(1));
                    
                    //反格式化
                    if(inverse){
                        fmtName = 'un' + fmtName.capitalize();
                        if(Fw.isFunction(formater = (Fw.util.Format[fmtName] || window[fmtName]))){
                            value = Fw.call(formater, null, args);
                        }
                        else{
                            Fw.Logger.warn('反格式化方法[%s%s]不存在', 'Fw.util.Format.', fmtName);
                        }
                    }
                    //格式化
                    else{
                        if(Fw.isFunction(formater = (Fw.util.Format[fmtName] || window[fmtName]))){
                            value = Fw.call(formater, null, args);
                        }
                        else{
                            Fw.Logger.warn('格式化方法[%s%s]不存在', 'Fw.util.Format.', fmtName);
                        }
                    }
                }
            }
            else{
                var rawValue = field.data('rawvalue'), displayValue = field.data('displayvalue');

                //反格式化
                if(inverse && Fw.isDefined(rawValue)){
                    field.removeData('rawvalue');
                    value = rawValue;
                }
                //格式化
                else if(Fw.isDefined(displayValue)){
                    field.removeData('displayvalue');
                    value = displayValue;
                }
            }
        }
        return value;
    },
    /**
     * 表单提交
     */
    submit: function(){
    	//sm 系统键盘失焦触发change
        $('input:not([type=checkbox]):not([type=radio]),textarea,select').blur();
        //sm 系统键盘失焦触发change
        Fw.defer(function(){
            this.values = this.getValues();
            if(this.fireEvent('beforesubmit', this.values, this) !== false){
                if(this.isValid()){
                    this.fireEvent('submit', this.values, this);
                }
                else{
                    var msgTarget = (this.errors.msgTarget || this.msgTarget).capitalize();
                    try{
                        Fw[msgTarget].info(this.errors.message);
                    }
                    catch(e){
                        Fw.Logger.error('校验消息提示接口[Fw.%s.info]不存在%o', msgTarget, e);
                    }
                }
            }
        }, 100, this);
    },
    /**
     * 表单重置
     */
    reset: function(name){
        if(name){
            //单个输入框重置
            this.setValues(name, this.defaultValues[name]);
        }
        else{
            //表单重置
            //this.el[0].reset();
            this.setValues(Fw.apply({}, this.defaultValues));
        }
    }
});

//
Fw.Logger.debug('加载表单组件');