/**
 * @name Fw.ListView
 * @class 列表组件
 * @constructor
 * @example
 * @return
 */
Fw.ListView = Fw.extend(Fw.Component, {
    //
    cls: 'x-list',
    groupCls: 'x-list-group',
    groupHeaderBaseCls: 'x-list-group-header',
    groupHeaderCls: 'ui-border-t',
    itemBaseCls: 'x-list-item',
    itemCls: 'ui-border-t',
    pressedCls: 'ui-list-active',
    //向右箭头（默认false）
    disclosure: false,
    //分组字段（默认null）
    // groupField: null,
    //分组字段（通过自定义函数返回分组属性）
    // groupField: function(values){
        // return Fw.util.Format.date(values.lastModiDate, 'yyyy年MM');
    // },
    //分组显示字段（默认groupField）
    // groupDisplayField: null,
    //设置下拉刷新组件的相关属性
    pullRefresh: false,
    // pullRefresh: {
        // pullText: '下拉开始刷新',
        // releaseRefreshText: '松开立即刷新', 
        // loadingText: '正在刷新...'
    // },
    //设置列表分页的相关属性
    // listPaging: {
        // loadMoreText: '加载更多...',
        // loadingText: '正在加载...',
        // noMoreText:'没有更多数据'
    // },
    /**
     * 初始化组件
     */
    initComponent : function() {
        Fw.ListView.superclass.initComponent.call(this);

        //
        if(this.ajax){
            //新建ajax工具类
            this.ajax = new Fw.Ajax(this.ajax);
            //初始化下拉刷新
            if(this.pullRefresh != false){
                this.pullRefresh = new Fw.PullRefresh(Fw.apply({
                    owner: this
                }, this.pullRefresh));
            }
            //初始化分页
            if(this.ajax.page !== false || this.listPaging){
                //列表分页
                this.listPaging = new Fw.ListPaging(Fw.apply({
                    owner: this
                }, this.listPaging));
            }
        }
    },
    /**
     * 初始化事件
     */
    initEvents : function() {
        //数据加载完成后触发
        this.ajax && this.ajax.on('query', this.onLoadData, this);
        //手指按下时触发
        this.el.on('touchstart', 'li.' + this.itemBaseCls, Fw.bind(this.onItemTouchStart, this));
        //手指滑动时触发
        this.el.on('touchmove', 'li.' + this.itemBaseCls, Fw.bind(this.onItemTouchMove, this));
        //手指抬起时触发
        this.el.on('touchend', 'li.' + this.itemBaseCls, Fw.bind(this.onItemTouchEnd, this));
    },
    /**
     * 初始化HTML模版
     */
    initTpl : function() {
        //基本样式
        this.baseCls = this.cls;
        
        //分组样式
        if(this.groupField){
            if(!this.groupDisplayField){
                this.groupDisplayField = this.groupField;
            }
            if(!this.groupHeaderTpl){
                this.groupHeaderTpl = ['${item.', this.groupDisplayField, '}'];
            }
            this.baseCls += ' ' + this.groupCls;
            this.initGroupHeaderTpl();
            this.allowGroup = true;
        }
        
        //向右箭头
        if(this.disclosure){
            this.baseCls += ' ui-list-link';
            if(Fw.isString(this.disclosure)){
                this.baseCls += ' ' + this.disclosure;
            }
        }
        
        //按压样式
        if(!Fw.isEmpty(this.pressedCls)){
            this.baseCls += ' ' + this.pressedCls;
        }
        //
        this.el.addClass(this.baseCls);
        
        //
        var tpl = [];
        tpl.push('<li class="${itemBaseCls} ${itemCls} ${disclosureCls}" _index="${index}" ', (this.allowGroup ? '_groupseq="${groupSeq}" ' : ''), '_page="${page}">');
        Fw.each(this.itemTpl, function(item){
            tpl.push(item);
        }, this);
        tpl.push('</li>');
        this.itemTpl = Fw.template(tpl);
    },
    /**
     * 初始化GROUP HEADER模版
     */
    initGroupHeaderTpl : function() {
        var tpl = [];
        tpl.push('<li id="${id}" class="${groupHeaderBaseCls} ${groupHeaderCls}" _total="${total}">');
        Fw.each(this.groupHeaderTpl, function(item){
            tpl.push(item);
        }, this);
        tpl.push('</li>');
        this.groupHeaderTpl = Fw.template(tpl);
    },
    /**
     * 手指按下时触发
     * @param {} e
     */
    onItemTouchStart: function(e){
        var el = $(e.target);
        var item = el.hasClass(this.itemBaseCls) ? el : el.parents('li.' + this.itemBaseCls);
        if(!item){
            return false;
        }
        //
        var index = item.attr('_index') * 1;
        if(this.data && this.fireEvent('beforeitemclick', this.data[index], index, el, e, this) !== false){
            this.tapItem = item;
            this.el.addClass(this.pressedCls);
        }
    },
    /**
     * 手指滑动时触发
     * @param {} e
     */
    onItemTouchMove: function(e){
        if(this.tapItem){
            this.el.removeClass(this.pressedCls);
            delete this.tapItem;
        }
    },
    /**
     * 手指抬起时触发
     * @param {} e
     */
    onItemTouchEnd: function(e){
        if(this.tapItem){
            e.preventDefault();
            var index = this.tapItem.attr('_index') * 1;
            this.fireEvent('itemclick', this.data[index], index, this.tapItem, e, this);
            delete this.tapItem;
        }
    },
    /**
     * 载入数据
     * @param {} page/records
     */
    loadData: function(page){
        //加载静态数据
        if(Fw.isArray(page)){
            this.ajax = {
                currentPage: 1
            };
            this.onLoadData(page);
        }
        //请求服务端数据
        else{
            this.ajax.query(Fw.isNumber(page) ? page || 1 : 1);
        }
    },
    /**
     * 载入数据后触发
     * @param {} ajax
     * @param {} data
     */
    onLoadData : function(ajax, data) {
    	//var existData = Fw.isArray(data) && data.length > 0;
        var existData = false;
    	if(Fw.isArray(data) && data.length > 0){
    		existData = true;
    	}else{
    		if(data && Fw.isArray(data.List) && data.List.length > 0){
    			existData = true;
    			data = data.List;
    		}
    	}
        if(this.data){
            if(existData){
                this.data = this.data.concat(data);
            }
        }
        else{
            this.data = existData ? data : null;
        }
        //将HTML渲染到页面
        this.listPaging && this.listPaging.el.show();
        if(this.ajax.currentPage === 1){
            //this.listPaging
            if(existData || !this.emptyText){
                this.el[0].innerHTML = '';
            }
            else if(this.emptyText){
                this.listPaging && this.listPaging.el.hide();
                this.el[0].innerHTML = this.emptyText;
            }
        }
        //
        if(existData){
            var count = this.data.length - data.length;
            for(var i = 0, len = data.length; i < len; i++){
                this.addItem(data[i], i, count);
            }
        }
    },
    /**
     * 新增列表记录
     * @param {} item
     * @param {} index
     * @param {} count
     */
    addItem: function(item, index, count){
        var html = '', cls;
        var sibling = null, groupSeq = null;
        //列表分组
        if(this.allowGroup){
            if(!this.groupPrefix){
                this.groupPrefix = Fw.id();
            }
            var groupId = Fw.call(this.groupField, null, [item]);
            groupId = this.groupPrefix + '-group-' + (groupId === this.groupField ? item[this.groupField] : groupId); 
            var groupEl = $('#' + groupId);
            var group = {
                id: groupId,
                item: item,
                total: groupEl[0] ? groupEl.attr('_total') * 1 : 0,
                groupHeaderBaseCls: this.groupHeaderBaseCls,
                groupHeaderCls: this.groupHeaderCls
            };
            //
            this.fireEvent('beforegroupadd', group, groupEl[0] ? groupEl : null, this);
            if(!groupEl[0]){
                html = this.groupHeaderTpl.render(group);
                this.bodyRender(html);
                sibling = groupEl = $('#' + groupId);
            }
            else{
                sibling = this.el.children('[_groupseq=' + (groupId + '-' + (group.total - 1)) + ']');
            }
            groupEl.attr('_total', group.total + 1);
            //
            groupSeq = groupId + '-' + group.total;
        }
        //列表记录
        if(!Fw.isDefined(this.oldDisclosure)){
            this.oldDisclosure = this.disclosure;   
        }
        if(this.fireEvent('beforeitemadd', item, index, this) !== false){
            var disclosureCls = '';
            if(this.oldDisclosure === true && this.disclosure === false){
                disclosureCls = 'x-list-item-disclosure-none';//不显示箭头
            }
            // else if(Fw.isString(this.disclosure)){
                // disclosureCls = this.disclosure;//自定义箭头样式
            // }
            //
            html = this.itemTpl.render({
                item: item,
                page: this.ajax.currentPage,
                index: count + index,
                itemBaseCls: this.itemBaseCls,
                itemCls: this.itemCls,
                groupSeq: groupSeq,
                disclosureCls: disclosureCls
            });
            
             this.bodyRender(html, sibling);
            //
//            if(this.allowGroup){
//                this.bodyRender(html, sibling);
//            }
//            else{
//                Fw.defer(this.bodyRender, index * 10, this, [html, sibling]);
//            }
        }
    },
    /**
     * 更新BODY HTML
     * @param {} html
     * @param {} sibling
     */
    bodyRender: function(html, sibling){
        if(html){
            if(sibling){
                $(html).insertAfter(sibling);
            }
            else{
                this.el.append(html);
            }
        }
        else{
            // this.el.empty();
            this.el[0].innerHTML = '';
        }
    }
});

/**
 * @name Fw.PullRefresh
 * @class 下拉刷新组件
 * @constructor
 * @example
 * @return
 */
Fw.PullRefresh = Fw.extend(Fw.Component, {
    /**
     * 初始化组件
     */
    initComponent : function() {
        this.ajax = this.owner.ajax;
        Fw.PullRefresh.superclass.initComponent.call(this);
        //
        this.warp = this.owner.el;//$('body');
        this.defaultHeight = this.el.css('height').replace('px', '') * 1;
        //this.defaultTop = this.el.css('top').replace('px', '') * 1;
        this.defaultWarpTop = this.warp.css('margin-top').replace('px', '') * 1;
        this.defaultTop = this.owner.el.offset().top - this.defaultHeight;
        
        var offset = 0;
        this.owner.el.parents().each(function(index, el){
            offset += $(el).css('padding-top').replace('px', '') * 1;
            offset += $(el).css('padding-bottom').replace('px', '') * 1;
        });
        
        this.defaultTop -= offset;
        this.el.css('top', this.defaultTop);
    },
    /**
     * 初始化事件
     */
    initEvents : function() {
        //载入数据后触发
        this.ajax.on('query', this.reset, this);
        //滑动时触发
        this.warp.swipe({
            preventDefaultEvents: false,
            //triggerOnTouchEnd: false,
            //triggerOnTouchLeave: false,
            threshold: 0,
            swipeStatus: Fw.bind(this.onSwipe, this)
        });
    },
    /**
     * 初始化HTML模版
     */
    initTpl : function() {
        var id = this.owner.el.attr('id');
        var tpl = [
            '<div id="${id}-pullrefresh" class="x-pullrefresh ui-loading-wrap" style="-webkit-box-pack:inherit;left: ${left}px!important;">',
                '<div id="${id}-pullrefresh-arrow" class="x-pullrefresh-arrow"></div>',
                '<p id="${id}-pullrefresh-text" class="x-pullrefresh-text"></p>',
            '</div>'
        ];
        var width = this.owner.el.width();
        this.owner.el.before(Fw.template(tpl, {
            id: id,
//            width: width,
            left: (width/2-60)
        }));
        this.el = $('#' + id + '-pullrefresh');
        this.arrowEl = $('#' + id + '-pullrefresh-arrow');
        this.textEl = $('#' + id + '-pullrefresh-text');
    },
    /**
     * 滑动时触发
     * @param {} e
     * @param {} status 滑动状态
     * @param {} direction 滑动方向
     * @param {} distance 滑动距离
     */
    onSwipe: function(e, status, direction, distance, duration, fingerCount, fingerData){
        switch(status){
            //滑动中
            case 'move':
                switch(direction){
                    //向下
                    case 'down':
                        if(this.currScrollTop === 0){
                            e.preventDefault();
                            this.el.css('top', this.defaultTop + distance);
                            this.warp.css('margin-top', distance);
                            //准备刷新
                            if(distance >= this.defaultHeight * 1.6){
                                this.textEl[0].innerHTML = Fw.PullRefresh.releaseRefreshText;
                                this.arrowEl.addClass('x-pullrefresh-release');
                                this.allowRefresh = true;
                            }
                            //提示下拉刷新
                            else{
                                this.textEl[0].innerHTML = Fw.PullRefresh.pullText;
                                this.arrowEl.removeClass('x-pullrefresh-release');
                                this.allowRefresh = false;
                            }
                        }
                        break;
                    //向上
                    /*
                    case 'up':
                        this.el.css('top', this.defaultTop - distance);
                        var warpTop = this.warp.css('padding-top').replace('px', '') * 1;
                        if(warpTop > this.defaultWarpTop){
                            this.warp.css('padding-top', this.defaultWarpTop - distance);
                        }
                        break;
                    */
                }
                break;
            //滑动结束
            case 'end':
                //刷新
                if(this.allowRefresh){
                    this.refresh();
                }
                //重置
                else{
                    this.reset(false);
                }
                break;
            default:
                this.currScrollTop = Fw.getScrollTop();
                if(fingerCount > 1 && this.allowRefresh){
                    this.refresh();
                }
                break;
        }
    },
    /**
     * 状态重置
     */
    reset: function(suppress){
        if(this.loading === true || suppress === false){
            this.el.css('top', this.defaultTop);
            this.warp.css('margin-top', this.defaultWarpTop);
            this.arrowEl.removeClass('ui-loading').addClass('x-pullrefresh-arrow');
            this.allowReset = false;
        }
        this.loading = false;
    },
    /**
     * 刷新数据
     */
    refresh: function(){
        if(this.allowRefresh && this.loading === false){
            this.el.css('top', this.defaultTop + this.defaultHeight);
            this.warp.css('margin-top', this.defaultHeight);
            //
            this.arrowEl.removeClass('x-pullrefresh-arrow x-pullrefresh-release').addClass('ui-loading');
            this.textEl[0].innerHTML = Fw.PullRefresh.loadingText;
            //
            this.loading = true;
            this.allowRefresh = false;
            delete this.owner.data;
            this.owner.ajax.query(1);
            //
            //Fw.defer(this.owner.ajax.query, 500, this.owner.ajax, [1]);
        }
    }
});

/**
 * @name Fw.ListPaging
 * @class 列表分页组件
 * @constructor
 * @example
 * @return
 */
Fw.ListPaging = Fw.extend(Fw.Component, {
    /**
     * 初始化组件
     */
    initComponent : function() {
        this.ajax = this.owner.ajax;
        Fw.apply(this.ajax, {
            pageLimit: 1
        });

        Fw.ListPaging.superclass.initComponent.call(this);
        //
        this.warp = this.owner.el;
        //old
//        this.warp = $('html');
        //old
        this.defaultHeight = this.el.css('height').replace('px', '') * 1;
    },
    /**
     * 初始化事件
     */
    initEvents : function() {
        //载入数据前触发
        this.ajax.on('beforequery', this.onBeforeLoadData, this);
        //载入数据后触发
        this.ajax.on('query', this.onLoadData, this);
        //sm
        //加载出错
        this.ajax.on('error', this.onFailure, this);
        //sm
        //手指按下时触发
        this.warp.on('touchstart', Fw.bind(this.onTouchStart, this));
        //手指滑动时触发
        //sm old code ...
//        this.warp.on('touchmove', Fw.bind(this.onTouchMove, this));
        
        //sm 添加android判断
        if(Fw.isAndroid){
        	$(document).on('scroll',Fw.bind(this.onTouchMove, this));//android
        }else{
        	this.warp.on('touchmove', Fw.bind(this.onTouchMove, this));//
        }//sm
        
        
        //手指抬起时触发
        this.warp.on('touchend', Fw.bind(this.onTouchEnd, this));
        //点击加载更多触发
        this.el.on('tap', Fw.bind(this.onTap, this));
    },
    //sm
    /**
     * 加载出错
     */
    onFailure: function(ajax, status, textStatus, error, xhr){
    	//隐藏
    	this.loadingEl.hide();
    	this.textEl[0].innerHTML = Fw.ListPaging.loadMoreText;
    	this.loading = false;
    	this.ajax.currentPage--;
    	//sm 重新绑定查询事件
    	this.loadingEvent();
    },
    //sm
    /**
     * 初始化HTML模版
     */
    initTpl : function() {
        var id = this.owner.el.attr('id');
        var tpl = [
            '<div id="${id}-loadmore" class="x-loadmore ui-loading-wrap ui-border-t">',
                '<i id="${id}-loadmore-loading" class="x-loadmore-loading ui-loading"></i>',
                '<p id="${id}-loadmore-text" class="x-loadmore-text">${loadMoreText}</p>',
            '</div>'
        ];
        this.owner.el.after(Fw.template(tpl, {
            id: id,
            loadMoreText: Fw.ListPaging.loadMoreText
        }));
        this.el = $('#' + id + '-loadmore');
        this.loadingEl = $('#' + id + '-loadmore-loading');
        this.textEl = $('#' + id + '-loadmore-text');
    },
    /**
     * 加载更多点击时触发
     * @param {} e
     */
    onTap: function(e){
    	
    	//sm
    	this.loadingEl.show();
    	//sm
    	
    	this.textEl[0].innerHTML = Fw.ListPaging.loadingText;
        Fw.defer(this.loadingEl.addClass, 10, this.loadingEl, ['ui-loading']);
        this.moveNext();
    },
    /**
     * 手指按下时触发
     * @param {} e
     */
    onTouchStart: function(e){
        var targetEl = $(e.target);
        if(String(targetEl.attr('id')).indexOf(this.el.attr('id')) === 0){
            this.touchEl = targetEl;
        }
    },
    /**
     * 手指抬起时触发
     * @param {} e
     */
    onTouchEnd: function(e){
        if(this.touchEl){
            this.textEl[0].innerHTML = Fw.ListPaging.loadingText;
            Fw.defer(this.loadingEl.addClass, 10, this.loadingEl, ['ui-loading']);
            this.moveNext();
        }
        this.touchEl = null;
    },
    /**
     * 手指滑动时触发
     * @param {} e
     */
    onTouchMove:function(e){
    	//old code...
//        var clientHeight = Fw.getWindowHeight();
//        var scrollTop = Fw.getScrollTop();
//        var scrollHeight = Fw.getScrollHeight();
//        if(clientHeight + scrollTop - 10 > scrollHeight && scrollTop > 0 && !this.loading){
//            this.textEl[0].innerHTML = Fw.ListPaging.loadingText;
//            Fw.defer(this.loadingEl.addClass, 10, this.loadingEl, ['ui-loading']);
//            this.moveNext();
//        }
    	
    	
    	 //sm android下拉分页
    	 var clientHeight = Fw.getWindowHeight();
         var scrollTop = Fw.getScrollTop();
         var scrollHeight = Fw.getScrollHeight();
         var hasMove;
         
         if(Fw.isAndroid){
         	hasMove = clientHeight + scrollTop  > scrollHeight - 10 && scrollTop > 0 && !this.loading;
         	// 判断是否存在 tab
         	var isTab = $('body').find('.ui-tab>ul>li').hasClass('current');
         	var isTabList = $('body').find('.ui-tab>ul>li.current .x-list').length;
         	//存在tab，并且当前tab不含有listView 不进行分页处理
         	if(isTab && isTabList == 0){
         		hasMove = false;
         	}
         }else{
         	hasMove = clientHeight + scrollTop - 10 > scrollHeight && scrollTop > 0 && !this.loading;
         }
         if(hasMove){
             this.textEl[0].innerHTML = Fw.ListPaging.loadingText;
             Fw.defer(this.loadingEl.addClass, 10, this.loadingEl, ['ui-loading']);
             this.moveNext();
         }
        //sm android下拉分页
    	
    },
    /**
     * 载入数据前触发
     */
    onBeforeLoadData : function(){
        //
        if(this.loading){
            this.textEl[0].innerHTML = Fw.ListPaging.loadingText;
        }
        else{
            this.textEl[0].innerHTML = Fw.ListPaging.loadMoreText;
        }
    },
    /**
     * 载入数据后触发
     */
    onLoadData: function(ajax, data){
    	//sm 重新绑定查询事件
    	this.loadingEvent();
    	
        var totalCount = this.ajax.totalCount;
        var start = ((this.ajax.currentPage - 1) * this.ajax.pageSize) + 1;
        var end = Math.min(this.ajax.currentPage * this.ajax.pageSize, totalCount);
        //没有更多数据
        this.existMore = end < totalCount;
        
        //sm
        var loading = false;
        //取消重复绑定事件
    	this.loadingOff = false;
    	
        if(!this.existMore){
        	//sm
        	this.loadingEl.hide();
        	//sm
            this.textEl[0].innerHTML = Fw.ListPaging.noMoreText;
            //sm 没有数据移除点击事件
            this.el.off('tap');
            //如果重新查询打开事件
            this.loadingOff = true;
            //不可以上拉刷新
            loading = true;
        }
        else{
        	//sm
        	this.loadingEl.show();
        	//sm
            this.textEl[0].innerHTML = Fw.ListPaging.loadMoreText;
        }
        Fw.defer(this.loadingEl.removeClass, 50, this.loadingEl, ['ui-loading']);
        this.loading = loading;
    },
    /**
     * sm 重新绑定点击更多事件
     */
    loadingEvent: function(){
    	//sm 重新绑定事件
    	if(this.loadingOff)
    	{
    		//点击加载更多触发
            this.el.on('tap', Fw.bind(this.onTap, this));
            //
            this.loadingOff = false;
    	}
    },
    /**
     * 移动到下1页
     */
    moveNext: function(){
        if(this.existMore){
            delete this.lockedPage;
            this.loadPage(++ this.ajax.currentPage);
        }
        else{
            if(!this.lockedPage){
                this.lockedPage = this.ajax.currentPage + 1;
            }
            var start = null, limit = null;
            if(Fw.isArray(this.owner.data) && this.owner.data.length > 0){
                limit = this.owner.data.length % this.ajax.pageSize;
                if(limit === 0){
                    limit = null;
                }
                else{
                    limit = this.ajax.pageSize - limit;
                    start = ((this.lockedPage - 1) * this.ajax.pageSize) - limit;
                    limit = this.ajax.pageSize + limit;
                }
            }
            this.loadPage(this.lockedPage, start, limit);
        }
    },
    /**
     * 分页
     * @param {Object} page
     */
    loadPage: function(page, start, limit){
        if (this.fireEvent('beforechange', this, page) !== false) {
            this.loading = true;
            this.ajax.query(page, start, limit);
        }
    }
});

//
Fw.Logger.debug('加载列表组件');