/**
 * 查询工具条
 */
Fw.SearchBar = Fw.extend(Fw.Component, {
    /**
     * 基本样式
     */
    cls: 'x-searchbar',
    /**
     * 输入框占位信息
     */
    placeholder: '搜索内容',
    /**
     * 查询按钮名称
     */
    queryText: '查询',
    /**
     * 查询类型/手动(pointer),自动(auto)
     */
    triggerType: 'pointer',
    /**
     * 取消按钮名称
     */
    cancelText: '取消',
    /**
     * 最少输入字符个数
     * 满足该参数时，触发查询事件
     */
    minChars: 4,
    /**
     * 初始化事件
     */
    initEvents: function(){
        //单击查询工具条时触发
        this.searchbar.tap(Fw.bind(this.onClick, this));
        //单击清除按钮时触发
        this.clearBtn.tap(Fw.bind(this.doClear, this));
        //单击取消按钮时触发
        this.cancelBtn.tap(Fw.bind(this.doCancel, this));
        //输入查询条件时触发
        this.searchIpt.on('input', Fw.bind(this.doSearch, this));
        //点击查询
        this.queryBtn.tap(Fw.bind(this.doQuery, this));
    },
    /**
     * 初始化HTML模版
     */
    initTpl: function(){
        //如果未在HTML中定义，则自动创建
        if(this.el.children().length === 0){
            var tpl = [
                '<div class="ui-searchbar ui-border-radius">',
                    '<i class="ui-icon-search"></i>',
                    '<div class="ui-searchbar-text">${placeholder}</div>',
                    '<div class="ui-searchbar-input">',
                        '<input type="text" value="" placeholder="${placeholder}"',
                        '{@if maxLength}',
                        ' maxlength="${maxLength}" ',
                        '{@/if}',
                    	' autocapitalize="off">',
                    '</div>',
                    '<i class="ui-icon-close"></i>',
                '</div>',
                '<button class="ui-searchbar-cancel ui-searchbar-query-btn">${queryText}</button>',
                '<button class="ui-searchbar-cancel ui-searchbar-cancel-btn">${cancelText}</button>'
            ];
            this.el.append(Fw.template(tpl).render({
                placeholder: this.placeholder,
                cancelText: this.cancelText,
                queryText: this.queryText,
                maxLength:this.maxLength
            }));
        }
        //
        this.el.addClass(this.cls);
        
        //已在HTML中定义
        this.searchbar = this.el.children('div.ui-searchbar');
        this.cancelBtn = this.el.children('button.ui-searchbar-cancel-btn');
        this.searchIpt = this.searchbar.find('div.ui-searchbar-input input');
        this.clearBtn = this.searchbar.children('i.ui-icon-close');
        //查询
        this.queryBtn = this.el.children('button.ui-searchbar-query-btn');
        //是否显示确定按钮
        this.queryBtn.hide();
    },
    /**
     * 单击查询工具条时触发
     */
    onClick: function(){
        this.el.addClass('focus');
        Fw.defer(this.searchIpt.focus, 100, this.searchIpt);
    },
    /**
     * 单击清除按钮时触发
     */
    doClear: function(){
        this.searchIpt.val('');
        this.searchIpt.blur();
        this.doSearch();
        
        if(this.triggerType  === 'pointer') //手动
        	this.doQuery();
    },
    /**
     * 单击取消按钮时触发
     */
    doCancel: function(){
        this.el.removeClass('focus');
        this.doClear('');
        
        //关闭查询按钮
        this.queryBtn.hide();
    },
    /**
     * 输入查询条件时触发
     */
    doSearch: function(){
    	//
        var value = this.searchIpt.val();
        //
        if(Fw.isEmpty(value) || value.length >= this.minChars)
        {
        	if(this.triggerType  === 'pointer') //手动
        	{
        		this.queryBtn.show();
        		
        	}else{
        		this.doQuery();
        	}
        }
        
    },
    /**
     * 点击查询
     */
    doQuery: function(){
    	var value = this.searchIpt.val();
    	if(this.fireEvent('beforesearch', value, this) !== false){
            this.fireEvent('search', value, this);
        }
    }
});

//
Fw.Logger.debug('加载查询工具条组件');