/**
 * 贷款客户信息变更
 * @Description: 1.客户相关重要基本信息需要变更的，手机银行界面新增一个模块，客户可以提出变更申请，并自动发送变更申请短信给管户客户经理。
 * 				 2.管户客户经理收到申请后联系客户并签署书面的变更确认书。
 */
var App = function() {
	var form;
    return {
        /**
         * 依赖组件
         */
        requires: [
			'Fw.Form',
			'Fw.Keyboard',
			'Fw.app.Session',
			'Fw.ConstantData'
        ],
        /**
         * 视图设置
         * 设置标题栏、按钮等
         * 如果为空或未设置，则标题栏不显示、左右按钮不显示、等待层不自动显示
         */
        viewConfig: {
            /**
             * 标题栏标题
             * 如果为空或未设置，则标题栏不显示、左右按钮不显示
             */
            title: '基本信息变更',
            /**
             * 左边按钮（默认显示）
             * 如果为false，则左边按钮不显示
             * leftButton: '左边按钮'
             */
            leftButton: {
                //自定义按钮标题（默认'返回'）
                text: '返回',
            },
            /**
             * 是否自动显示等待层（默认true）
             * 需手动隐藏等待层
             */
            loadMask: false
        },
        
        /**
         * 左边按钮点击事件
         */
        leftButtonHandler: function(){
        	var param = Fw.decode(Fw.getParameters()?Fw.getParameters():'{}');
			if(undefined != param.backFlag && param.backFlag == "LOAN"){
    			Fw.redirect("./main.html");
    		}else{
    			Fw.device.Device.gotoIndex();
    		}
        },
        
        /**
         * 初始化
         * 应用入口
         */
        init : function(require) {
        	//定义表单
        	if(Fw.isEmpty(form)){
        		form = new Fw.Form({
                    //EL
                    contentEl : 'form',
                    //是否激活重置事件（默认false）
                    //单击输入框时，自动重置为初始值（仅处理文本输入框、多行文本输入框）
                    resettable: false,
                    //校验提示方式（tips：浮动层、msg：弹出框，默认为tips）
                    msgTarget: 'tips'
                });
        		form.on('submit', this.doSubmit, this);
        	}            
        	//初始化数据
        	this.initData();
        	//初始化事件
        	this.initEvents();
        },
        
        /**
         * 初始化事件
         */
        initEvents:function(){
            $('#submit-btn').on('click', Fw.bind(this.toSubmit,this));
        },       
        /**
         * 初始化数据
         */
        initData:function(){
        	Fw.ConstantData.setIdType($("[name=CertType]"),0);
        	form.setValues("CustomerName",Fw.app.Session.getCustName());
        	form.setValues("CertId",Fw.app.Session.getIdNo());
        	form.setValues("CertType",Fw.app.Session.getIdType());
        },
        toSubmit: function(){
        	form.submit();
        },
        doSubmit: function(values){
        	Fw.util.Ajax.execute({
				service : 'loadService',
				method : 'executeNetLoanMCifInfoUpdate',
				params : {
					CustomerName: values.CustomerName,
					CertType: $("[name=CertType]").val(),
					CertId: values.CertId
				},
				// 是否使用数据挡板（默认false）
				staticData : false,
				// 请求成功后回调
				success : function(data, result) {
					Fw.Msg.info("基本信息变更提交成功！",function(){
						Fw.redirect('./main.html');
					});
				},
				// 请求错误后回调
				failure : function(status, textStatus, error) {
					
				}
			});
        }
    };
}();


/**
 * 页面加载完毕后，初始化应用
 */
Fw.onReady(App); 