var App = function() {
	var tab;
	
    return {
        /**
         * 依赖组件
         */
        requires: [
			'Fw.Tab',
            'Fw.TimerGlider',
            //日历组件
			'Fw.Calendar',
        ],
        /**
         * 视图设置
         * 设置标题栏、按钮等
         * 如果为空或未设置，则标题栏不显示、左右按钮不显示、等待层不自动显示
         */
        viewConfig: {
            /**
             * 标题栏标题
             * 如果为空或未设置，则标题栏不显示、左右按钮不显示
             */
            title: '乘车记录查询',
            /**
             * 左边按钮（默认显示）
             * 如果为false，则左边按钮不显示
             * leftButton: '左边按钮'
             */
            leftButton: {
                //自定义按钮标题（默认'返回'）
                text: '返回',
            },
            /**
             * 是否自动显示等待层（默认true）
             * 需手动隐藏等待层
             */
            loadMask: false
        },
        
        /**
         * 左边按钮点击事件
         */
        leftButtonHandler: function(){
    		//返回上级目录
    		 Fw.device.Device.gotoIndex();
        },
        
        /**
         * 初始化
         * 应用入口
         */
        init : function(require) {
        	//定义选项卡
            tab = new Fw.Tab({
            	//选项卡EL
                contentEl: 'tab',
                //默认选项卡（默认0）
                activeTab: 0,
                //是否锁定tab标题
                fixed:true,
                reload: true
            });
            
        	//初始化事件
        	this.initEvents();
        	//初始化数据
        	this.initData();
        },
        
        /**
         * 左边按钮点击事件
         */
        leftButtonHandler: function(){
        	//获取当前选项卡序号
        	var index = tab.getActiveItemIndex();
        	if(index == 0) {
        		//返回上级目录
        		 Fw.device.Device.gotoIndex();
        	} else {
        		//返回上一页
	        	tab.movePrev();
        	}
        },
        
        /**
         * 初始化事件
         */
        initEvents:function(){
        	$('#query-btn').on('click',Fw.bind(this.doSubmit,this));
        },
        
        /**
         * 初始化数据
         */
        initData:function(){
        	this.initCurDate();
        },
        
        //获取当前日期
        initCurDate: function(){
        	Fw.util.Ajax.get({
				service : 'curdateService/getCurDate',
				// 请求成功后回调
				success : function(data, result) {
					$('[name="startDate"]').val(Fw.util.Format.fmtSomeMonth(data['CurDate'], -1));
					$('[name="endDate"]').val(data['CurDate']);
				},
			});
        },
        
        
        /**
         * 提交表单
         */
        doSubmit: function(){
        	var startDate = new Date($("[name=startDate]").val());
        	var endDate = new Date($("[name=endDate]").val());
        	endDate.setMonth(endDate.getMonth()-1);
        	if(endDate > startDate){
        		Fw.Tips.info("查询时间跨度不能超过1个月！");
        		return false;
        	}
        	App.initTimerGlider();
        	tab.setActiveItem(1);
        },
        
        /**
         * 加载时间轴
         */
        initTimerGlider: function(){
	        var timerGlider = new Fw.TimerGlider({
	            contentEl: 'timeline',
	            ajax: {
	            	page: false,
	            	service: 'busQrcodeService',
	            	method: 'queryMCRideInfoQry',
	            	staticData: false,
	            	params: {
	            		BegDat: $("[name=startDate]").val().replace(/\//g ,""),
	            		EndDat: $("[name=endDate]").val().replace(/\//g ,"")
	                },
	            },
	            groupField: 'Month',
	            groupHeaderTpl: [
	                    '${item.Month}'
                ],
                 //下拉刷新（默认false）
                pullRefresh: false,
	            itemTpl:[
						'<div class="timeline-content">',
							'<div class="content-left">',
								'<h4 class="ui-nowrap">${item.Day}日</h4>',
								'<h4 class="ui-nowrap">${item.BusNo}</h4>',
							'</div>',
							'<div class="content-right">',
								'<h4 class="ui-nowrap outlay"> -${item.Amount|fmtAmt}</h4>',
							'</div>',
						'</div>',
						'<div class="content-detail">',
							'<h4 class="ui-nowrap">乘车路线：<span style="margin-left: 5px;">${item.BusNo}</span></h4>',
							'<h4 class="ui-nowrap">乘车日期：<span style="margin-left: 5px;">${item.BusDate|fmtDate} ${item.BusTime|fmtDate}</span></h4>',
							'<h4 class="ui-nowrap">付款账户：<span style="margin-left: 5px;">${item.PayerAccNo|fmtAcctNo, true}</span></h4>',
							'<h4 class="ui-nowrap">交易金额：<span style="margin-left: 5px;">${item.Amount|fmtAmt}</span></h4>',
							'<h4 class="ui-nowrap">收款人　：<span style="margin-left: 5px;">${item.PayeeName}</span></h4>',
							'<h4 class="ui-nowrap">付款凭证号：<span style="margin-left: 5px;">${item.VoucherNum}</span></h4>',
						'</div>'
					]
	        });
	        //每行记录点击事件
	        timerGlider.on('itemclick',this.onItem);
	        //加载数据
	        timerGlider.loadData();
	        timerGlider.ajax.on('query', this.onLoadData, this);
	    },
	    
	    /**
	     * 点击记录触发事件
	     */
	    onItem: function(record, index, item, e, cmp){
	    	//当前节点
			var currentTarget = $(e.currentTarget);
			//子节点
			var nextTarget = currentTarget.children('.content-detail');
			//动画
			nextTarget.toggle();
        },
        
        onLoadData: function(){
        	$('#timelineEnd').html("");
        	var endmsg=[];
        	endmsg[endmsg.length]='<div id="list-loadmore" class="x-loadmore ui-loading-wrap ui-border-t" style="transform-origin: 0px 0px 0px; opacity: 1; transform: scale(1, 1);">',
        	endmsg[endmsg.length]='	<i id="list-loadmore-loading" class="x-loadmore-loading     " style="transform-origin: 0px 0px 0px; opacity: 1; transform: scale(1, 1); display: none;"></i>',
        	endmsg[endmsg.length]='	<p id="list-loadmore-text" class="x-loadmore-text">没有更多数据</p>'
        	endmsg[endmsg.length]='</div>'
        	$('#timelineEnd').append(endmsg.join(""));
        },

    };
}();


/**
 * 页面加载完毕后，初始化应用
 */
Fw.onReady(App); 