var App = function() {
	var tab;
	var listView;
	var cardList; //汇总账户
	
    return {
        /**
         * 依赖组件
         */
        requires: [
			'Fw.Tab',
			'Fw.ListView',
			'Fw.app.Session'
        ],
        /**
         * 视图设置
         * 设置标题栏、按钮等
         * 如果为空或未设置，则标题栏不显示、左右按钮不显示、等待层不自动显示
         */
        viewConfig: {
            /**
             * 标题栏标题
             * 如果为空或未设置，则标题栏不显示、左右按钮不显示
             */
            title: '资金归集账户',
            /**
             * 左边按钮（默认显示）
             * 如果为false，则左边按钮不显示
             * leftButton: '左边按钮'
             */
            leftButton: {
                //自定义按钮标题（默认'返回'）
                text: '返回',
            },
            /**
             * 是否自动显示等待层（默认true）
             * 需手动隐藏等待层
             */
            loadMask: false
        },
        /**
         * 初始化
         * 应用入口
         */
        init : function(require) {
        	if(!tab){
        		//定义选项卡
                tab = new Fw.Tab({
                    //选项卡EL
                    contentEl: 'tab',
                    //默认选项卡（默认0）
                    activeTab: 0,
                    //是否锁定tab标题
                    fixed:true
                });
        	}
        	this.initListView();
        	//初始化事件
        	this.initEvents();
        	//初始化数据
        	this.initData();
        },
        
        /**
         * 初始化事件
         */
        initEvents:function(){
        	//绑定change事件
            $('[name="cardList"]').on('change', Fw.bind(this.changeCard, this));
        },
        
        /**
         * 左边按钮点击事件
         */
        leftButtonHandler: function(){
        	//获取当前选项卡序号
        	var index = tab.getActiveItemIndex();
        	if(index == 0) {
        		//返回上级目录
        		 Fw.device.Device.gotoIndex();
        	} else {
        		//返回上一页
	        	tab.movePrev();
        	}
        },
        
        /**
         * 初始化数据
         */
        initData:function(){
        	cardList = Fw.app.Session.getAllCards();
        	var j = 0;
        	for(var i=0; i<cardList.length; i++){
    			var card = cardList[i];
    			if(card.BankAcType != "PCRC" & card.BankAcType != "PDAC"){
    				$('[name="cardList"]').append("<option value="+Fw.encode(card)+">"
    						+Fw.util.Format.fmtAcctNo(card.No, true)+"/"+Fw.util.Format.fmtCardType(card.BankAcType)+"</option>");
    				if(j == 0){
    					this.changeCard();
    				}
    				j++;
    			}
    		}
    		if(j == 0){
    			Fw.Tips.info("暂无汇总账户");
    		}
        },
        
        /**
		 * 切换卡
		 */
        changeCard: function(){
        	var card = Fw.decode($('[name="cardList"]').val());
        	var acNo = card.No;
        	listView.ajax.setParams("SweepType", '3'); //1 行内  2 行外 3 全部
        	listView.ajax.setParams("PayeeAcc", acNo);
        	//加载数据
        	listView.loadData();
        	//载入数据后触发
        	listView.ajax.on('query', this.onQuery, this);
        	//选中记录时触发
         	listView.on('itemclick', this.onSelectRecord, this);
        },
        
        /**
		 * 初始化归集账户listView
		 */
        initListView:function(){
        	if(!listView){
        		//定义列表
            	listView = new Fw.ListView({
                    //EL
                    contentEl : 'listView',//加载数据
    			    cls: 'sx-list',
    			    groupCls: 'sx-list-group',
    			    groupHeaderBaseCls: 'sx-list-group-header',
    			    groupHeaderCls: 'sui-border-t',
    			    itemBaseCls: 'sx-list-item',
    			    itemCls: 'sui-border-t',
    			    pressedCls: 'sui-list-active',
                    //数据请求对象（为空时，可通过loadData加载静态数据，但不支持beforequery、query事件）
                    ajax : {
                        //数据源（服务接口）
                        service: 'cashSweepService',
                        method: 'queryMCFundFocusSignQuery',
                        //请求参数
                        params: {
                        	ChannelCode: '006', // 006 手机银行
                        	SweepType: '', // 归集账户类型
                        	PayeeAcc: '', // 归集账号
        				    PageRowNum1: '5', // 签约记录每页显示条数  未签约记录翻页时不输
        				    OperType1: '0', // 签约记录操作类型  0 - 首页，1 - 尾页，2 - 前一页，3 - 下一页，4 – 当前页；首次送0（未签约记录翻页时不输）
        				    CurrPage1: '0', // 签约记录当前页码  Opertype选择2、3、4时，currpage必输，首次送0
        				    PageRowNum2: '', // 未签约记录每页显示条数 
        				    OperType2: '', // 未签约记录操作类型
        				    CurrPage2: '', // 未签约记录当前页码
                        },
                        //是否分页（默认true）
    					page: true,
    					pageSize: 5,
                    },
                    //分组字段（默认null）
    				groupField : 'orderIndex',
                    groupHeaderTpl : [
                         
              		],
                    //向右箭头（默认false）
                    disclosure: false,
                    //数据展现模版
                    itemTpl: [
                      		'<div class="sx-list-content-left"></div>', 
                    		'<div class="sx-list-content-ui">',
                    			'<div class="sx-font-key">',
                    				'<div class="font-width-8">被归集账户类型</div>',
                    				'<div class="font-width-6">被归集账户</div>',
                    				'<div class="font-width-9">被归集账户开户行</div>', 
                    				'<div>签约状态</div>',
                    			'</div>',
                    			'<div class="sx-font-value">',
                    				'<div>',
                    					'{@if item.sweeptype == 1}',
                    						'行内',
                    					'{@else}',
             								'行外',
             							'{@/if}',
             						'</div>',
                    				'<div>${item.sweepacc|fmtAcctNo,true}</div>',
                    				'<div>${item.sweepclrbank}</div>', 
                    				'<div>',
	                    				'{@if item.signstat == 1}',
		            						'签约',
		            					'{@else}',
		     								'解约',
		     							'{@/if}',
                    				'</div>',
                    			'</div>',
                    		'</div>', 
                    		'<div class="sx-list-content-e sx-arr"></div>',
                    ]
                });
        	}
        },
        
        onQuery: function(ajax, data, result) {
        	
        },
        
        /**
		 * 选中记录
		 */
        onSelectRecord: function(itemCls){
        	$('#PayeeAcc').html(Fw.util.Format.fmtAcctNo(itemCls.PayeeAcNo, true));
        	$('#Sweeptype').html(this.fmtSweepType(itemCls.sweeptype));
        	$('#Sweepacc').html(Fw.util.Format.fmtAcctNo(itemCls.sweepacc, true));
        	$('#Sweepclrbank').html(itemCls.sweepclrbank);
        	$('#Signdate').html(Fw.util.Format.fmtDate(itemCls.signdate));
        	$('#Unsigndate').html(Fw.util.Format.fmtDate(itemCls.unsigndate));
        	$('#Signstat').html(this.fmtSignStat(itemCls.signstat));
        	$('#Sweepcycle').html(this.fmtSweepCycle(itemCls.sweepcycle, itemCls.smonth, itemCls.sweekday));
        	$('#sweepflag').html(this.fmtSweepFlag(itemCls.sweepflag));
        	$('#Remamount').html(Fw.util.Format.fmtAmt(Fw.util.Format.fmtUndefined(itemCls.remamount)));
        	$('#Slimtamout').html(Fw.util.Format.fmtAmt(Fw.util.Format.fmtUndefined(itemCls.slimtamout)));
        	$('#Xlimtamout').html(Fw.util.Format.fmtAmt(Fw.util.Format.fmtUndefined(itemCls.xlimtamout)));
        	$('#Remak').html(itemCls.Remak);
        	tab.moveNext();
        },

        /**
		 * 被归集账户类型
		 */
        fmtSweepType: function(sweeptype) {
        	if(sweeptype == "1"){
        		return "行内";
        	}else{
        		return "行外";
        	}
        },
        
        /**
		 * 签约状态
		 */
        fmtSignStat: function(signstat) {
        	if(signstat == "1"){
        		return "签约";
        	}else{
        		return "解约";
        	}
        },
        
        /**
		 * 资金归集方式
		 */
        fmtSweepFlag: function(sweepflag) {
        	switch(sweepflag){
        		case "1": return "全额归集";
        		case "2": return "保底归集";
        		case "3": return "智能归集";
        		default:  return "";
        	}
        },
        
        /**
		 * 资金归集日期
		 */
        fmtSweepCycle: function(sweepcycle, smonth, sweekday) {
        	switch(sweepcycle){
	    		case "1": return "每日";
	    		case "2": 
	    			switch(sweekday){
	    				case "1": return "每周一";
	    				case "2": return "每周二";
	    				case "3": return "每周三";
	    				case "4": return "每周四";
	    				case "5": return "每周五";
	    				case "6": return "每周六";
	    				case "7": return "每周日";
	    				default:  return "";
	    			}
	    		case "3": 
	    			switch(smonth){
	    				case "5": return "每月5日";
	    				case "15": return "每月15日";
	    				case "25": return "每月25日";
	    				default:  return "";
	    			}
	    		default:  return "";
	    	}
        },
        
    };
}();


/**
 * 页面加载完毕后，初始化应用
 */
Fw.onReady(App); 