/**
 * 创建应用
 */
var App = function() {
    var tab;
    var buttons = $('button');
    var actionsheet = $('.ui-actionsheet');
    var listView;
    //
    return {
        /**
         * 依赖组件
         */
        requires: [
            'Fw.Tab',
            //列表
            'Fw.ListView'
        ],
        /**
         * 视图设置
         * 设置标题栏、按钮等
         * 如果为空或未设置，则标题栏不显示、左右按钮不显示、等待层不自动显示
         */
        viewConfig: {
            /**
             * 标题栏标题
             * 如果为空或未设置，则标题栏不显示、左右按钮不显示
             */
            title: '选项卡',
            /**
             * 左边按钮（默认显示）
             * 如果为false，则左边按钮不显示
             * leftButton: '左边按钮'
             */
            leftButton: {
                //自定义按钮标题（默认'返回'）
                text: '左边按钮',
                //自定义按钮点击事件（默认App.leftButtonHandler）
                //handler: XXX
                handler: Fw.device.Device.gotoIndex
            },
            /**
             * 右边按钮（默认不显示）
             * rightButton: '右边按钮'
             */
            rightButton: {
                //按钮标题
                text: '分享',
                //自定义按钮点击事件（默认App.rightButtonHandler）
                //handler: XXX
            },
            /**
             * 是否自动显示等待层（默认true）
             * 需手动隐藏等待层
             */
            //loadMask: false
        },
        /**
         * 初始化
         * 应用入口
         */
        init : function(require) {
            //定义选项卡
            tab = new Fw.Tab({
                //选项卡EL
                contentEl: 'tab',
                //默认选项卡（默认0）
                activeTab: 0,
                //是否锁定tab标题
                fixed:true,
                //是否禁用切换tab
//                enabled: true
            });
            
            //禁止滚动(Tab 切换)
//            tab.disable();
            
            //开启滚动(Tab 切换)
//            tab.enable();
            
            
            
            //加载listView
            this.initListView();
            

            //初始化事件
            this.initEvents();
            
            //手动隐藏等待层
            Fw.LoadMask.hide();
        },
        /**
         * 初始化事件
         */
        initEvents: function(){
            //选项卡切换前触发
            tab.on('beforetabchange', this.onBeforeTabChange);
            //选项卡切换后触发
            tab.on('tabchange', this.onTabChange);
            //点击按钮时触发（上一步、下一步）
            buttons.tap(Fw.bind(this.onButtonClick, this));
        },
        /**
         * 选项卡切换前触发
         */
        onBeforeTabChange: function(tab, newIndex, oldIndex){
        	Fw.Tips.info('itemparent newIndex == '+newIndex+' oldIndex == '+oldIndex);
            switch(newIndex){
                //第一个
                case 0:
                    //禁用上一步
                    buttons.eq(0).attr('disabled', 'true');
                    break;
                //最后一个
                case tab.items.length - 1:
                    //禁用下一步
                    //buttons.eq(1).attr('disabled', 'true');
                    
                    //设置跳转参数
                    //仅当设置了data-remote时有效
                    tab.setRedirectParams({
                        p1: '参数1',
                        p2: '参数1',
                        p3: '参数1'
                    });
                    break;
                default:
                    buttons.removeAttr('disabled');
            }
            //... ...
        },
        /**
         * 初始化listView
         */
        initListView: function(){
        	//定义列表
        	if(!listView){//如果没有被实例化，开始实例化
	            listView = new Fw.ListView({
	                //EL
	                contentEl : 'list',
	                //数据请求对象（为空时，可通过loadData加载静态数据，但不支持beforequery、query事件）
	                ajax : {
	                    //数据源（自定以URL）
	                    //url: '/data/demo1Service/XXX.json',
	                    //数据源（服务接口）
	                    //service: 'demo1Service.queryXXX',
	                    service: 'demo1Service/query',
	                    //method: 'query',//默认为query，方法名称必须是query开头
	                    //请求参数
	                    // params: {
	                        // name: '名称'
	                    // },
	                    //是否自动显示等待层（默认true）
	                    //允许自定义等待层提示信息（loadMask: '请稍后...'）
	                    //loadMask: true,
	                    //简单分页组件（只显示上一页、下一页，默认false）
	                    simplePage: true,
	                    //分页参数
	                    pageSize: 10,
	                    //是否使用数据挡板（默认false）
	                    staticData: true
	                },
	                //分组字段（默认null）
	                groupField: 'lastModiDate',
	                //分组字段（通过自定义函数返回分组属性）
	                // groupField: function(values){
	                    // return Fw.util.Format.fmtDate(values.lastModiDate, 'yyyy年MM');
	                // },
	                //分组显示字段（默认groupField）
	                // groupDisplayField: 'title',
	                //自定义分组模版（用于显示分组标题）
	                groupHeaderTpl: [
	                    '${item.lastModiDate|fmtDate,"yyyy年MM月"}'
	                ],
	                //下拉刷新（默认false）
	                pullRefresh: true,
	                //向右箭头（默认false）
	                disclosure: true,
	                //数据展现模版
	                itemTpl: [
	                    '<div class="ui-avatar">',
	                        '<span style="background-image:url(http://placeholder.qiniudn.com/100x100)"></span>',
	                    '</div>',
	                    '<div class="ui-list-info">',
	                        '<h4 class="ui-nowrap">${item.name}</h4>',
	                        '<h4 class="ui-nowrap">${item.remark}</h4>',
	                        '<p style="text-align: right;">${item.lastModiDate|fmtDate,"yyyy年MM月dd日"}</p>',
	                    '</div>'
	                ]
	            });
        	}
            //加载数据
            listView.loadData();
        },
        /**
         * 选项卡切换后触发
         */
        onTabChange: function(tab, index){
            //Fw.Tips.info('切换到选项卡' + (index + 1) + '，' + tab.getTabBar(index).text());
        },
        /**
         * 点击按钮时触发（上一步、下一步）
         */
        onButtonClick: function(e){
            //判断是否切换选项卡
            switch(e.target.id){
                //切换到上一个选项卡
                case 'prev-btn':
                    tab.movePrev();
                    break;
                //切换到下一个选项卡
                case 'next-btn':
                    tab.moveNext();
                    break;
                default:
                    //隐藏分享菜单
                    actionsheet.removeClass('show');
            }
        },
        /**
         * 左边按钮点击事件
         */
        leftButtonHandler: function(){
            Fw.Tips.info('点击左边按钮...');
        },
        /**
         * 右边按钮点击事件
         */
        rightButtonHandler: function(){
            //显示分享菜单
            actionsheet.addClass('show');
        }
    };
}();

/**
 * 页面加载完毕后，初始化应用
 */
Fw.onReady(App);
