var App = function() {
	var tab;
	var ticketsListView;
	
	return {
        /**
         * 依赖组件
         */
        requires: [
			'Fw.Tab',
			'Fw.ListView',
			//日历组件
			'Fw.Calendar',
        ],
        /**
         * 视图设置
         * 设置标题栏、按钮等
         * 如果为空或未设置，则标题栏不显示、左右按钮不显示、等待层不自动显示
         */
        viewConfig: {
            /**
             * 标题栏标题
             * 如果为空或未设置，则标题栏不显示、左右按钮不显示
             */
            title: '我的观影券',
            /**
             * 左边按钮（默认显示）
             * 如果为false，则左边按钮不显示
             * leftButton: '左边按钮'
             */
            leftButton: {
                //自定义按钮标题（默认'返回'）
                text: '返回',
            },
            /**
             * 是否自动显示等待层（默认true）
             * 需手动隐藏等待层
             */
            loadMask: false
        },
        
        /**
         * 左边按钮点击事件
         */
        leftButtonHandler: function(){
        	//获取当前选项卡序号
        	var index = tab.getActiveItemIndex();
    		if(index == 1){
    			tab.setActiveItem(0);
    		}else{
    			//返回上级目录
    			Fw.redirect("./grabMovieTickets.html");
    		}
        },
        
        /**
		 * 初始化 应用入口
		 */
        init : function(require) {
        	//定义选项卡
            tab = new Fw.Tab({
                //选项卡EL
                contentEl: 'tab',
            });
            
            App.initListView();
        	//初始化事件
        	this.initEvents();
        	//初始化数据
        	this.initData();
        },
        
        /**
         * 初始化事件
         */
        initEvents:function(){
            //点击按钮时触发（保存、重置、重发验证码）
            $("#query-btn").on('click', Fw.bind(this.onClickQuery, this));
        },
        
        /**
         * 初始化数据
         */
        initData:function(){
        	//初始化日期
        	this.initCurDate();
        },
        
        /**
         * 获取当前日期
         */
        initCurDate:function(){
        	Fw.util.Ajax.get({
				service : 'curdateService',
				method:'getCurDate',
				// 请求成功后回调
				success : function(data, result) {
					$('[name=startDate]').val(Fw.util.Format.fmtSomeMonth(data['CurDate'],-3));
					$('[name=endDate]').val(data['CurDate']);
				},
			});
        },

        /**
		 * 初始化列表
		 */
        initListView: function(){
        	//定义列表
        	ticketsListView = new Fw.ListView({
                //EL
                contentEl : 'ticketsListView',//加载数据
			    cls: 'sx-list',
			    groupCls: 'sx-list-group',
			    groupHeaderBaseCls: 'sx-list-group-header',
			    groupHeaderCls: 'sui-border-t',
			    itemBaseCls: 'sx-list-item',
			    itemCls: 'sui-border-t',
			    pressedCls: 'sui-list-active',
                //数据请求对象（为空时，可通过loadData加载静态数据，但不支持beforequery、query事件）
                ajax : {
                    //数据源（服务接口）
                    service: 'grabTicketsService',
                    method: 'queryMCCardReceiptQry',
                    //请求参数
                    params: {
                    	
                    },
                    //是否分页（默认true）
					page: false,
					staticData: false
                },
                //分组字段（默认null）
				groupField : 'CardNo',
                groupHeaderTpl : [
                                  
          		],
                //向右箭头（默认false）
                disclosure: false,
                //数据展现模版
                itemTpl: [
                    '<div class="sui-tickets">',
                    	'<div class="sui-tickets-title">猫眼观影券</div>',
						'<div class="sui-tickets-desc">',
							'<label>券码：</label>',
							'<span>${item.CardNo}</span>',
							'<span name="copyTicketsNo" class="sui-tickets-copy">复制</span>',
						'</div>',
						'<div class="sui-tickets-desc">',
							'<label>领券日期：</label>',
							'<span>${item.TransDate}</span>',
						'</div>',
						'<div class="sui-tickets-desc">',
							'<label>付款日期：</label>',
							'<span>${item.PaymentDate}</span>',
						'</div>',
						/*'<div class="line-split">----------</div>',
						'<div>',
							'<p class="sui-tickets-amount">30</p><span class="sui-tickets-amountB">元</span>',
						'</div>',*/
					'</div>',
                ]
            });
        },
        
        /**
         * 列表加载完成触发
         */
        onQueryList: function(ajax, data, result) {
        	var endmsg=[];
        	endmsg[endmsg.length]='<div class="x-loadmore ui-loading-wrap">',
        	endmsg[endmsg.length]='<p class="x-loadmore-text">没有更多数据</p>',
        	endmsg[endmsg.length]='</div>'
        	$('#ticketsListView').append(endmsg.join(""));
        	
        	tab.setActiveItem(1);
        },
        
        /**
         * 查询观影券
         */
        onClickQuery: function(){
        	var startDate = $('[name=startDate]').val();
        	var endDate = $('[name=endDate]').val();
        	var start = Fw.util.Format.toDate(startDate);
            var end = Fw.util.Format.toDate(endDate);
            
            var beforeThreeMonth = new Date(end.setMonth(end.getMonth()-3));
            if(beforeThreeMonth > start){
            	Fw.Tips.warn("查询跨度不能超过3个月!");
            	return;
            }
            
            var startDateStr = Fw.util.Format.fmtJSNXStringDate(startDate);
        	var endDateStr = Fw.util.Format.fmtJSNXStringDate(endDate);
        	
        	$('#ticketsListView').empty();
        	ticketsListView.ajax.clearParams();//清除所有查询条件
        	ticketsListView.ajax.setParams("StartDatea", startDateStr);
        	ticketsListView.ajax.setParams("EndDatea", endDateStr);
        	//加载数据
        	ticketsListView.loadData();
        	//载入数据后触发
        	ticketsListView.ajax.on('query', this.onQueryList, this);
        	//选中记录时触发
        	ticketsListView.on('itemclick', this.onSelectTicket, this);
        },
        
        /**
         * 点击卡券
         */
        onSelectTicket: function(itemCls, index, tapItem, e, subclass){
        	if("sui-tickets-copy" == e.target.className){ //复制
        		Fw.device.Device.copyToClipboard(itemCls.CardNo);
        	}
        },

        
	};
	
}();


/**
 * 页面加载完毕后，初始化应用
 */
Fw.onReady(App); 