/**
 * 创建应用
 */
var App = function() {
    var searchbar, listView;
    //
    return {
        /**
         * 依赖组件
         */
        requires : [
            //查询工具条
            'Fw.SearchBar',
            //列表
            'Fw.ListView'
        ],
        /**
         * 视图设置
         * 设置标题栏、按钮等
         * 如果为空或未设置，则标题栏不显示、左右按钮不显示、等待层不自动显示
         */
        viewConfig: {
            /**
             * 标题栏标题
             * 如果为空或未设置，则标题栏不显示、左右按钮不显示
             */
            title: '开户网点',
            /**
             * 左边按钮（默认显示）
             * 如果为false，则左边按钮不显示
             * leftButton: '左边按钮'
             */
            leftButton: {
                //自定义按钮标题（默认'返回'）
                text: '左边按钮'
                //自定义按钮点击事件（默认App.leftButtonHandler）
                //handler: XXX
            },
        },
        /**
         * 初始化
         * 应用入口
         */
        init : function(require) {
            //定义查询工具条
            searchBar = new Fw.SearchBar({
                //EL
                contentEl : 'searchbar',
                //最少输入字符个数（默认4）
                //满足该参数时，触发查询事件
                minChars: 1,
                //最大输入长度
                maxLength:15,
                //输入框占位信息
                placeholder: '请输入您要搜索的开户行关键字'
            });
            var parameters = Fw.decode(Fw.getParameters());
            var cityid = parameters.cityid;
            var bankid = parameters.bankid;
            var bankname = parameters.bankname;	//银行名称
            //定义列表
            listView = new Fw.ListView({
                //EL
                contentEl : 'list',
                //数据请求对象（为空时，可通过loadData加载静态数据，但不支持beforequery、query事件）
                ajax : {
                	service: 'openBankService',
                    method: 'queryOpenBankList',//默认为query，方法名称必须是query开头
                    //请求参数
                     params: {
                    	 act: "selectbank",
                    	 cityid: cityid,
         	    		 bankid:bankid,
         	    		 bankname:bankname
                     },
                    //是否自动显示等待层（默认true）
                    //允许自定义等待层提示信息（loadMask: '请稍后...'）
                    //loadMask: true,
                    //是否分页（默认true）
                    //page: false,
                    //分页参数
                    pageSize: 10,
                    //是否使用数据挡板（默认false）
                    staticData: false
                },
                //向右箭头（默认false）
                disclosure: true,
                //数据展现模版
                itemTpl: [
                    '<div>',
                    '<p style="font-size: 14px;padding-top: 3px;">${item.LNAME}</p>',
                    '</div>'
                ]
            });
            
            //初始化事件
            this.initEvents();
            
            //加载数据
            listView.loadData();
        },
        /**
         * 初始化事件
         */
        initEvents: function(){
            //初始化查询工具条事件
            //触发查询事件前触发
            searchBar.on('beforesearch', this.onBeforeSearch);
            //触发查询事件（直接将查询事件桥接到listView）
            searchBar.on('search', listView.loadData, listView);

            //初始化列表事件
            //选中记录时触发
            listView.on('itemclick', this.onSelect, this);
            //渲染到页面前触发
            listView.on('beforeitemadd', this.onBeforeAddItem, this);
            //查询数据前触发
            listView.ajax.on('beforequery', this.onBeforeQuery, this);
            //查询数据后触发
            listView.ajax.on('query', this.onQuery, this);
        },
        /**
         * 选中记录时触发
         */
        onSelect : function(record, index, item, e, cmp) {
            var parameters = Fw.decode(Fw.getParameters());
            record.ProvinceName = parameters.ProvinceName;
            record.ProvinceCode = parameters.province;
            record.CityName = parameters.CityName;
            record.cityid = parameters.cityid;
            record.bankName = parameters.bankname;
            record.bankid = parameters.bankid;
            record.specifAccount=parameters.specifAccount;
            var deptName = record.LNAME;	//银行网点
            
        	
            if(Fw.isEqual(parameters.key,"1")){
            	Fw.redirect("querySign.html",Fw.encode(record));
            }else{
            	Fw.redirect("receiveMoneySign.html",Fw.encode(record));
            }
        },
        /**
         * 渲染到页面前触发
         */
        onBeforeAddItem: function(record, index, cmp){

        },
        /**
         * 触发查询事件前触发
         */
        onBeforeSearch : function(value) {
            //查询条件为空
            if(Fw.isEmpty(value)){
                listView.ajax.clearParams('bankname');//清除指定key的查询条件
            }
          //查询条件值类型校验
            else if(/[""#$%&\^*]/.test(value)){
                Fw.Tips.info('查询条件不允许含有%');
                return false; //则不触发查询事件
            }
            else{
                //设置查询条件
                listView.ajax.setParams('bankname', value);
            }
        },
        /**
         * 查询数据前触发
         * @param {Object} Ajax对象
         * @param {Object} 请求报文
         * @param {String} 请求路径
         */
        onBeforeQuery : function(ajax, message, url) {
            
        },
        /**
         * 查询数据后触发
         * @param {Object} Ajax对象
         * @param {Object} 响应数据（result[Fw.Ajax.rootProperty]部分）
         * @param {Object} 响应数据（全部）
         */
        onQuery : function(ajax, data, result) {
            
        }
    };
}();

/**
 * 页面加载完毕后，初始化应用
 */
Fw.onReady(App);
