/**
 * 状态同步
 */
var App = function() {
    return {
        /**
         * 依赖组件
         */
        requires: [
			//日历组件
			'Fw.Calendar',
			'Fw.TimerGlider',
            'Fw.Tab',
        	'Fw.app.Session',
        	'Fw.Form',
        	'../js/huaiAnSocialSecurityUtils.js'
        ],
        
        /**
         * 视图设置
         * 设置标题栏、按钮等
         * 如果为空或未设置，则标题栏不显示、左右按钮不显示、等待层不自动显示
         */
        viewConfig: {
            /**
             * 标题栏标题
             * 如果为空或未设置，则标题栏不显示、左右按钮不显示
             */
            title: '状态同步',
            /**
             * 左边按钮（默认显示）
             * 如果为false，则左边按钮不显示
             * leftButton: '左边按钮'
             */
            leftButton: {
                //自定义按钮标题（默认'返回'）
                text: '左边按钮',
            },
            /**
             * 是否自动显示等待层（默认true）
             * 需手动隐藏等待层
             */
            loadMask: true
        },
        
        /**
         * 左边按钮点击事件
         */
        leftButtonHandler: function(){
        	//获取当前选项卡序号
        	var index = this.tab.getActiveItemIndex();
        	if(index == 1) {
        		this.tab.setActiveItem(0);
        	} else {
        		//返回上级目录
        		Fw.redirect("../../menu.html", Fw.getParameters());
        	}
        },
        
        /**
         * 初始化
         * 应用入口
         */
        init : function(require) {
            //定义选项卡
            this.tab = new Fw.Tab({
                //选项卡EL
                contentEl: 'tab',
                //默认选项卡（默认0）
                activeTab: 0,
                //是否锁定tab标题
                fixed:true,
            });
            // 查询表单
            if(!this.formQry){
            	this.formQry = new Fw.Form({
                    contentEl : 'formQry',
                    resettable: false,
                    msgTarget: 'tips'
                });
            	this.formQry.on('beforesubmit', this.beforeQuery, this);
            	this.formQry.on('submit', this.doQuery, this);
            }
	        this.initData();
            //初始化事件
            this.initEvents();
            //手动隐藏等待层
            Fw.LoadMask.hide(500);
        },
        /**
         * 初始化数据
         */
        initData: function(){
        	// 初始化日期
        	this.initDateQry();
        	// 初始化付款账户
        	this.cards = Fw.app.Session.getAllCards();
        	$('[name="AcNo"]').empty();
        	var index = [];
        	for(var i=0;i<this.cards.length;i++){
        		if(this.cards[i].BankAcType != "PCRC"){
        			index.push(i+"");
        			$('[name="AcNo"]').append('<option value="'+i+'">'+Fw.util.Format.fmtAcctNo(this.cards[i]['No'],true)+'</option>');        	
        		}
        	}
        	var num = index.length==0 ? "0" : index[0];
        	this.formQry.setValues("AcNo",num);
			
        },
        // *************初始化日期***********************
        initDateQry : function(){
        	Fw.util.Ajax.get({
				service : 'curdateService/getCurDate',
				// 请求成功后回调
				success : function(data, result) {
					var CurDate = data['CurDate'];
					// 查询区间
		        	App.formQry.setValues('BeginDate',Fw.util.Format.fmtSomeMonth(CurDate,-3));
		        	App.formQry.setValues('EndDate',CurDate);
				},
			});
        },
        /**
         * 初始化事件
         */
        initEvents: function(){
        	// 状态同步查询触发按钮
            $('#query-btn').on('click',Fw.bind(this.SinglePayFlushQry,this));
        },
        // **************************状态同步查询******************************
        SinglePayFlushQry:function(){
        	this.formQry.submit();
        },
        beforeQuery:function(values){
        	// 日期控制 查询跨度不能超过3个月
        	var startDate = values.BeginDate;
        	var endDate = values.EndDate;
        	var start = Fw.util.Format.toDate(startDate);
            var end = Fw.util.Format.toDate(endDate);
            var beforeThreeMonth = new Date(end.setMonth(end.getMonth()-3));
            if(beforeThreeMonth > start){
            	Fw.Tips.warn("查询跨度不能超过3个月!");
            	return false;
            }
        	// 参数设置  （日期格式：yyyy-MM-dd）
        	//var dateFmt = 'yyyy-MM-dd';
        	this.paramsQry = values;
        	this.paramsQry.BeginDatea = values.BeginDate.replaceAll("/","");
        	this.paramsQry.EndDatea =values.EndDate.replaceAll("/","");
        	// 设置卡
        	var card = this.cards[$('[name="AcNo"]').val()];
        	this.paramsQry.AcNo = card.No;
        	this.paramsQry.IdType = $("select[name=idType]").val();  //证件类型
        	this.paramsQry.Jyzt =$("select[name=Jyzt]").val();  //交易状态  S成功，F失败，U超时，""全部
        	this.paramsQry.Nt1 = ""; 
        	this.paramsQry.Nt2 = ""; 
        	this.paramsQry.Nt3 = ""; 
        	this.paramsQry.Nt4 = ""; 
        	this.paramsQry.Nt5 = ""; 
        	// 填充查询列表头部信息
        	$('#AcNo').text(this.paramsQry.AcNo);
        	$('#startDate').text(values.BeginDate);
        	$('#endDate').text(values.EndDate);
        },
        doQuery:function(values){
        	var timerGlider = new Fw.TimerGlider({
	            contentEl: 'timeline',
	            ajax: {
	            	service: 'HuaiAnSocialSecurityService',
	            	method: 'queryMCSocialSecurityXYStateQry',
	            	params : App.paramsQry,
	            	staticData: false
	            },
	            groupField: 'GDATE',
	            groupHeaderTpl: [
	                    '${item.GDATE}月'
                ],
                 //下拉刷新（默认false）
                pullRefresh: false,
	            itemTpl:[
	                    '<div class="sx-timeline-item">',
		                     '<div class="sx-content-a">',
		                     '</div>',
		                     '<div class="sx-content-b">',
			                     '<div class="sx-top-8" >交易日期</div>',
			                     '<div>交易流水</div>',
			                     '<div>缴费险种</div>',
			                     '<div>缴费金额</div>',
			                     '<div>缴费状态</div>',
		                     '</div>',
		                     '<div class="sx-content-d">',
							 '</div>',
							 '<div class="sx-content-c">',
			                     '<div class="sx-top-8">${item.OriWorkDate}</div>',
			                     '<div>${item.OriAgentSerialNo}</div>',
			                     '<div>${item.InsType|formatInsureType}</div>',
			                     '<div>${item.Amount|fmtAmt}</div>',
			                     '<div>${item.TradeStatus1|formatTradeStatus}</div>',
			                     
		                     '</div>',
	                     '</div>',
				]
	        });
	        //每行记录点击事件
	        //timerGlider.on('itemclick',this.onItem);
	        //加载数据
	        timerGlider.loadData();
	        // 跳转
	        this.tab.setActiveItem(1);
        },
    };
}();

/**
 * 页面加载完毕后，初始化应用
 */
Fw.onReady(App);