/**
 * 状态同步
 */
var App = function() {
    return {
        /**
         * 依赖组件
         */
        requires: [
			//日历组件
			'Fw.Calendar',
			'Fw.TimerGlider',
            'Fw.Tab',
        	'Fw.app.Session',
        	'Fw.Form'
        ],
        
        /**
         * 视图设置
         * 设置标题栏、按钮等
         * 如果为空或未设置，则标题栏不显示、左右按钮不显示、等待层不自动显示
         */
        viewConfig: {
            /**
             * 标题栏标题
             * 如果为空或未设置，则标题栏不显示、左右按钮不显示
             */
            title: '职工社保缴费状态查询',
            /**
             * 左边按钮（默认显示）
             * 如果为false，则左边按钮不显示
             * leftButton: '左边按钮'
             */
            leftButton: {
                //自定义按钮标题（默认'返回'）
                text: '左边按钮',
            },
            /**
             * 是否自动显示等待层（默认true）
             * 需手动隐藏等待层
             */
            loadMask: true
        },
        
        /**
         * 左边按钮点击事件
         */
        leftButtonHandler: function(){
        	//获取当前选项卡序号
        	var index = this.tab.getActiveItemIndex();
        	if(index == 1) {
        		this.tab.setActiveItem(0);
        	} else {
        		//返回上级目录
        		Fw.redirect("../../menu.html", Fw.getParameters());
        	}
        },
        
        /**
         * 初始化
         * 应用入口
         */
        init : function(require) {
            //定义选项卡
            this.tab = new Fw.Tab({
                //选项卡EL
                contentEl: 'tab',
                //默认选项卡（默认0）
                activeTab: 0,
                //是否锁定tab标题
                fixed:true,
            });
            // 查询表单
            if(!this.formQry){
            	this.formQry = new Fw.Form({
                    contentEl : 'formQry',
                    resettable: false,
                    msgTarget: 'tips'
                });
            	this.formQry.on('beforesubmit', this.beforeQuery, this);
            	this.formQry.on('submit', this.doQuery, this);
            }
	        this.initData();
            //初始化事件
            this.initEvents();
            //手动隐藏等待层
            Fw.LoadMask.hide(500);
        },
        /**
         * 初始化数据
         */
        initData: function(){
        	// 初始化日期
        	this.initDateQry();
        	// 初始化付款账户
        	this.cards = Fw.app.Session.getAllCards();
        	$('[name="AcNo"]').empty();
        	var index = [];
        	for(var i=0;i<this.cards.length;i++){
        		if(this.cards[i].BankAcType != "PCRC"){
        			index.push(i);
        			$('[name="AcNo"]').append('<option value="'+i+'">'+Fw.util.Format.fmtAcctNo(this.cards[i]['No'],true)+'</option>');        	
        		}
        	}
        	var num = index.length==0 ? "0" : index[0];
        	this.formQry.setValues("AcNo",num);
        	$('[name="AcNo"]').val(num);
        },
        // *************初始化日期***********************
        initDateQry : function(){
        	Fw.util.Ajax.get({
				service : 'curdateService/getCurDate',
				// 请求成功后回调
				success : function(data, result) {
					var CurDate = data['CurDate'];
					// 查询区间
		        	App.formQry.setValues('BeginDate',Fw.util.Format.fmtSomeMonth(CurDate,-3));
		        	App.formQry.setValues('EndDate',CurDate);
				},
			});
        },
        /**
         * 初始化事件
         */
        initEvents: function(){
        	// 状态同步查询触发按钮
            $('#query-btn').on('click',Fw.bind(this.SinglePayFlushQry,this));
        },
        // **************************状态同步查询******************************
        SinglePayFlushQry:function(){
        	this.formQry.submit();
        },
        beforeQuery:function(values){
        	// 日期控制 查询跨度不能超过3个月
        	var startDate = values.BeginDate;
        	var endDate = values.EndDate;
        	var start = Fw.util.Format.toDate(startDate);
            var end = Fw.util.Format.toDate(endDate);
            var beforeThreeMonth = new Date(end.setMonth(end.getMonth()-3));
            if(beforeThreeMonth > start){
            	Fw.Tips.warn("查询跨度不能超过3个月!");
            	return false;
            }
        	// 参数设置  （日期格式：yyyy-MM-dd）
        	var dateFmt = 'yyyy-MM-dd';
        	var BeginDate = Fw.util.Format.fmtDate(values.BeginDate,dateFmt);
        	var EndDate = Fw.util.Format.fmtDate(values.EndDate,dateFmt);
        	// 设置卡
        	var card = this.cards[$('[name="AcNo"]').val()];
        	this.paramsQry = {
    			AcNo: card.No,
    			BeginDatea: BeginDate,
    			EndDatea: EndDate
        	};
        	// 填充查询列表头部信息
        	$('#AcNo').text(Fw.util.Format.fmtAcctNo(card.No,true));
        	$('#startDate').text(values.BeginDate);
        	$('#endDate').text(values.EndDate);
        },
        doQuery:function(values){
        	var timerGlider = new Fw.TimerGlider({
	            contentEl: 'timeline',
	            ajax: {
	            	service: 'LianYunGangSocialSecurityService',
	            	method: 'querySocialSecurityGYSys',
	            	params : App.paramsQry,
	            	staticData: false
	            },
	            groupField: 'GDATE',
	            groupHeaderTpl: [
	                    '${item.GDATE}月'
                ],
                 //下拉刷新（默认false）
                pullRefresh: false,
	            itemTpl:[
	                    '<div class="sx-timeline-item">',
		                     '<div class="sx-content-a">',
		                     '</div>',
		                     '<div class="sx-content-b">',
			                     '<div class="sx-top-8" >交易日期</div>',
			                     '<div>交易流水</div>',
			                     '<div>缴费用户</div>',
			                     '<div>缴费金额</div>',
		                     '</div>',
		                     '<div class="sx-content-d">',
							 '</div>',
							 '<div class="sx-content-c">',
			                     '<div class="sx-top-8">${item.OworkDate}</div>',
			                     '<div>${item.OrequestSeq}</div>',
			                     '<div>${item.Userid|fmtSubStr, 0, 8}</div>',
			                     '<div>${item.Amount|fmtAmt}</div>',
		                     '</div>',
	                     '</div>',
				]
	        });
	        //每行记录点击事件
	        timerGlider.on('itemclick',this.onItem);
	        //加载数据
	        timerGlider.loadData();
	        // 跳转
	        this.tab.setActiveItem(1);
	        Fw.LoadMask.hide();
        },
        // ********************当前单笔缴费状态（点击触发）******************************
        onItem: function(record, index, item, e, cmp){
        	// 状态同步 参数设置
        	var param = {
        			OworkDate:record.OworkDate,
        			OrequestSeq:record.OrequestSeq,
        			OagentserialNo:record.OagentserialNo,
        			
        			Userid: record.Userid,
        			Amount: record.Amount
        	};
        	// 跳转
        	App.toShowSysInfo(param);
        },
        // *******************状态同步（查询真实数据）********************************
        toShowSysInfo: function(param){
        	Fw.util.Ajax.execute({
                service:'LianYunGangSocialSecurityService',
                method: 'executeGYMedicalInsuranceSync',
                params: {
                	OriWorkDate:param.OworkDate,
        			OrequestSeq:param.OrequestSeq,
        			OriAgentSerialNo:param.OagentserialNo
                },
                //是否自动显示等待层（默认true）
                //允许自定义等待层提示信息（loadMask: '请稍后...'）
                //loadMask: true,
                //是否使用数据挡板（默认false）
                staticData: false,
                //请求成功后回调
                success: function(data, result) {
                	// 填充结果数据
                	$("#result").html("交易成功");
                	$("#AcNo2").html(Fw.util.Format.fmtAcctNo($('#AcNo').html(),true));
                	$("#PeopId").html(param.Userid);
                	$("#Amount").html(Fw.util.Format.fmtAmt(param.Amount));
                	$("#OworkDate").html(Fw.util.Format.fmtDate(data.OworkDate));
                	$("#OagentSerialNo").html(data.OagentSerialNo);
                	// 跳转
                	App.tab.setActiveItem(2);
                },
                //请求错误后回调
                failure: function(status, textStatus, error) {
                }
        	});        	
        }
    };
}();

/**
 * 页面加载完毕后，初始化应用
 */
Fw.onReady(App);